@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseUtils.cpp,v 2.14 2010/09/21 11:43:21 taurel Exp $";
+static const char *RcsId = "$Header$";
 //+=============================================================================
 //
 // file :         DataBaseUtils.cpp
@@ -9,7 +9,7 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 //
 // author(s) :    A.Gotz, P.Verdier, JL Pons
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -29,6 +29,12 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 // You should have received a copy of the GNU General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
+// $Revision: 20316 $
+// $Date: 2012-05-23 14:56:16 +0200 (Wed, 23 May 2012) $
+//
+// $HeadURL:$
+//
+//=============================================================================
 
 #if HAVE_CONFIG_H
 #include <ac_config.h>
@@ -38,8 +44,18 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 #include <DataBase.h>
 #include <DataBaseClass.h>
 
+#include <errmsg.h>
+
 #include <stdio.h>
 
+#ifdef _TG_WINDOWS_
+#include <ws2tcpip.h>
+#else
+#include <arpa/inet.h>
+#include <sys/socket.h>
+#include <netdb.h>
+#endif
+
 using namespace std;
 
 namespace DataBase_ns {
@@ -50,7 +66,7 @@ namespace DataBase_ns {
 // 
 // description : 	utility method to replace all occurrences of
 //			wildcards (*) with SQL wildcards % and to escape
-//			all occurrences	of % and _ with \
+//			all occurrences	of '%' and '_' with '\'
 //
 // in :			string **wildcard_c_str - wildcard C string
 //
@@ -117,7 +133,7 @@ string DataBase::replace_wildcard(const char *wildcard_c_str)
 // method : 		DataBase::escape_string(char *string_c_str)
 // 
 // description : 	utility method to escape all occurrences	
-//			of ' and " with \
+//			of ' and " with '\'
 //
 // in :			const char *string_c_str -  C string to be modified.
 //
@@ -644,11 +660,7 @@ void DataBase::purge_property(const char *table,const char *field,const char *ob
   TangoSys_MemStream sql_query;
   MYSQL_RES *result;
   MYSQL_ROW row2;
-  int j;
   
-  TimeVal before, after;
-  GetTime(before);
-	
   sql_query.str("");
   sql_query << "SELECT DISTINCT id FROM " << table
             << " WHERE " << field << "=\"" << object << "\" AND name=\"" << name 
@@ -657,16 +669,10 @@ void DataBase::purge_property(const char *table,const char *field,const char *ob
   result = query(sql_query.str(),"purge_property()",con_nb);
   int nb_item = mysql_num_rows(result);
 
-  GetTime(after);
-  double time_elapsed = Elapsed(before, after);
-  //cout << "purge_property(select DISTINCT) : " << time_elapsed << endl;
-
-  GetTime(before);
-
   if(nb_item>historyDepth) {
     // Purge 
     int toDelete = nb_item-historyDepth;
-    for(j=0;j<toDelete;j++) {
+    for(int j=0;j<toDelete;j++) {
         row2 = mysql_fetch_row(result);
         sql_query.str("");
         sql_query << "DELETE FROM " << table << " WHERE id='" << row2[0] << "'";
@@ -675,11 +681,6 @@ void DataBase::purge_property(const char *table,const char *field,const char *ob
   }
   
   mysql_free_result(result);
-  
-  GetTime(after);
-  time_elapsed = Elapsed(before, after);
-  //cout << "purge_property(DELETE) : " << time_elapsed << endl;
-
 }
 
 //+------------------------------------------------------------------
@@ -695,12 +696,8 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
   TangoSys_MemStream sql_query;
   MYSQL_RES *result;
   MYSQL_ROW row2;
-  int j;
   
   //cout << "purge_att_property(" << object << "," << attribute << "," << name << ")" << endl;
-
-  TimeVal before, after;
-  GetTime(before);
 		
   sql_query.str("");
   sql_query << "SELECT DISTINCT id FROM " << table
@@ -709,17 +706,11 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
 
   result = query(sql_query.str(),"purge_att_property()",con_nb);
   int nb_item = mysql_num_rows(result);
-
-  GetTime(after);
-  double time_elapsed = Elapsed(before, after);
-  //cout << "  select DISTINCT : " << time_elapsed << endl;
-
-  GetTime(before);
   
   if(nb_item>historyDepth) {
     // Purge 
     int toDelete = nb_item-historyDepth;
-    for(j=0;j<toDelete;j++) {
+    for(int j=0;j<toDelete;j++) {
 		row2 = mysql_fetch_row(result);
         sql_query.str("");
         sql_query << "DELETE FROM " << table << " WHERE id='" << row2[0] << "'";
@@ -727,11 +718,44 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
     }
   }
   mysql_free_result(result);
+}
 
-  GetTime(after);
-  time_elapsed = Elapsed(before, after);
-  //cout << "  DELETE : " << time_elapsed << endl;
-      
+//+------------------------------------------------------------------
+/**
+ *	method:	base_connect()
+ *
+ *	description:	Basic action to build a Mysql connection
+ *
+ */
+//+------------------------------------------------------------------
+
+void DataBase::base_connect(int loop)
+{
+
+//
+// Initialise mysql database structure and connect to TANGO database
+//
+
+	conn_pool[loop].db = mysql_init(conn_pool[loop].db);
+	mysql_options(conn_pool[loop].db,MYSQL_READ_DEFAULT_GROUP,"client");
+
+#if   (MYSQL_VERSION_ID > 50000)
+
+	//C. Scafuri: auto reconnection has been off since 5.0.3. From 5.0.13 it is possible to set it as an option
+	// with reconnection enabled DataBase keeps working after timeouts and mysql shutdown/restart
+	if(mysql_get_client_version() >= 50013)
+	{
+		my_bool my_auto_reconnect=1;
+		if (mysql_options(conn_pool[loop].db,MYSQL_OPT_RECONNECT,&my_auto_reconnect) !=0)
+		{
+			ERROR_STREAM << "DataBase: error setting mysql auto reconnection: " << mysql_error(conn_pool[loop].db) << endl;
+		}
+		else
+		{
+			WARN_STREAM << "DataBase: set mysql auto reconnect to true" << endl;
+		}
+	}
+#endif
 }
 
 //+------------------------------------------------------------------
@@ -743,7 +767,7 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
  */
 //+------------------------------------------------------------------
 
-void DataBase::create_connection_pool(const char *mysql_user,const char *mysql_password)
+void DataBase::create_connection_pool(const char *mysql_user,const char *mysql_password,const char *mysql_host)
 {
 #ifndef HAVE_CONFIG_H
 	char *database = (char *)"tango";
@@ -756,39 +780,100 @@ void DataBase::create_connection_pool(const char *mysql_user,const char *mysql_p
 		WARN_STREAM << "DataBase::create_connection_pool(): mysql database user =  " << mysql_user 
 	           	 << " , password = " << mysql_password << endl;
 	}
-					
+
+	const char *host;
+	string my_host;
+	string ho,port;
+	unsigned int port_num = 0;
+
+	if (mysql_host != NULL)
+	{
+		my_host = mysql_host;
+		WARN_STREAM << "DataBase::create_connection_pool(): mysql host = " << mysql_host << endl;
+		string::size_type pos = my_host.find(':');
+		if (pos != string::npos)
+		{
+			ho = my_host.substr(0,pos);
+			pos++;
+			port = my_host.substr(pos);
+			stringstream ss(port);
+			ss >> port_num;
+			if (!ss)
+				port_num = 0;
+			host = ho.c_str();
+		}
+		else
+			host = my_host.c_str();
+		WARN_STREAM << "DataBase::create_connection_pool(): mysql host = " << host << ", port = " << port_num << endl;
+	}
+	else
+		host = NULL;
+
 	for (int loop = 0;loop < conn_pool_size;loop++)
 	{
 	
-// Initialise mysql database structure and connect to TANGO database
+		base_connect(loop);
 
-		conn_pool[loop].db = mysql_init(conn_pool[loop].db);
-		mysql_options(conn_pool[loop].db,MYSQL_READ_DEFAULT_GROUP,"client");
+//
+// Inmplement a retry. On some OS (Ubuntu 10.10), it may happens that MySQl needs some time to start.
+// This retry should cover this case
+// We also have to support case when this server is started while mysql is not ready yet
+// (this has been experienced on Ubuntu after a reboot when the ureadahead cache being invalidated
+// by a package installing file in /etc/init.d
+// Bloody problem!!!
+//
 
 
-#if   (MYSQL_VERSION_ID > 50000)
-
-	//C. Scafuri: auto reconnection has been off since 5.0.3. From 5.0.13 it is possible to set it as an option
-	// with reconnection enabled DataBase keeps working after timeouts and mysql shutdown/restart
-		if(mysql_get_client_version() >= 50013)
+		WARN_STREAM << "Going to connect to MySQL for conn. " << loop << endl;
+		if (!mysql_real_connect(conn_pool[loop].db, host, mysql_user, mysql_password, database, port_num, NULL, CLIENT_MULTI_STATEMENTS))
 		{
-			my_bool my_auto_reconnect=1;
-			if (mysql_options(conn_pool[loop].db,MYSQL_OPT_RECONNECT,&my_auto_reconnect) !=0)
-				ERROR_STREAM << "DataBase: error setting mysql auto reconnection: " << mysql_error(conn_pool[loop].db) << endl;
+			if (loop == 0)
+			{
+				int retry = 5;
+				while (retry > 0)
+				{
+					sleep(1);
+					int db_err = mysql_errno(conn_pool[loop].db);
+					WARN_STREAM << "Connection to MySQL failed with error " << db_err << endl;
+					if (db_err == CR_CONNECTION_ERROR || db_err == CR_CONN_HOST_ERROR)
+					{
+						mysql_close(conn_pool[loop].db);
+						conn_pool[loop].db = NULL;
+						base_connect(loop);
+					}
+					WARN_STREAM << "Going to retry to connect to MySQL for connection " << loop << endl;
+					if (!mysql_real_connect(conn_pool[loop].db, host, mysql_user, mysql_password, database, port_num, NULL, CLIENT_MULTI_STATEMENTS))
+					{
+						WARN_STREAM << "Connection to MySQL (re-try) failed with error " << mysql_errno(conn_pool[loop].db) << endl;
+						retry--;
+						if (retry == 0)
+						{
+							WARN_STREAM << "Throw exception because no MySQL connection possible after 5 re-tries" << endl;
+							TangoSys_MemStream out_stream;
+							out_stream << "Failed to connect to TANGO database (error = " << mysql_error(conn_pool[loop].db) << ")" << ends;
+				
+							Tango::Except::throw_exception((const char *)"CANNOT_CONNECT_MYSQL",
+												out_stream.str(),
+												(const char *)"DataBase::init_device()");
+						}
+					}
+					else
+					{
+						WARN_STREAM << "MySQL connection succeed after retry" << endl;
+						retry = 0;
+					}
+				}
+			}
 			else
-				WARN_STREAM << "DataBase: set mysql auto reconnect to true" << endl;
-		}
-#endif
-
-
-		if (!mysql_real_connect(conn_pool[loop].db, NULL, mysql_user, mysql_password, database, 0, NULL, CLIENT_MULTI_STATEMENTS))
-		{
-			TangoSys_MemStream out_stream;
-			out_stream << "Failed to connect to TANGO database (error = " << mysql_error(conn_pool[loop].db) << ")" << ends;
-			
-			Tango::Except::throw_exception((const char *)"CANNOT_CONNECT_MYSQL",
-											out_stream.str(),
-											(const char *)"DataBase::init_device()");
+			{
+				WARN_STREAM << "Failed to connect to MySQL for conn. " << loop << ". No re-try in this case" << endl;
+				TangoSys_MemStream out_stream;
+				out_stream << "Failed to connect to TANGO database (error = " << mysql_error(conn_pool[loop].db) << ")" << ends;
+				
+				Tango::Except::throw_exception((const char *)"CANNOT_CONNECT_MYSQL",
+												out_stream.str(),
+												(const char *)"DataBase::init_device()");
+			}
 		}
 	}
 
@@ -797,6 +882,172 @@ void DataBase::create_connection_pool(const char *mysql_user,const char *mysql_p
 	
 }
 
+//+------------------------------------------------------------------
+/**
+ *	method:	host_port_from_ior()
+ *
+ *	description:	Get host and port from a device IOR
+ *
+ */
+//+------------------------------------------------------------------
+
+bool DataBase::host_port_from_ior(const char *iorstr,string &h_p)
+{
+	size_t s = (iorstr ? strlen(iorstr) : 0);
+
+  	if (s < 4)
+    	return false;
+
+  	const char *p = iorstr;
+  	if (p[0] != 'I' || p[1] != 'O' || p[2] != 'R' || p[3] != ':')
+    	return false;
+
+  	s = (s - 4) / 2;  // how many octets are there in the string
+  	p += 4;
+
+  	cdrMemoryStream buf((CORBA::ULong)s,0);
+
+  	for (int i=0; i<(int)s; i++)
+	{
+    	int j = i*2;
+    	CORBA::Octet v;
+    
+    	if (p[j] >= '0' && p[j] <= '9')
+		{
+      		v = ((p[j] - '0') << 4);
+    	}
+    	else if (p[j] >= 'a' && p[j] <= 'f')
+		{
+      		v = ((p[j] - 'a' + 10) << 4);
+    	}
+    	else if (p[j] >= 'A' && p[j] <= 'F')
+		{
+      		v = ((p[j] - 'A' + 10) << 4);
+    	}
+    	else
+      		return false;
+
+    	if (p[j+1] >= '0' && p[j+1] <= '9')
+		{
+      		v += (p[j+1] - '0');
+    	}
+    	else if (p[j+1] >= 'a' && p[j+1] <= 'f')
+		{
+      		v += (p[j+1] - 'a' + 10);
+    	}
+    	else if (p[j+1] >= 'A' && p[j+1] <= 'F')
+		{
+      		v += (p[j+1] - 'A' + 10);
+    	}
+    	else
+      		return false;
+
+    	buf.marshalOctet(v);
+  	}
+
+  	buf.rewindInputPtr();
+  	CORBA::Boolean b = buf.unmarshalBoolean();
+  	buf.setByteSwapFlag(b);
+
+  	IOP::IOR ior;
+
+  	ior.type_id = IOP::IOR::unmarshaltype_id(buf);
+  	ior.profiles <<= buf;
+
+
+    if (ior.profiles.length() == 0 && strlen(ior.type_id) == 0)
+	{
+      	return false;
+    }
+    else
+	{
+      	for (unsigned long count=0; count < ior.profiles.length(); count++)
+		{
+			if (ior.profiles[count].tag == IOP::TAG_INTERNET_IOP)
+			{
+	  			IIOP::ProfileBody pBody;
+	  			IIOP::unmarshalProfile(ior.profiles[count],pBody);
+
+//
+// Three possible cases for host name:
+// 1 - The host is stored in IOR as IP numbers
+// 2 - The host name is stored in IOR as the canonical host name
+// 3 - The FQDN is stored in IOR
+// We allways try to get the host name as the FQDN
+//
+
+				ho = pBody.address.host.in();
+				bool host_is_name = false;
+
+				string::size_type pos = ho.find('.');
+				if (pos == string::npos)
+					host_is_name = true;
+				else
+				{
+					for (unsigned int loop =0;loop < pos;++loop)
+					{
+						if (isdigit((int)ho[loop]) == 0)
+						{
+							host_is_name = true;
+							break;
+						}
+					}
+				}
+
+				if (host_is_name == false)
+				{
+					struct sockaddr_in s;
+					char service[20];
+
+					s.sin_family = AF_INET;
+					int res;
+#ifdef _TG_WINDOWS_
+					s.sin_addr.s_addr = inet_addr(ho.c_str());
+					if (s.sin_addr.s_addr != INADDR_NONE)
+#else
+					res = inet_pton(AF_INET,ho.c_str(),&(s.sin_addr.s_addr));
+					if (res == 1)
+#endif
+					{
+						res = getnameinfo((const struct sockaddr *)&s,sizeof(s),ho_name,sizeof(ho_name),service,sizeof(service),0);
+						if (res == 0)
+						{
+							h_p = ho_name;
+							h_p = h_p + ':';
+						}
+						else
+							h_p = ho + ':';
+					}
+					else
+						h_p = ho + ':';
+				}
+				else
+				{
+					if (pos == string::npos)
+					{
+						Tango::DeviceProxy::get_fqdn(ho);
+					}
+
+					h_p = ho + ':';
+				}
+
+//
+// Add port number
+//
+
+				stringstream ss;
+				ss << pBody.address.port;
+				h_p = h_p + ss.str();
+
+				break;
+			}
+		}
+	}
+
+	return true;
+}
+
+
 //+------------------------------------------------------------------
 /**
  *	method:	AutoLock class ctor and dtor
