@@ -9,7 +9,7 @@ static const char *RcsId = "$Header$";
 //
 // $Author: pascal_verdier $
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -29,56 +29,8 @@ static const char *RcsId = "$Header$";
 // You should have received a copy of the GNU General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 15177 $
-//
-// $Log$
-// Revision 3.15  2010/10/15 06:20:32  pascal_verdier
-// Copyright added.
-//
-// Revision 3.14  2010/09/21 12:18:57  pascal_verdier
-// GPL Licence added to header.
-//
-// Revision 3.13  2010/08/09 06:52:25  pascal_verdier
-// Patch for Python module added (thanks to Tiago).
-//
-// Revision 3.12  2010/02/09 15:09:49  pascal_verdier
-// Define  _TG_WINDOWS_  replace WIN32.
-// LogFileHome property added.
-//
-// Revision 3.11  2008/12/12 13:30:44  pascal_verdier
-// Problem on python server options fixed.
-//
-// Revision 3.10  2008/06/18 08:17:03  pascal_verdier
-// Pb with case unsensitive on win32 fixed.
-//
-// Revision 3.9  2008/06/06 07:56:50  pascal_verdier
-// Case unsensitive on instance name added.
-//
-// Revision 3.8  2008/06/04 09:08:03  pascal_verdier
-// javaw process control added.
-// Java -cp classpath parsing mmodified.
-//
-// Revision 3.7  2008/05/15 08:07:18  pascal_verdier
-// TangoSys_MemStream replaced by TangoSys_OMemStream
-// (for leaking problem under win32)
-//
-// Revision 3.6  2008/04/28 12:36:09  pascal_verdier
-// Eception in solaris modified.
-//
-// Revision 3.5  2008/04/24 06:33:52  pascal_verdier
-// Bug in solaris management fixed.
-//
-// Revision 3.4  2008/04/10 12:15:05  jensmeyer
-// Added compile options for MacOSX and FreeBSD
-//
-// Revision 3.3  2008/04/09 14:39:57  pascal_verdier
-// Better trace on pread failed
-//
-// Revision 3.2  2008/03/03 13:26:15  pascal_verdier
-// is_process_running() method added.
-//
-// Revision 3.1  2008/02/29 15:15:05  pascal_verdier
-// Checking running processes by system call added.
+// $Revision: 20365 $
+// $Date: 2012-05-25 09:58:15 +0200 (Fri, 25 May 2012) $
 //
 //-=============================================================================
 
@@ -105,6 +57,9 @@ static const char *RcsId = "$Header$";
 #endif	/*	_TG_WINDOWS_		*/
 #endif	/*	TIME_VAR	*/
 
+#ifdef _TG_WINDOWS_
+typedef LONG (WINAPI *TNtQueryInformationProcess)(HANDLE,UINT,PVOID,ULONG,PULONG);
+#endif
 
 namespace Starter_ns
 {
@@ -191,7 +146,7 @@ string ProcessData::wchar2string(WCHAR *wch, int size)
 		ch[i] = wch[i];
 	ch[i] = 0x0;
 	string	str(ch);
-	delete ch;
+	delete [] ch;
 	return str;
 }
 //=============================================================
@@ -206,7 +161,7 @@ WCHAR *ProcessData::string2wchar(string str)
 	for (i=0 ; ch[i]!=0 ; i++)
 		wch[i] = (short)ch[i];
 	wch[i] = 0x0;
-	delete ch;
+	delete [] ch;
 	return wch;
 }
 //=============================================================
@@ -217,7 +172,7 @@ void ProcessData::read_process_list_from_sys()
 	for (int i=0 ; i<proc_list.size() ; i++)
 		delete proc_list[i];
 	proc_list.clear();
-
+#ifdef OLD
 	// Take a snapshot of all processes in the system.
 	HANDLE	hProcessSnap = 
 		CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS | TH32CS_SNAPMODULE, 0);
@@ -248,10 +203,9 @@ void ProcessData::read_process_list_from_sys()
 
 	//	Get module ntdll
     NTQIP						*lpfnNtQueryInformationProcess;
-	PROCESS_BASIC_INFORMATION	pbi;
 	WCHAR	*wc = string2wchar("ntdll.dll");
 	HINSTANCE	hLibrary = GetModuleHandle(wc);
-	delete wc;
+	delete [] wc;
     if (hLibrary != NULL)
     {
         lpfnNtQueryInformationProcess = (NTQIP *)GetProcAddress(hLibrary, "ZwQueryInformationProcess");
@@ -265,57 +219,62 @@ void ProcessData::read_process_list_from_sys()
 						(const char *)"Starter::get_process_list()");
     }
 
-	__INFOBLOCK	block;
-    __PEB 		PEB;
-	char		*c_cmdline = NULL;
-    DWORD		dwSize=0;
-
-	// Now walk the snapshot of processes, and
-	pbi.PebBaseAddress = (PPEB)0x7ffdf000;
 	do
 	{
 		// Retrieve the priority class.
-		DWORD	dwPriorityClass = 0;
 		HANDLE	hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, pe32.th32ProcessID );
 		if( hProcess != NULL )
 		{
-	        if (lpfnNtQueryInformationProcess != NULL)
-    	        (*lpfnNtQueryInformationProcess)(hProcess, ProcessBasicInformation, &pbi, sizeof(pbi), &dwSize);
+			PROCESS_BASIC_INFORMATION	pbi;
+			DWORD		dwSize=0;
+			if (lpfnNtQueryInformationProcess != NULL)
+				(*lpfnNtQueryInformationProcess)(hProcess, ProcessBasicInformation, &pbi, sizeof(pbi), &dwSize);
+			cout << pbi.PebBaseAddress <<  "  -  " << dwSize << endl;
 
 			string cmdline("");
-			dwPriorityClass = GetPriorityClass( hProcess );
-			if( !dwPriorityClass )
-				errorCodeToString(GetLastError(), "GetPriorityClass" );
-
-			if (ReadProcessMemory(hProcess, 
-					pbi.PebBaseAddress,
-					&PEB,
-					sizeof(PEB),
-					&dwSize))
+			SIZE_T		sizeT=0;
+			__PEB 		PEB;
+			DWORD dwPriorityClass = GetPriorityClass( hProcess );
+			if( dwPriorityClass )
 			{
 				if (ReadProcessMemory(hProcess, 
-						(LPVOID)PEB.dwInfoBlockAddress,
-						&block,
-						sizeof(block),
-						&dwSize))
+						pbi.PebBaseAddress,
+						&PEB,
+						sizeof(PEB),
+						&sizeT))
 				{
-					WCHAR	*buff = new WCHAR[block.wMaxLength+1];
-    	        	if (ReadProcessMemory(hProcess, 
-									(LPVOID)block.dwCmdLineAddress, 
-									buff, 
-									block.wMaxLength, 
-									&dwSize))
-						cmdline = wchar2string(buff, dwSize);
+					cout << hex << PEB.dwInfoBlockAddress << "  -  " << sizeT << endl;
+					sizeT = 0;
+					__INFOBLOCK	block;
+					if (ReadProcessMemory(hProcess, 
+							(LPVOID)PEB.dwInfoBlockAddress,
+							&block,
+							sizeof(block),
+							&sizeT))
+					{
+						sizeT = 0;
+						WCHAR	*buff = new WCHAR[block.wMaxLength+1];
+    	        		if (ReadProcessMemory(hProcess, 
+										(LPVOID)block.dwCmdLineAddress, 
+										buff, 
+										block.wMaxLength, 
+										&sizeT)) {
+							cmdline = wchar2string(buff, sizeT);
+							cout << cmdline << endl;
+						}
+						else
+							errorCodeToString(GetLastError(), "3-ReadProcessMemory()" );
+						delete [] buff;
+					}
 					else
-						errorCodeToString(GetLastError(), "3-ReadProcessMemory()" );
-					delete buff;
+						errorCodeToString(GetLastError(), "2-ReadProcessMemory()" );
 				}
 				else
-					errorCodeToString(GetLastError(), "2-ReadProcessMemory()" );
+					errorCodeToString(GetLastError(), "1-ReadProcessMemory()" );
 			}
 			else
-				errorCodeToString(GetLastError(), "1-ReadProcessMemory()" );
-			
+				errorCodeToString(GetLastError(), "GetPriorityClass" );
+	
 			CloseHandle( hProcess );
 
 			//	build process object to be added in vector
@@ -346,8 +305,169 @@ void ProcessData::read_process_list_from_sys()
 	} while ( Process32Next(hProcessSnap, &pe32) );
 
 	CloseHandle(hProcessSnap);
+	
+#else	//=============================================================================
+
+	// Take a snapshot of all processes in the system.
+	HANDLE	hProcessSnap = 
+		CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS | TH32CS_SNAPMODULE, 0);
+	if( hProcessSnap == INVALID_HANDLE_VALUE)
+	{
+		string desc = errorCodeToString(GetLastError(), "CreateToolhelp32Snapshot" );
+		Tango::Except::throw_exception(
+						(const char *)"PROCESS_LIST_FAILED",
+						(const char *) desc.c_str(),
+						(const char *)"Starter::get_process_list()");
+	}
+
+	// Set the size of the structure before using it.
+	PROCESSENTRY32	pe32;
+	pe32.dwSize = sizeof( PROCESSENTRY32 );
+
+	// Retrieve information about the first process,
+	// and exit if unsuccessful
+	if( !Process32First( hProcessSnap, &pe32 ) )
+	{
+		string desc = errorCodeToString(GetLastError(), "Process32First" );  // Show cause of failure
+		CloseHandle( hProcessSnap );     // Must clean up the snapshot object!
+		Tango::Except::throw_exception(
+						(const char *)"PROCESS_LIST_FAILED",
+						(const char *) desc.c_str(),
+						(const char *)"Starter::get_process_list()");
+	}
+
+	//	Get module ntdll
+    NTQIP		*lpfnNtQueryInformationProcess;
+	HINSTANCE	hLibrary = GetModuleHandleA("ntdll.dll");
+    if (hLibrary != NULL)
+    {
+        lpfnNtQueryInformationProcess = (NTQIP *)GetProcAddress(hLibrary, "ZwQueryInformationProcess");
+    }
+	else
+    {
+		string	desc = errorCodeToString(GetLastError(), "GetModuleHandle() ");
+		Tango::Except::throw_exception(
+						(const char *)"PROCESS_LIST_FAILED",
+						(const char *) desc.c_str(),
+						(const char *)"Starter::get_process_list()");
+    }
+
+	do
+	{
+		// Retrieve the priority class.
+		//HANDLE	hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, pe32.th32ProcessID );
+		HANDLE	hProcess = OpenProcess(
+			PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, pe32.th32ProcessID );
+		if( hProcess != NULL )
+		{
+			string strLine("");
+
+			//	Get PEB address
+			PVOID	pebAddress = getPebAddress(hProcess);
+
+			//	Get the address of parameters
+			PVOID	rtlUserProcParamsAddress = getPebStructure(hProcess, pebAddress);
+			if (rtlUserProcParamsAddress!=NULL)
+			{
+				//	Read the command line in UNICODE structure
+				UNICODE_STRING	commandLine =
+					getUnicodeCommandLine(hProcess, rtlUserProcParamsAddress);
+					
+				//	Convert the command line
+				WCHAR *commandLineContents = new WCHAR[commandLine.Length];
+				if (ReadProcessMemory(hProcess, commandLine.Buffer,
+					commandLineContents, commandLine.Length, NULL))
+				{
+					strLine = wchar2string(
+						commandLineContents, commandLine.Length/2);
+				}
+				else
+					errorCodeToString(GetLastError(), "Cannot convert the command line" );
+				delete [] commandLineContents;
+			}
+			else
+				errorCodeToString(GetLastError(), "Cannot get the address of parameters" );
+
+
+			CloseHandle( hProcess );
+
+			//	build process object to be added in vector
+			Process	*process = new Process();
+
+			//	Remove extention from exe name
+			string	full_name = wchar2string(pe32.szExeFile);
+			string::size_type	pos = full_name.find('.');
+			if (pos!=string::npos)
+				process->name = full_name.substr(0, pos);
+			else
+				process->name = full_name;
+
+			//	Parse name frome cmd line because file manager truncate it at 15 chars
+			if (win2000 && process->name.length()>13)
+				process->name = parseNameFromCmdLine(process->name, strLine);
+
+			//	On win32 -> exe file is case unsesitive
+			transform(process->name.begin(), process->name.end(),
+				process->name.begin(), ::tolower);
+
+
+			//	add pid and cmd line
+			process->pid  = pe32.th32ProcessID;
+			process->line = strLine;
+			proc_list.push_back(process);
+		}
+	} while ( Process32Next(hProcessSnap, &pe32) );
+	CloseHandle(hProcessSnap);
+#endif
 }
 
+//============================================================================ 
+//============================================================================
+UNICODE_STRING ProcessData::getUnicodeCommandLine(HANDLE hProcess, PVOID paramAddress)
+{
+	UNICODE_STRING	commandLine;
+
+#if (_MSC_VER >= 1600)	//	VC10
+	RTL_USER_PROCESS_PARAMETERS	*pProcessParam =
+		(RTL_USER_PROCESS_PARAMETERS *) paramAddress;
+	ReadProcessMemory(hProcess,
+		(PCHAR) &pProcessParam->CommandLine,
+		&commandLine, sizeof(commandLine), NULL);
+#else
+	ReadProcessMemory(hProcess,
+		(PCHAR) paramAddress + 0x40,
+		&commandLine, sizeof(commandLine), NULL);
+#endif
+	return commandLine;
+}
+//============================================================================ 
+//============================================================================
+PVOID ProcessData::getPebStructure(HANDLE hProcess, PVOID pebAddress)
+{
+	PVOID	rtlUserProcParamsAddress = NULL;
+
+#if (_MSC_VER >= 1600)	//	VC10
+	PEB		*peb = (PEB *)pebAddress;
+	ReadProcessMemory(hProcess,
+		(PCHAR)&peb->ProcessParameters,
+		&rtlUserProcParamsAddress, sizeof(PVOID), NULL);
+#else
+	ReadProcessMemory(hProcess,
+		(PCHAR)pebAddress+0x10,
+		&rtlUserProcParamsAddress, sizeof(PVOID), NULL);
+#endif
+	return rtlUserProcParamsAddress;
+}
+//============================================================================ 
+//============================================================================
+PVOID  ProcessData::getPebAddress(HANDLE pHandle)
+{
+	NTQIP	*ntQueryInformationProcess = 
+		(NTQIP *)GetProcAddress(GetModuleHandleA("ntdll.dll"), "NtQueryInformationProcess");
+	PROCESS_BASIC_INFORMATION	pbi;
+	ntQueryInformationProcess(pHandle, ProcessBasicInformation, &pbi, sizeof(pbi), NULL);
+	return pbi.PebBaseAddress;
+}
 
 
 // ============================================================================
@@ -430,7 +550,7 @@ void ProcessData::read_process_list_from_sys()
 	} 
 
 	struct dirent	*ent;
-	while (ent = readdir (proc))
+	while ( (ent=readdir(proc)) )
 	{
 		if (isdigit (ent->d_name[0]))
 		{
@@ -471,7 +591,7 @@ void ProcessData::read_process_list_from_sys()
 //=============================================================
 bool  ProcessData::manageProcFiles(Process *process)
 {
-#if (defined linux) || (defined __darwin__) || (defined __freebsd__)
+#if (defined __GLIBC__) || (defined __darwin__) || (defined __freebsd__)
 
 	//	Read command line file
 	TangoSys_OMemStream	fname;
@@ -560,6 +680,8 @@ bool  ProcessData::manageProcFiles(Process *process)
 							line << buff << " ";
 
 					process->line = line.str();
+					free(argvp);
+					close(fdesc);
 				}
 				else
 				{
@@ -575,8 +697,6 @@ bool  ProcessData::manageProcFiles(Process *process)
 							(const char *) tms.str().c_str(),
 							(const char *)"Starter::manageProcFiles()");
 				}
-				free(argvp);
-				close(fdesc);
 			}
 		}
 		else
@@ -842,7 +962,7 @@ void ProcessData::update_process_list()
 
 
 #ifndef _TG_WINDOWS_
-		if (process->line_args.size()>0)
+		if (process->line_args.empty()==false)
 			process->name = process->line_args[0];
 		else
 			process->name = "";
@@ -856,7 +976,7 @@ void ProcessData::update_process_list()
 		build_server_names(process);
 #ifdef TRACE2
 		cout << process->pid << "	" << process->name;
-		if (process->proc_args.size()>0)
+		if (process->proc_args.empty()==false)
 			cout << " " << process->proc_args[0];
 		cout << endl;
 #endif
@@ -889,7 +1009,7 @@ int ProcessData::get_server_pid(string argin)
 	{
 		Process	*process = proc_list[i];
 		// server is a process with at least one arg
-		if (process->proc_args.size()>0)
+		if (process->proc_args.empty()==false)
 		{
 			string	servname(process->name);
 			servname += "/";
@@ -1007,7 +1127,7 @@ vector<Process> CheckProcessUtil::get_process_list()
 }
 //=============================================================
 //=============================================================
-void *CheckProcessUtil::run_undetached(void *ptr)
+void *CheckProcessUtil::run_undetached(TANGO_UNUSED(void *ptr))
 {
 	while (stop_thread==false)
 	{
