@@ -9,7 +9,7 @@ static const char *RcsId = "$Header$";
 //
 // $Author: pascal_verdier $
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -29,82 +29,8 @@ static const char *RcsId = "$Header$";
 // You should have received a copy of the GNU General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 15451 $
-//
-// $Log$
-// Revision 3.23  2010/10/18 12:58:52  pascal_verdier
-// Pogo-7 compatibility
-//
-// Revision 3.22  2010/10/15 06:20:33  pascal_verdier
-// Copyright added.
-//
-// Revision 3.21  2010/10/08 08:48:50  pascal_verdier
-// Include files order changed.
-//
-// Revision 3.20  2010/09/21 12:18:58  pascal_verdier
-// GPL Licence added to header.
-//
-// Revision 3.19  2010/02/09 15:09:49  pascal_verdier
-// Define  _TG_WINDOWS_  replace WIN32.
-// LogFileHome property added.
-//
-// Revision 3.18  2010/02/09 10:25:38  pascal_verdier
-// __MACOS__ compiler flag added.
-//
-// Revision 3.17  2008/12/12 13:29:56  pascal_verdier
-// Log in file start and stop for servers and itself.
-//
-// Revision 3.16  2008/09/23 14:19:40  pascal_verdier
-// Log files history added.
-//
-// Revision 3.15  2008/04/04 14:30:51  jensmeyer
-// Added compile option for FreeBSD and MacOSX.
-// They have a different signal handling.
-//
-// Revision 3.14  2008/02/29 15:15:05  pascal_verdier
-// Checking running processes by system call added.
-//
-// Revision 3.13  2007/09/28 06:12:30  pascal_verdier
-// Remove traces
-//
-// Revision 3.12  2007/09/27 15:11:15  pascal_verdier
-// GetLastError() management added on Win 32
-//
-// Revision 3.11  2007/09/25 13:06:17  pascal_verdier
-// In win32 char* replaced by LPWSTR :-)
-//
-// Revision 3.10  2007/06/20 06:48:21  pascal_verdier
-// delete replaced by  delete[].
-//
-// Revision 3.9  2007/03/23 09:23:44  pascal_verdier
-// Bug in windows batch file startup fixed.
-//
-// Revision 3.8  2007/02/01 09:17:44  pascal_verdier
-// Monitor added on some shared data.
-// Wait 3 seconds added at startup for first ping timeout.
-//
-// Revision 3.7  2006/11/20 06:58:37  pascal_verdier
-// Mutex on start process data added.
-//
-// Revision 3.6  2006/11/10 14:53:28  pascal_verdier
-// Remove vc8 warnings.
-//
-// Revision 3.5  2006/06/13 19:38:15  pascal_verdier
-// Minor changes.
-//
-// Revision 3.4  2006/06/06 12:01:26  pascal_verdier
-// Bug in log file fixed.
-//
-// Revision 3.3  2006/06/05 07:20:36  pascal_verdier
-// Server startup is now delayed (with timeout) in a startup level.
-// New state MOVING added.
-// At startup, starter loop until properties have been read.
-//
-// Revision 3.2  2006/04/24 08:58:10  pascal_verdier
-// *** empty log message ***
-//
-// Revision 3.1  2006/04/24 07:06:27  pascal_verdier
-// A thread is started for each level when at servers startup.
+// $Revision: 20367 $
+// $Date: 2012-05-25 10:36:17 +0200 (Fri, 25 May 2012) $
 //
 //+=============================================================================
 
@@ -138,14 +64,13 @@ StartProcessThread::StartProcessThread(vector<NewProcess *> v_np, int level, Sta
 		processes.push_back(v_np[i]);
 	thread_level = level;
 	starter      = st;
-
 }
 //+------------------------------------------------------------------
 /**
  * Execute the fork of the sub process in a thread.
  */
 //+------------------------------------------------------------------
-void *StartProcessThread::run_undetached(void *ptr)
+void StartProcessThread::run(TANGO_UNUSED(void *ptr))
 {
 	unsigned int	i;
 	CheckProcessUtil	*process_util = starter->util->proc_util;
@@ -163,9 +88,6 @@ void *StartProcessThread::run_undetached(void *ptr)
 	//	Start the level process start loop
 	for (i=0 ; i<processes.size() ; i++)
 	{
-		//cout << "Starting " <<
-		//	processes[i]->servname << "/" << processes[i]->instancename << endl;
-
 		start_process(processes[i]);
 
 		//	Build server name from admin name
@@ -254,7 +176,6 @@ void *StartProcessThread::run_undetached(void *ptr)
 //	starter->starting_processes--;
 	//	remove in level vector to start another level
 	starter->start_proc_data->remove_current_level();
-	return NULL;
 }
 #ifndef	_TG_WINDOWS_
 //+------------------------------------------------------------------
@@ -392,7 +313,7 @@ string StartWinThread::get_server_name_with_cotes(string servname)
 //	WIN 32 Thread to fork a sub process
 //		If batch file, the spawnv is blocking !!!
 //+----------------------------------------------------------
-void *StartWinThread::run_undetached(void *ptr)
+void StartWinThread::run(void *ptr)
 {
 	//	Check if batch file
 	string	str_server(process->servname);
@@ -405,8 +326,10 @@ void *StartWinThread::run_undetached(void *ptr)
 	cmd += process->instancename;
 	cout << "system(" << cmd << ");" << endl;
 	system(cmd.c_str());
-	return NULL;
+	return;
 
+//	Not used any more
+/*************************
 	if (str_server.find(".bat") != string::npos)
 	{
 		char	*argv[3];
@@ -556,7 +479,7 @@ void *StartWinThread::run_undetached(void *ptr)
 		ofs.close();
 	}
 	delete[] servname;
-	return NULL;
+***********************/
 }
 
 #endif	//	_TG_WINDOWS_
