@@ -9,7 +9,7 @@ static const char *RcsId = "$Header$";
 //
 // $Author: pascal_verdier $
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -29,133 +29,8 @@ static const char *RcsId = "$Header$";
 // You should have received a copy of the GNU General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 15451 $
-//
-// $Log$
-// Revision 3.28  2010/10/18 12:58:52  pascal_verdier
-// Pogo-7 compatibility
-//
-// Revision 3.27  2010/10/15 06:20:33  pascal_verdier
-// Copyright added.
-//
-// Revision 3.26  2010/10/08 08:48:50  pascal_verdier
-// Include files order changed.
-//
-// Revision 3.25  2010/09/21 12:18:58  pascal_verdier
-// GPL Licence added to header.
-//
-// Revision 3.24  2010/02/09 15:09:49  pascal_verdier
-// Define  _TG_WINDOWS_  replace WIN32.
-// LogFileHome property added.
-//
-// Revision 3.23  2008/12/12 13:29:56  pascal_verdier
-// Log in file start and stop for servers and itself.
-//
-// Revision 3.22  2008/09/23 14:19:41  pascal_verdier
-// Log files history added.
-//
-// Revision 3.21  2008/05/15 08:07:18  pascal_verdier
-// TangoSys_MemStream replaced by TangoSys_OMemStream
-// (for leaking problem under win32)
-//
-// Revision 3.20  2008/03/27 07:55:17  pascal_verdier
-// *** empty log message ***
-//
-// Revision 3.19  2008/03/03 13:25:35  pascal_verdier
-// Do not try to narrow notifd if not running.
-//
-// Revision 3.18  2008/02/29 15:15:05  pascal_verdier
-// Checking running processes by system call added.
-//
-// Revision 3.17  2007/11/12 08:09:28  pascal_verdier
-// const char* cast added on throw_exception call.
-//
-// Revision 3.16  2007/09/25 12:12:00  pascal_verdier
-// little memory leaks fixed.
-//
-// Revision 3.15  2007/05/25 06:08:00  pascal_verdier
-// Iterator management modified (strange in vc8 ?)
-//
-// Revision 3.14  2007/05/03 06:54:58  pascal_verdier
-// Re-try on DServer exported added before polling startup added.
-//
-// Revision 3.13  2007/03/30 14:21:49  pascal_verdier
-// remove 2 warnings on linux.
-//
-// Revision 3.12  2006/11/20 06:58:37  pascal_verdier
-// Mutex on start process data added.
-//
-// Revision 3.11  2006/11/10 14:53:28  pascal_verdier
-// Remove vc8 warnings.
-//
-// Revision 3.10  2006/06/05 07:20:36  pascal_verdier
-// Server startup is now delayed (with timeout) in a startup level.
-// New state MOVING added.
-// At startup, starter loop until properties have been read.
-//
-// Revision 3.9  2006/04/24 07:06:28  pascal_verdier
-// A thread is started for each level when at servers startup.
-//
-// Revision 3.8  2006/02/09 11:59:19  pascal_verdier
-// A ping thread is now started for each server.
-//
-// Revision 3.7  2005/09/19 13:35:22  pascal_verdier
-// change on traces.
-//
-// Revision 3.6  2005/08/26 07:18:16  pascal_verdier
-// FQDN management added.
-// Time between startup level added.
-//
-// Revision 3.5  2005/05/18 12:11:56  pascal_verdier
-// Start a batch file under windows.
-//
-// Revision 3.4  2005/04/21 07:18:48  pascal_verdier
-// Add a little timeout for ping.
-// Fix a bug for windows (SIGCHLD).
-//
-// Revision 3.3  2004/06/29 04:24:26  pascal_verdier
-// First revision using events.
-//
-// Revision 3.2  2004/05/19 08:56:32  pascal_verdier
-// Bug fixed on service mode.
-// CreateProcess used under Windows.
-//
-// Revision 3.0.1.4  2004/02/27 09:53:02  pascal_verdier
-// - The starter device is now warned by Database server when something change on a server.
-// It replace the DbGetHostServersInfo polling.
-// - HostState, ControlledRunningServers, ontrolledStoppedServers attributes added.
-//
-// Revision 3.0.1.3  2003/12/08 08:53:52  pascal_verdier
-// Cluster (multi-host) control implemented.
-// Control of notify daemon implemented but not tested.
-//
-// Revision 3.0.1.2  2003/10/15 10:37:08  pascal_verdier
-// *** empty log message ***
-//
-// Revision 3.0.1.1  2003/09/18 12:02:49  pascal_verdier
-// Problem on Windows service startup fixed.
-//
-// Revision 3.0  2003/06/17 12:06:36  pascal_verdier
-// TANGO 3.x server.
-// polling bugs fixed.
-//
-// Revision 2.0  2003/01/09 13:35:50  verdier
-// TANGO 2.2
-//
-// Revision 1.22  2002/12/18 08:09:19  verdier
-// omniORB compatibility
-//
-// Revision 1.6  2002/10/15 18:55:21  verdier
-// The host state is now calculated during the State polled command.
-//
-// Revision 1.5  2002/02/27 15:19:32  verdier
-// WinNt services implemented.
-//
-// Revision 1.2  2001/04/03 09:40:50  verdier
-// WInNt version is running.
-//
-// Revision 1.1  2001/02/12 09:34:21  verdier
-// Initial revision
+// $Revision: 20367 $
+// $Date: 2012-05-25 10:36:17 +0200 (Fri, 25 May 2012) $
 //
 //-=============================================================================
 
@@ -216,7 +91,11 @@ StarterUtil::StarterUtil(Tango::DeviceProxy *database, vector<string> host_names
 	LogPath(starter_log_file, logHome);
 	starter_log_file += "/Starter.log";
 
-	cout << "---->  starter_log_file = " << starter_log_file << endl;
+	LogPath(starter_stat_file, logHome);
+	starter_stat_file += "/Starter.stat";
+
+	cout << "---->  starter_log_file  = " << starter_log_file << endl;
+	cout << "---->  starter_stat_file = " << starter_stat_file << endl;
 }
 //+------------------------------------------------------------------
 /**
@@ -387,7 +266,6 @@ char *StarterUtil::get_file_date(char *filename)
 //+------------------------------------------------------------------
 void StarterUtil::log_starter_info(string message)
 {
-
 	stringstream	strlog;
 	strlog << strtime(time(NULL)) << "\t" << message  << endl;
 
@@ -395,7 +273,7 @@ void StarterUtil::log_starter_info(string message)
 	ifstream	ifs((char *)starter_log_file.c_str());
 	if (ifs)
 	{
-		strlog << ifs.rdbuf() << ends;
+		strlog << ifs.rdbuf();
 		ifs.close();
 	}
 	//	Check for nb lines
@@ -410,12 +288,81 @@ void StarterUtil::log_starter_info(string message)
 
 	//	Write to log file
 	ofstream	ofs((char *)starter_log_file.c_str());
-	ofs << str << ends;
+	ofs << str;
 	ofs.close();
 }
 //+------------------------------------------------------------------
 /**
- *	check if there is no to much log file and rename last one
+ *	Reset statistics for starter.
+ */
+//+------------------------------------------------------------------
+void StarterUtil::reset_starter_stat_file(vector<ControlledServer> *servers)
+{
+	//	build present situation log
+	stringstream	strlog;
+	vector<ControlledServer>::iterator it;
+	for (it=servers->begin() ; it<servers->end() ; ++it)
+	{
+		if (it->controlled && it->startup_level>0)
+		{
+			time_t	failure_time = it->failure_time;
+			if (failure_time<0)
+				failure_time = time(NULL);
+
+			strlog << it->name           << "\t" <<
+	                ((it->state==Tango::ON)? "ON\t" : "FAULT\t") <<
+	                it->started_time << "\t" <<
+						failure_time << "\t" <<
+					"false" << endl;
+		}
+	}
+
+	//	Write an empty statistics file
+	ofstream	ofs((char *)starter_stat_file.c_str());
+	ofs << strlog.str();
+	ofs.close();
+}
+//+------------------------------------------------------------------
+/**
+ *	Log statistica for starter.
+ *	@param	message	 mesage to be logged
+ */
+//+------------------------------------------------------------------
+void StarterUtil::log_starter_statistics(ControlledServer *server)
+{
+	stringstream	strlog;
+	strlog << server->name           << "\t" <<
+	                ((server->state==Tango::ON)? "ON\t" : "FAULT\t") <<
+	                server->started_time  << "\t" <<
+					server->failure_time  << "\t" <<
+					((server->auto_start)? "true" : "false") << endl;
+	server->auto_start = false;
+
+	//	Read and close log file.
+	ifstream	ifs((char *)starter_stat_file.c_str());
+	if (ifs)
+	{
+		strlog << ifs.rdbuf();
+		ifs.close();
+	}
+	//	Check for nb lines
+	string	str(strlog.str());
+	string::size_type	pos = 0;
+	int nb = 0;
+	while (nb<STARTER_STAT_DEPTH && (pos=str.find('\n', pos+1))!=string::npos)
+		nb++;
+
+	if (pos!=string::npos)
+		str = str.substr(0, pos);
+
+	//	Write to log file
+	ofstream	ofs((char *)starter_stat_file.c_str());
+	ofs << str;
+	ofs.close();
+}
+//+------------------------------------------------------------------
+/**
+ *	check if there is not to much log file and rename last one
  *	@param	filename	file's name to get the date and rename.
  */
 //+------------------------------------------------------------------
@@ -430,13 +377,13 @@ void StarterUtil::manage_log_file_history(char *filename, int nb_max)
 	
 	//	Get the log file list
 	vector<string>	list =  get_log_file_list(log_file);
-	for (unsigned int i=0 ; i<list.size() ; i++)
-		cout << list[i] << endl;
+	//for (unsigned int i=0 ; i<list.size() ; i++)
+	//	cout << list[i] << endl;
 	
 	//	Check if too much files -> delete
 	while (list.size()>((unsigned int)nb_max-1))	//	-1 because a new one will exist bellow
 	{
-		cout << "Removing " << list[0] << endl;
+		//cout << "Removing " << list[0] << endl;
 		if (remove(list[0].c_str())<0)
 			cerr << "remove failed : " << strerror(errno) << endl;
 		list.erase(list.begin());
@@ -464,12 +411,11 @@ void StarterUtil::manage_log_file_history(char *filename, int nb_max)
 	strcat(new_filename, strdate);
 	strcat(new_filename, ".log");
 	int ret = rename(filename, new_filename);
-	cout << "Renaming " << filename << " to " << new_filename;
 	if (ret<0)
-		cout << " failed : " << strerror(errno) << endl;
-	else
-		cout << "  done: " << ret << endl;
-	delete new_filename;
+		cerr << "Renaming " << filename << " to " << new_filename << " failed : " << strerror(errno) << endl;
+	//else
+	//	cout << "Renaming " << filename << " to " << new_filename << "  done." << endl;
+	delete [] new_filename;
 }
 //-----------------------------------------------------------------------------
 //-----------------------------------------------------------------------------
@@ -528,7 +474,7 @@ vector<string> StarterUtil::get_log_file_list(string logfile)
 
 		} 
 		struct dirent	*ent;
-		while (ent=readdir (dir))
+		while ( (ent=readdir(dir)) )
 		{
 			string	name(ent->d_name);
 			pos = name.find(filter);
@@ -540,6 +486,7 @@ vector<string> StarterUtil::get_log_file_list(string logfile)
 				list.push_back(full_name);
 			}
 		}
+		closedir(dir);
 #else
 		cout << "Searching " << filter << "  in " << path << endl;
 		path += "\\";
@@ -647,7 +594,7 @@ vector<string>	StarterUtil::get_host_ds_list()
 
 		//	Check servers really used (Erase this one and database server
 		vector<string>::iterator pos;
-		for (pos=tmp.begin() ; pos<tmp.end() ; pos++)
+		for (pos=tmp.begin() ; pos<tmp.end() ; ++pos)
 		{
 			int	idx = (*pos).find_first_of("/");
 			if (idx>0)
@@ -661,7 +608,7 @@ vector<string>	StarterUtil::get_host_ds_list()
 					s=="logconsumer")
 				{
 					tmp.erase(pos);
-					pos--;	//	because erase decrease size !
+					--pos;	//	because erase decrease size !
 				}
 			}
 		}
@@ -677,14 +624,14 @@ vector<string>	StarterUtil::get_host_ds_list()
 }
 //+------------------------------------------------------------------
 /**
- *	Read DS info from database to know if it is controled
+ *	Read DS info from database to know if it is controlled
  *		and it's starting level.
  *
  *	@param	devname	device to get info.
  *	@param	server	object to be updated from db read.
  */
 //+------------------------------------------------------------------
-void StarterUtil::get_server_info(ControledServer *server)
+void StarterUtil::get_server_info(ControlledServer *server)
 {
 	try
 	{
@@ -694,24 +641,24 @@ void StarterUtil::get_server_info(ControledServer *server)
 		Tango::DeviceData	argout = dbase->command_inout("DbGetServerInfo", argin);
 		vector<string>	result;
 		argout >> result;
-		server->controled     = (atoi(result[2].c_str())==0)? false: true;
+		server->controlled     = (atoi(result[2].c_str())==0)? false: true;
 		server->startup_level =  atoi(result[3].c_str());
 	}
 	catch(Tango::DevFailed &e)
 	{
 		Tango::Except::print_exception(e);
-		server->controled = false;
+		server->controlled = false;
 		server->startup_level = 0;
 	}
-	//cout << server->name << " - " << ((server->controled)? "true": "false");
+	//cout << server->name << " - " << ((server->controlled)? "true": "false");
 	//cout << " ----> Startup level " << server->startup_level <<endl;
 }
 //+------------------------------------------------------------------
 /**
- *	Allocate and fill the servers controled object
+ *	Allocate and fill the servers controlled object
  */
 //+------------------------------------------------------------------
-void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
+void StarterUtil::build_server_ctrl_object(vector<ControlledServer> *servers)
 {
 	bool trace = false;
 	if (trace)	cout << "build_server_ctrl_object()" << endl;
@@ -729,7 +676,7 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 
 	if (trace)	cout << "--------------  Check if list of servers modified  --------------" << endl;
 	
-	//	Check servers really used (erase this one and database server
+	//	Check servers really used (erase this one and database server)
 	vector<string>::iterator pos;
 	vector<string>	result;
 	for (pos=result_from_db.begin() ; pos<result_from_db.end() ; pos+=3)
@@ -753,13 +700,13 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 	}
 
 	//	Check if some servers have disappeared
-	vector<ControledServer>::iterator it;
+	vector<ControlledServer>::iterator it;
 	bool	redo = true;	//	Iterators management seems to have changed
 	                        //	between vc6 and vc8  (??)
 	while (redo)
 	{
 		redo = false;
-		for (it=servers->begin() ; it<servers->end() ; it++)
+		for (it=servers->begin() ; it<servers->end() ; ++it)
 		{
 			string	s1(it->name);
 			bool	found = false;
@@ -781,19 +728,23 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 	for (pos=result.begin() ; pos<result.end() ; )
 	{
 		string	name(*pos++);
-		ControledServer	*p_serv = get_server_by_name(name, *servers);
+		ControlledServer	*p_serv = get_server_by_name(name, *servers);
 		if (p_serv==NULL)
 		{
 			if (trace)	cout << name << " appeared " << endl;
 			//	Create a new server instance
-			ControledServer	server;
+			ControlledServer	server;
 
 			server.name = name;
 			server.admin_name = "dserver/" + server.name;
 			server.dev = NULL;
-			server.controled = (atoi((*pos++).c_str())==0)? false: true;
+			server.controlled = (atoi((*pos++).c_str())==0)? false: true;
 			server.startup_level =  atoi((*pos++).c_str());
-			server.state   = Tango::FAULT;
+			server.state         = Tango::FAULT;
+			server.stopped       = false;
+			server.auto_start    = false;
+			server.started_time  = time(NULL);
+			server.failure_time  = -1;
 
 			//	Add a thread to ping server
 			server.thread_data = new PingThreadData(server.name);
@@ -807,7 +758,7 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 		else
 		{
 			//	Update levels
-			p_serv->controled     = (atoi((*pos++).c_str())==0)? false: true;
+			p_serv->controlled    = (atoi((*pos++).c_str())==0)? false: true;
 			p_serv->startup_level =  atoi((*pos++).c_str());
 		}
 	}
@@ -815,16 +766,16 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 
 //+------------------------------------------------------------------
 /**
- *	search a server in ControledServer array by it's name .
+ *	search a server in ControlledServer array by it's name .
  *
  *	@param servname	Server searched name.
  */
 //+------------------------------------------------------------------
-ControledServer *StarterUtil::get_server_by_name(string &servname, vector<ControledServer> &servers)
+ControlledServer *StarterUtil::get_server_by_name(string &servname, vector<ControlledServer> &servers)
 {
 	for (unsigned int i=0 ; i<servers.size() ; i++)
 	{
-		ControledServer	*server = &servers[i];
+		ControlledServer	*server = &servers[i];
 		if (server->name == servname)
 			return server;
 	}
@@ -911,109 +862,6 @@ Tango::DevState StarterUtil::is_notifyd_alive()
 	}
 	return notifd_state;
 }
-
-
-
-
-
-
-
-
-
-
-
-//+----------------------------------------------------------------------------
-/**
- *	Create a thread to start  polling.
- */
-//+----------------------------------------------------------------------------
-PollingState::PollingState(string devname)
-{
-	device_name = devname;
-}
-//+----------------------------------------------------------------------------
-/**
- *	Execute the thread commands
- */
-//+----------------------------------------------------------------------------
-void *PollingState::run_undetached(void *ptr)
-{
-	//	Check if dserver exported
-	bool		done = false;
-	Tango::Util *tg = Tango::Util::instance();
-	Tango::DServer		*adm_dev = NULL;
-	for (int i=0 ; !done && i<10 ; i++)
-	{
-		try
-		{
-			ms_sleep(500);
-			cout << "Try if DServer exported " << i << "/10" << endl;
-			adm_dev = tg->get_dserver_device();
-			adm_dev->ping();
-			done = true;
-		}
-		catch(Tango::DevFailed &e)
-		{
-			Tango::Except::print_exception(e);
-		}
-	}
-	//	If dserver not started , do nothing
-	if (adm_dev!=NULL)
-	{
-		long	upd_period = 1000;
-		add_obj_polling(adm_dev,"attribute", "State",          upd_period);
-		add_obj_polling(adm_dev,"attribute", "HostState",      upd_period);
-		add_obj_polling(adm_dev,"attribute", "RunningServers", upd_period);
-		add_obj_polling(adm_dev,"attribute", "StoppedServers", upd_period);
-	}
-	return NULL;
-}
- 
-//+------------------------------------------------------------------
-/**
- *	Add the Polling object.
- */
-//+------------------------------------------------------------------
-void PollingState::add_obj_polling(Tango::DServer *adm_dev,string type, string objname, int upd_period)
-{
-	Tango::DevVarLongStringArray	*lsa = new Tango::DevVarLongStringArray();
-	lsa->lvalue.length(1);
-	lsa->svalue.length(3);
-	lsa->lvalue[0] = upd_period;
-	lsa->svalue[0] = CORBA::string_dup(device_name.c_str());
-	lsa->svalue[1] = CORBA::string_dup(type.c_str());
-	lsa->svalue[2] = CORBA::string_dup(objname.c_str());
-	CORBA::Any	any;
-	any <<= lsa;
-	try {
-		CORBA::Any	*any_out =
-			adm_dev->command_inout("AddObjPolling", any);
-		delete any_out;
-	}
-	catch (Tango::DevFailed &e) {
-
-		string	reason(e.errors[0].reason);
-		if (reason=="API_AlreadyPolled")
-		{
-			//	If already polled -> just update period.
-			//----------------------------------------------
-			try {
-				CORBA::Any	*any_out =
-					adm_dev->command_inout("UpdObjPollingPeriod", any);
-				delete any_out;
-				//cout <<"UpdObjPollingPeriod(" << objname <<
-				//			", " << upd_period << " ms)" << endl;
-			}
-			catch (Tango::DevFailed &e) {
-				Tango::Except::print_exception(e);
-			}
-		}
-		else
-			Tango::Except::print_exception(e);
-	}
-}
-//+----------------------------------------------------------------------------
-
-
+//-----------------------------------------------------------------------------
 
 }	//	namespace
