@@ -1,7 +1,7 @@
 //
 // AccessProxy.h -	include file for TANGO AccessProxy class
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -12,12 +12,12 @@
 // it under the terms of the GNU Lesser General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // Tango is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
@@ -45,12 +45,12 @@ public:
 
 protected:
 	string							user;
-	string							host;
+	vector<string>					host_ips;
 	bool							forced;
 	map<string,vector<string> > 	allowed_cmd_table;
 
 	void get_allowed_commands(string &,vector<string> &);
-	
+
 private:
 	void real_ctor();
 };
