@@ -1,12 +1,12 @@
-static const char *RcsId = "$Id: attr_proxy.cpp 15556 2011-02-11 08:25:58Z taurel $\n$Name$";
+static const char *RcsId = "$Id: attr_proxy.cpp 20437 2012-05-31 12:33:29Z taurel $\n$Name$";
 //
-// attr_proxy.cpp 	- C++ source code file for TANGO attribute proxy api 
+// attr_proxy.cpp 	- C++ source code file for TANGO attribute proxy api
 //
 // programmer(s)	- Andy Gotz (goetz@esrf.fr)
 //
 // original 		- July 2003
 //
-// Copyright (C) :      2003,2004,2005,2006,2007,2008,2009,2010,2011
+// Copyright (C) :      2003,2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -17,12 +17,12 @@ static const char *RcsId = "$Id: attr_proxy.cpp 15556 2011-02-11 08:25:58Z taure
 // it under the terms of the GNU Lesser General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // Tango is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
@@ -34,6 +34,7 @@ static const char *RcsId = "$Id: attr_proxy.cpp 15556 2011-02-11 08:25:58Z taure
 #endif
 
 #include <tango.h>
+#include <eventconsumer.h>
 
 #ifdef _TG_WINDOWS_
 #include <sys/timeb.h>
@@ -45,7 +46,7 @@ static const char *RcsId = "$Id: attr_proxy.cpp 15556 2011-02-11 08:25:58Z taure
 #include <signal.h>
 
 #include <algorithm>
-                                                    
+
 using namespace CORBA;
 
 namespace Tango
@@ -53,16 +54,16 @@ namespace Tango
 
 //-----------------------------------------------------------------------------
 //
-// AttributeProxy::AttributeProxy() - constructor for attribute proxy object 
+// AttributeProxy::AttributeProxy() - constructor for attribute proxy object
 //
 //-----------------------------------------------------------------------------
 
-AttributeProxy::AttributeProxy (string &name):dev_proxy(NULL),ext(NULL)
+AttributeProxy::AttributeProxy (string &name):dev_proxy(NULL),ext(Tango_NullPtr)
 {
 	real_constructor(name);
 }
 
-AttributeProxy::AttributeProxy (const char *na):dev_proxy(NULL),ext(NULL)
+AttributeProxy::AttributeProxy (const char *na):dev_proxy(NULL),ext(Tango_NullPtr)
 {
 	string name(na);
 	real_constructor(name);
@@ -81,11 +82,11 @@ void AttributeProxy::real_constructor (string &name)
 //
 // Create the associated DeviceProxy object
 //
-		
+
 	if (dbase_used == true)
 	{
 		if (from_env_var == true)
-		{	
+		{
 			ApiUtil *ui = ApiUtil::instance();
 			dev_proxy = new DeviceProxy(device_name);
 			if (ui->in_server() == true)
@@ -108,11 +109,11 @@ void AttributeProxy::real_constructor (string &name)
 	else
 	{
 		db_attr = NULL;
-		
+
 		string::size_type stop;
 		stop = name.rfind(DEVICE_SEP);
 		string nodb_dev_name  = name.substr(0,stop);
-		nodb_dev_name = nodb_dev_name + "#dbase=no";
+		nodb_dev_name = nodb_dev_name + MODIFIER_DBASE_NO;
 
 		dev_proxy = new DeviceProxy(nodb_dev_name);
 	}
@@ -131,18 +132,18 @@ void AttributeProxy::real_constructor (string &name)
 	{
 		delete db_attr;
 		delete dev_proxy;
-		
+
 		if (strcmp(dfe.errors[0].reason.in(),"API_AttrNotFound") == 0)
 		{
 			TangoSys_OMemStream desc;
 			desc << "Attribute " << attr_name << " is not supported by device " << device_name << ends;
-			
+
 			ApiWrongNameExcept::throw_exception((const char*)"API_UnsupportedAttribute",
 						desc.str(),
 						(const char*)"AttributeProxy::real_constructor()");
 		}
 	}
-	
+
 }
 
 
@@ -159,7 +160,7 @@ void AttributeProxy::ctor_from_dp(const DeviceProxy *dev_ptr,string &att_name)
 //
 // Init local data members from device proxy object
 //
-	
+
 	dbase_used = dev_proxy->dbase_used;
 	from_env_var = dev_proxy->from_env_var;
 	host = dev_proxy->host;
@@ -168,19 +169,19 @@ void AttributeProxy::ctor_from_dp(const DeviceProxy *dev_ptr,string &att_name)
 	db_host = dev_proxy->db_host;
 	db_port = dev_proxy->db_port;
 	db_port_num = dev_proxy->db_port_num;
-	
+
 	attr_name = att_name;
 
 //
 // Now AttributeProxy members
 //
-	
+
 	device_name = dev_proxy->device_name;
-			
+
 	if (dbase_used == true)
 	{
 		if (from_env_var == true)
-		{	
+		{
 			ApiUtil *ui = ApiUtil::instance();
 			if (ui->in_server() == true)
 			{
@@ -196,9 +197,7 @@ void AttributeProxy::ctor_from_dp(const DeviceProxy *dev_ptr,string &att_name)
 			db_attr = new DbAttribute(attr_name,device_name,db_host,db_port);
 		}
 	}
-	
-	ext = new AttributeProxyExt();
-	
+
 //
 // Check that the device support this attribute
 //
@@ -212,14 +211,13 @@ void AttributeProxy::ctor_from_dp(const DeviceProxy *dev_ptr,string &att_name)
 	catch (Tango::DevFailed &dfe)
 	{
 		delete db_attr;
-		delete ext;
 		delete dev_proxy;
-		
+
 		if (strcmp(dfe.errors[0].reason.in(),"API_AttrNotFound") == 0)
 		{
 			TangoSys_OMemStream desc;
 			desc << "Attribute " << attr_name << " is not supported by device " << device_name << ends;
-			
+
 			ApiWrongNameExcept::throw_exception((const char*)"API_UnsupportedAttribute",
 						desc.str(),
 						(const char*)"AttributeProxy::ctor_from_dp()");
@@ -227,13 +225,13 @@ void AttributeProxy::ctor_from_dp(const DeviceProxy *dev_ptr,string &att_name)
 	}
 }
 
-AttributeProxy::AttributeProxy (const DeviceProxy *dev_ptr,const char *att_name):ext(NULL)
+AttributeProxy::AttributeProxy (const DeviceProxy *dev_ptr,const char *att_name):ext(Tango_NullPtr)
 {
 	string att_na(att_name);
 	ctor_from_dp(dev_ptr,att_na);
 }
 
-AttributeProxy::AttributeProxy (const DeviceProxy *dev_ptr,string &att_name):ext(NULL)
+AttributeProxy::AttributeProxy (const DeviceProxy *dev_ptr,string &att_name):ext(Tango_NullPtr)
 {
 	ctor_from_dp(dev_ptr,att_name);
 }
@@ -245,7 +243,7 @@ AttributeProxy::AttributeProxy (const DeviceProxy *dev_ptr,string &att_name):ext
 //
 //-----------------------------------------------------------------------------
 
-AttributeProxy::AttributeProxy(const AttributeProxy &prev)
+AttributeProxy::AttributeProxy(const AttributeProxy &prev):ext(Tango_NullPtr)
 {
 
 //
@@ -259,17 +257,17 @@ AttributeProxy::AttributeProxy(const AttributeProxy &prev)
 	db_host = prev.db_host;
 	db_port = prev.db_port;
 	db_port_num = prev.db_port_num;
-	
+
 //
 // Now AttributeProxy members
 //
-	
+
 	device_name = prev.device_name;
-			
+
 	if (dbase_used == true)
 	{
 		if (from_env_var == true)
-		{	
+		{
 			ApiUtil *ui = ApiUtil::instance();
 			if (ui->in_server() == true)
 			{
@@ -290,7 +288,13 @@ AttributeProxy::AttributeProxy(const AttributeProxy &prev)
 			dev_proxy = new DeviceProxy(device_name);
 		}
 	}
-	
+
+#ifdef HAS_UNIQUE_PTR
+    if (prev.ext.get() != NULL)
+    {
+        ext.reset(new AttributeProxyExt);
+    }
+#else
 	if (prev.ext != NULL)
 	{
 		ext = new AttributeProxyExt();
@@ -298,72 +302,85 @@ AttributeProxy::AttributeProxy(const AttributeProxy &prev)
 	}
 	else
 		ext = NULL;
-		
+#endif
+
 }
 
 //-----------------------------------------------------------------------------
 //
-// AttributeProxy::AttributeProxy() - assignment operator 
+// AttributeProxy::AttributeProxy() - assignment operator
 //
 //-----------------------------------------------------------------------------
 
 AttributeProxy &AttributeProxy::operator=(const AttributeProxy &rval)
 {
 
+    if (this != &rval)
+    {
+
 //
 // First Connection call members
 //
-	if (dbase_used == true)
-		delete db_attr;
-	dbase_used = rval.dbase_used;
-	from_env_var = rval.from_env_var;
-	host = rval.host;
-	port = rval.port;
-	port_num = rval.port_num;
-	db_host = rval.db_host;
-	db_port = rval.db_port;
-	db_port_num = rval.db_port_num;
-	
+
+        if (dbase_used == true)
+            delete db_attr;
+        dbase_used = rval.dbase_used;
+        from_env_var = rval.from_env_var;
+        host = rval.host;
+        port = rval.port;
+        port_num = rval.port_num;
+        db_host = rval.db_host;
+        db_port = rval.db_port;
+        db_port_num = rval.db_port_num;
+
 //
 // Now AttributeProxy members
 //
-	
-	attr_name = rval.attr_name;
-	device_name = rval.device_name;
-			
-	if (dbase_used == true)
-	{
-		if (from_env_var == true)
-		{	
-			ApiUtil *ui = ApiUtil::instance();
-			if (ui->in_server() == true)
-			{
-				db_attr = new DbAttribute(attr_name,device_name,Tango::Util::instance()->get_database());
-				dev_proxy = new DeviceProxy(device_name);
-			}
-			else
-			{
-				db_attr = new DbAttribute(attr_name,device_name);
-				dev_proxy = new DeviceProxy(device_name);
-			}
-		}
-		else
-		{
-			string noenv_dev_name(db_host);
-			noenv_dev_name = noenv_dev_name + ":" + db_port + "/" + device_name;
-			dev_proxy = new DeviceProxy(noenv_dev_name);
-			db_attr = new DbAttribute(attr_name,device_name,db_host,db_port);
-		}
-	}
 
-	if (rval.ext != NULL)
-	{
-		ext = new AttributeProxyExt();
-		*ext = *(rval.ext);
-	}
-	else
-		ext = NULL;
-			
+        attr_name = rval.attr_name;
+        device_name = rval.device_name;
+
+        if (dbase_used == true)
+        {
+            if (from_env_var == true)
+            {
+                ApiUtil *ui = ApiUtil::instance();
+                if (ui->in_server() == true)
+                {
+                    db_attr = new DbAttribute(attr_name,device_name,Tango::Util::instance()->get_database());
+                    dev_proxy = new DeviceProxy(device_name);
+                }
+                else
+                {
+                    db_attr = new DbAttribute(attr_name,device_name);
+                    dev_proxy = new DeviceProxy(device_name);
+                }
+            }
+            else
+            {
+                string noenv_dev_name(db_host);
+                noenv_dev_name = noenv_dev_name + ":" + db_port + "/" + device_name;
+                dev_proxy = new DeviceProxy(noenv_dev_name);
+                db_attr = new DbAttribute(attr_name,device_name,db_host,db_port);
+            }
+        }
+
+#ifdef HAS_UNIQUE_PTR
+        if (rval.ext.get() != NULL)
+            ext.reset(new AttributeProxyExt);
+        else
+            ext.reset();
+#else
+        if (rval.ext != NULL)
+        {
+            ext = new AttributeProxyExt();
+            *ext = *(rval.ext);
+        }
+        else
+            ext = NULL;
+#endif
+    }
+
 	return *this;
 }
 
@@ -385,9 +402,9 @@ void AttributeProxy::parse_name(string &full_name)
 // Attribute name in lower case letters
 //
 
-	string cased_name = full_name;	
+	string cased_name = full_name;
 	transform(full_name.begin(),full_name.end(),full_name.begin(),::tolower);
-		
+
 //
 // Try to find protocol specification in attribute name and analyse it
 //
@@ -412,9 +429,9 @@ void AttributeProxy::parse_name(string &full_name)
 		if (protocol == TANGO_PROTOCOL)
 		{
 			name_wo_prot = full_name.substr(pos + 3);
-		}		
+		}
 		else if (protocol == TACO_PROTOCOL)
-		{		
+		{
 			TangoSys_OMemStream desc;
 			desc << "Taco protocol is not supported" << ends;
 			ApiWrongNameExcept::throw_exception((const char*)"API_UnsupportedProtocol",
@@ -422,14 +439,14 @@ void AttributeProxy::parse_name(string &full_name)
 						(const char*)"AttributeProxy::parse_name()");			exit(0);
 		}
 		else
-		{		
+		{
 			TangoSys_OMemStream desc;
 			desc << protocol;
 			desc << " protocol is an unsupported protocol" << ends;
 			ApiWrongNameExcept::throw_exception((const char*)"API_UnsupportedProtocol",
 						desc.str(),
 						(const char*)"AttributeProxy::parse_name()");
-		}			
+		}
 	}
 
 //
@@ -440,11 +457,11 @@ void AttributeProxy::parse_name(string &full_name)
 	if (pos != string::npos)
 	{
 		string mod = name_wo_prot.substr(pos + 1);
-		
+
 		if (mod == DBASE_YES)
 		{
 			string::size_type len = name_wo_prot.size();
-			name_wo_db_mod = name_wo_prot.substr(0,len - (len - pos)); 
+			name_wo_db_mod = name_wo_prot.substr(0,len - (len - pos));
 			dbase_used = true;
 		}
 		else if (mod == DBASE_NO)
@@ -455,8 +472,6 @@ void AttributeProxy::parse_name(string &full_name)
 		}
 		else
 		{
-			cerr << mod << " is a non supported database modifier!" << endl;
-
 			TangoSys_OMemStream desc;
 			desc << mod;
 			desc << " modifier is an unsupported db modifier" << ends;
@@ -473,7 +488,7 @@ void AttributeProxy::parse_name(string &full_name)
 
 	if (dbase_used == false)
 	{
-	
+
 //
 // Extract host name and port number
 //
@@ -481,19 +496,15 @@ void AttributeProxy::parse_name(string &full_name)
 		pos = name_wo_db_mod.find(HOST_SEP);
 		if (pos == string::npos)
 		{
-			cerr << "Host and port not correctly defined in device name " << full_name << endl;
-
 			ApiWrongNameExcept::throw_exception((const char*)"API_WrongAttributeNameSyntax",
 						(const char*)"Host and port not correctly defined in device name",
 						(const char*)"AttributeProxy::parse_name()");
 		}
-		
+
 		host = name_wo_db_mod.substr(0,pos);
 		string::size_type tmp = name_wo_db_mod.find(PORT_SEP);
 		if (tmp == string::npos)
 		{
-			cerr << "Host and port not correctly defined in device name " << full_name << endl;
-
 			ApiWrongNameExcept::throw_exception((const char*)"API_WrongAttributeNameSyntax",
 						(const char*)"Host and port not correctly defined in device name",
 						(const char*)"AttributeProxy::parse_name()");
@@ -503,14 +514,14 @@ void AttributeProxy::parse_name(string &full_name)
 		s << port << ends;
 		s >> port_num;
 		device_name = name_wo_db_mod.substr(tmp + 1);
-		
+
 		db_host = db_port = NOT_USED;
 		db_port_num = 0;
-				 
+
 	}
 	else
 	{
-	
+
 //
 // Search if host and port are specified
 //
@@ -560,15 +571,13 @@ void AttributeProxy::parse_name(string &full_name)
 	int n_sep = 0;
 	string device_name_tmp(device_name);
 	string::size_type device_name_end_pos=0;
-	do 
+	do
 	{
 		pos = device_name_tmp.find(DEVICE_SEP);
-		if (pos != string::npos) 
+		if (pos != string::npos)
 		{
 			if (pos == 0)
 			{
-				cerr << "Wrong attribute name syntax (domain/family/member/attr_name) in " << cased_name << endl;
-
 				ApiWrongNameExcept::throw_exception((const char*)"API_WrongAttributeNameSyntax",
 				(const char*)"Attribute name must have four fields separated by /'s or no /'s at all if it is an alias (e.g. my/device/name/an_attr or myalias)",
 				(const char*)"AttributeProxy::parse_name()");
@@ -577,21 +586,17 @@ void AttributeProxy::parse_name(string &full_name)
 			device_name_tmp = device_name_tmp.substr(pos+1);
 			if (device_name_tmp.size() == 0)
 			{
-				cerr << "Wrong attribute name syntax (domain/family/member/attr_name) in " << cased_name << endl;
-
 				ApiWrongNameExcept::throw_exception((const char*)"API_WrongAttributeNameSyntax",
 				(const char*)"Attribute name must have four fields separated by /'s or no /'s at all if it is an alias (e.g. my/device/name/an_attr or myalias)",
-				(const char*)"AttributeProxy::parse_name()");			
+				(const char*)"AttributeProxy::parse_name()");
 			}
 			device_name_end_pos += pos+1;
 		}
-	} 
+	}
 	while (pos != string::npos);
 
 	if ((n_sep > 1) && (n_sep != 3))
 	{
-		cerr << "Wrong attribute name syntax (domain/family/member/attr_name) in " << cased_name << endl;
-
 		ApiWrongNameExcept::throw_exception((const char*)"API_WrongAttributeNameSyntax",
 			(const char*)"Attribute name must have four fields separated by /'s or no /'s at all if it is an alias (e.g. my/device/name/an_attr or myalias)",
 			(const char*)"AttributeProxy::parse_name()");
@@ -601,14 +606,11 @@ void AttributeProxy::parse_name(string &full_name)
 // if this is an alias (no slashes in name) then get the device and attribute
 // name from the database
 //
-	
+
 	if (n_sep == 0)
 	{
 		if (dbase_used == false)
 		{
-			cerr << "Wrong attribute name syntax (domain/family/member/attr_name) in " << cased_name << endl;
-			cerr << "Rem: Alias are forbidden when not using database" << endl;
-			
 			ApiWrongNameExcept::throw_exception((const char *)"API_WrongAttributeNameSyntax",
 						(const char *)"Attribute alias is not supported when not using database",
 						(const char *)"AttributeProxy::parse_name()");
@@ -621,8 +623,6 @@ void AttributeProxy::parse_name(string &full_name)
 		pos = device_name.find(HOST_SEP);
 		if (pos != string::npos)
 		{
-			cerr << "Wrong alias name syntax in " << cased_name << " (: is not allowed in alias name)" << endl;
-
 			ApiWrongNameExcept::throw_exception((const char *)"API_WrongAttributeNameSyntax",
 			(const char *)"Wrong alias name (: not allowed in alias name)",
 			(const char *)"AttributeProxy::parse_name()");
@@ -631,13 +631,11 @@ void AttributeProxy::parse_name(string &full_name)
 		pos = device_name.find(RES_SEP);
 		if (pos != string::npos)
 		{
-			cerr << "Wrong alias name syntax in " << cased_name << " (-> is not allowed in alias name)" << endl;
-
 			ApiWrongNameExcept::throw_exception((const char *)"API_WrongAttributeNameSyntax",
 			(const char *)"Wrong alias name (-> not allowed in alias name)",
 			(const char *)"DeviceProxy::parse_name()");
 		}
-					
+
 //
 // Get full attribute name from database but connect to database first if it is not done already
 //
@@ -645,7 +643,7 @@ void AttributeProxy::parse_name(string &full_name)
 		ApiUtil *ui = ApiUtil::instance();
 		string db_attr_name;
 		if (from_env_var == true)
-		{	
+		{
 			if (ui->in_server() == true)
 			{
 				try
@@ -695,7 +693,7 @@ void AttributeProxy::parse_name(string &full_name)
 			int ind = ui->get_db_ind(db_host,db_port_num);
 			try
 			{
-				(ui->get_db_vect())[ind]->get_attribute_alias(device_name,db_attr_name);	
+				(ui->get_db_vect())[ind]->get_attribute_alias(device_name,db_attr_name);
 			}
 			catch (DevFailed &dfe)
 			{
@@ -711,23 +709,23 @@ void AttributeProxy::parse_name(string &full_name)
 				else
 					throw;
 			}
-		}		
+		}
 
 //
 // A fast syntax check on the full attribute name returned from the database
 //
 
 		string attr_name_tmp = db_attr_name;
-		do 
+		do
 		{
 			pos = attr_name_tmp.find(DEVICE_SEP);
-			if (pos != string::npos) 
+			if (pos != string::npos)
 			{
 				n_sep++;
 				attr_name_tmp = attr_name_tmp.substr(pos+1);
 				device_name_end_pos += pos+1;
 			}
-		} 
+		}
 		while (pos != string::npos);
 
 		if (n_sep != 3)
@@ -741,7 +739,7 @@ void AttributeProxy::parse_name(string &full_name)
 		device_name = db_attr_name.substr(0,device_name_end_pos - 1);
 	}
 
-	
+
 //
 // attribute name has four fields, separate them into device and attribute names
 // but keep attr_name as a case sentitive name
@@ -765,15 +763,15 @@ void AttributeProxy::parse_name(string &full_name)
 					(const char*)"AttributeProxy::parse_name()");
 
 			}
-			
+
 			if (from_env_var == false)
 			{
 				pos = name_wo_db_mod.rfind(DEVICE_SEP);
 				device_name = name_wo_db_mod.substr(0,pos);
 			}
 		}
-		
-		
+
+
 		pos = cased_name.rfind(DEVICE_SEP);
 		string::size_type pos_mod = cased_name.rfind(MODIFIER);
 		if (pos_mod != string::npos)
@@ -781,8 +779,8 @@ void AttributeProxy::parse_name(string &full_name)
 		else
 			attr_name = cased_name.substr(pos + 1);
 	}
-		
-		
+
+
 }
 
 //-----------------------------------------------------------------------------
@@ -795,10 +793,11 @@ AttributeProxy::~AttributeProxy()
 {
 	if (dbase_used == true)
 		delete db_attr;
-	if (dev_proxy != NULL)
-		delete dev_proxy;
-	if (ext != NULL)
-		delete ext;		
+    delete dev_proxy;
+
+#ifndef HAS_UNIQUE_PTR
+    delete ext;
+#endif
 }
 
 //-----------------------------------------------------------------------------
@@ -850,7 +849,7 @@ void AttributeProxy::set_transparency_reconnection(bool val)
 
 //-----------------------------------------------------------------------------
 //
-// AttributeProxy::get_transparency_reconnection() - Get underlying device 
+// AttributeProxy::get_transparency_reconnection() - Get underlying device
 // transparency reconnection flag
 //
 //-----------------------------------------------------------------------------
@@ -865,8 +864,8 @@ bool AttributeProxy::get_transparency_reconnection()
 // AttributeProxy::get_property() - get a property from the database
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::get_property(string &property_name, DbData &user_data) 
+
+void AttributeProxy::get_property(string &property_name, DbData &user_data)
 {
 	if (dbase_used == false)
 	{
@@ -874,7 +873,7 @@ void AttributeProxy::get_property(string &property_name, DbData &user_data)
 		desc << "Method not available for attribute belonging to ";
 		desc << device_name;
 		desc << " which is a non database device";
-		
+
 		ApiNonDbExcept::throw_exception((const char *)"API_NonDatabaseDevice",
 					desc.str(),
 					(const char *)"AttributeProxy::get_property");
@@ -886,7 +885,7 @@ void AttributeProxy::get_property(string &property_name, DbData &user_data)
 		db_data[0] = DbDatum(attr_name);
 
 		db_attr->get_property(db_data);
-		
+
 		long nb_prop = db_data.size();
 		int i;
 		for (i = 1;i < nb_prop;i++)
@@ -898,7 +897,7 @@ void AttributeProxy::get_property(string &property_name, DbData &user_data)
 				break;
 			}
 		}
-		
+
 		if (i == nb_prop)
 		{
 			user_data.resize(0);
@@ -907,7 +906,7 @@ void AttributeProxy::get_property(string &property_name, DbData &user_data)
 			no_data.value_string.resize(0);
 			user_data.push_back(no_data);
 		}
-		
+
 	}
 
 	return;
@@ -918,8 +917,8 @@ void AttributeProxy::get_property(string &property_name, DbData &user_data)
 // AttributeProxy::get_property() - get a property from the database
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::get_property(vector<string> &property_names, DbData &user_data) 
+
+void AttributeProxy::get_property(vector<string> &property_names, DbData &user_data)
 {
 	if (dbase_used == false)
 	{
@@ -927,7 +926,7 @@ void AttributeProxy::get_property(vector<string> &property_names, DbData &user_d
 		desc << "Method not available for attribute belonging to device ";
 		desc << device_name;
 		desc << " which is a non database device";
-		
+
 		ApiNonDbExcept::throw_exception((const char *)"API_NonDatabaseDevice",
 					desc.str(),
 					(const char *)"AttributeProxy::get_property");
@@ -939,10 +938,10 @@ void AttributeProxy::get_property(vector<string> &property_names, DbData &user_d
 		db_data.resize(1);
 		db_data[0] = DbDatum(attr_name);
 
-		db_attr->get_property(db_data);	
-	
+		db_attr->get_property(db_data);
+
 		int i,j;
-		user_data.resize(0);		
+		user_data.resize(0);
 		long nb_recev_prop = db_data.size();
 		for (i = 0;i < nb_prop;i++)
 		{
@@ -954,7 +953,7 @@ void AttributeProxy::get_property(vector<string> &property_names, DbData &user_d
 					break;
 				}
 			}
-		
+
 			if (j == nb_recev_prop)
 			{
 				DbDatum no_data;
@@ -962,7 +961,7 @@ void AttributeProxy::get_property(vector<string> &property_names, DbData &user_d
 				no_data.value_string.resize(0);
 				user_data.push_back(no_data);
 			}
-		}	
+		}
 	}
 
 	return;
@@ -973,8 +972,8 @@ void AttributeProxy::get_property(vector<string> &property_names, DbData &user_d
 // AttributeProxy::get_property() - get a property from the database
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::get_property(DbData &user_data) 
+
+void AttributeProxy::get_property(DbData &user_data)
 {
 	if (dbase_used == false)
 	{
@@ -982,7 +981,7 @@ void AttributeProxy::get_property(DbData &user_data)
 		desc << "Method not available for attribute belonging to device ";
 		desc << device_name;
 		desc << " which is a non database device";
-		
+
 		ApiNonDbExcept::throw_exception((const char *)"API_NonDatabaseDevice",
 					desc.str(),
 					(const char *)"AttributeProxy::get_property");
@@ -994,7 +993,7 @@ void AttributeProxy::get_property(DbData &user_data)
 		db_data.resize(1);
 		db_data[0] = DbDatum(attr_name);
 
-		db_attr->get_property(db_data);	
+		db_attr->get_property(db_data);
 
 		int i,j;
 		long nb_recev_prop = db_data.size();
@@ -1008,12 +1007,12 @@ void AttributeProxy::get_property(DbData &user_data)
 					break;
 				}
 			}
-		
+
 			if (j == nb_recev_prop)
 			{
 				user_data[i].value_string.resize(0);
 			}
-		}			
+		}
 	}
 
 	return;
@@ -1024,8 +1023,8 @@ void AttributeProxy::get_property(DbData &user_data)
 // AttributeProxy::put_property() - put a property from the database
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::put_property(DbData &user_data) 
+
+void AttributeProxy::put_property(DbData &user_data)
 {
 	if (dbase_used == false)
 	{
@@ -1033,7 +1032,7 @@ void AttributeProxy::put_property(DbData &user_data)
 		desc << "Method not available for attribute belonging to device ";
 		desc << device_name;
 		desc << " which is a non database device";
-		
+
 		ApiNonDbExcept::throw_exception((const char *)"API_NonDatabaseDevice",
 					desc.str(),
 					(const char *)"AttributeProxy::put_property");
@@ -1047,7 +1046,7 @@ void AttributeProxy::put_property(DbData &user_data)
 		db_data.push_back(att_name);
 		for (int i = 0;i < nb_prop;i++)
 			db_data.push_back(user_data[i]);
-		
+
 		db_attr->put_property(db_data);
 	}
 
@@ -1059,8 +1058,8 @@ void AttributeProxy::put_property(DbData &user_data)
 // AttributeProxy::delete_property() - delete a property from the database
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::delete_property(string &property_name) 
+
+void AttributeProxy::delete_property(string &property_name)
 {
 	if (dbase_used == false)
 	{
@@ -1068,7 +1067,7 @@ void AttributeProxy::delete_property(string &property_name)
 		desc << "Method not available for attribute belonging to device ";
 		desc << device_name;
 		desc << " which is a non database device";
-		
+
 		ApiNonDbExcept::throw_exception((const char *)"API_NonDatabaseDevice",
 					desc.str(),
 					(const char *)"AttributeProxy::delete_property");
@@ -1092,8 +1091,8 @@ void AttributeProxy::delete_property(string &property_name)
 // AttributeProxy::delete_property() - delete a property from the database
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::delete_property(vector<string> &property_names) 
+
+void AttributeProxy::delete_property(vector<string> &property_names)
 {
 	if (dbase_used == false)
 	{
@@ -1101,7 +1100,7 @@ void AttributeProxy::delete_property(vector<string> &property_names)
 		desc << "Method not available for attribute belonging to device ";
 		desc << device_name;
 		desc << " which is a non database device";
-		
+
 		ApiNonDbExcept::throw_exception((const char *)"API_NonDatabaseDevice",
 					desc.str(),
 					(const char *)"AttributeProxy::delete_property");
@@ -1116,7 +1115,7 @@ void AttributeProxy::delete_property(vector<string> &property_names)
 		{
 			db_data.push_back(DbDatum(property_names[i]));
 		}
-		
+
 		db_attr->delete_property(db_data);
 	}
 
@@ -1128,8 +1127,8 @@ void AttributeProxy::delete_property(vector<string> &property_names)
 // AttributeProxy::delete_property() - delete a property from the database
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::delete_property(DbData &user_data) 
+
+void AttributeProxy::delete_property(DbData &user_data)
 {
 	if (dbase_used == false)
 	{
@@ -1137,7 +1136,7 @@ void AttributeProxy::delete_property(DbData &user_data)
 		desc << "Method not available for attribute belonging to device ";
 		desc << device_name;
 		desc << " which is a non database device";
-		
+
 		ApiNonDbExcept::throw_exception((const char *)"API_NonDatabaseDevice",
 					desc.str(),
 					(const char *)"AttributeProxy::delete_property");
@@ -1152,7 +1151,7 @@ void AttributeProxy::delete_property(DbData &user_data)
 		{
 			db_data[i] = user_data[i];
 		}
-		
+
 		db_attr->delete_property(db_data);
 	}
 
@@ -1164,8 +1163,8 @@ void AttributeProxy::delete_property(DbData &user_data)
 // AttributeProxy::get_config() - return attribute config
 //
 //-----------------------------------------------------------------------------
- 
-AttributeInfoEx AttributeProxy::get_config() 
+
+AttributeInfoEx AttributeProxy::get_config()
 {
 	return (dev_proxy->get_attribute_config(attr_name));
 }
@@ -1175,13 +1174,13 @@ AttributeInfoEx AttributeProxy::get_config()
 // AttributeProxy::set_attribute_config() - set config for attribute
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::set_config(AttributeInfo &dev_attr_info) 
+
+void AttributeProxy::set_config(AttributeInfo &dev_attr_info)
 {
 	AttributeInfoList attr_info_list;
 
 	attr_info_list.push_back(dev_attr_info);
-	try 
+	try
 	{
 		dev_proxy->set_attribute_config(attr_info_list);
 
@@ -1197,12 +1196,12 @@ void AttributeProxy::set_config(AttributeInfo &dev_attr_info)
 	}
 }
 
-void AttributeProxy::set_config(AttributeInfoEx &dev_attr_info) 
+void AttributeProxy::set_config(AttributeInfoEx &dev_attr_info)
 {
 	AttributeInfoListEx attr_info_list;
 
 	attr_info_list.push_back(dev_attr_info);
-	try 
+	try
 	{
 		dev_proxy->set_attribute_config(attr_info_list);
 
@@ -1219,12 +1218,12 @@ void AttributeProxy::set_config(AttributeInfoEx &dev_attr_info)
 }
 //-----------------------------------------------------------------------------
 //
-// AttributeProxy::read() - read attribute 
+// AttributeProxy::read() - read attribute
 //
 //-----------------------------------------------------------------------------
- 
 
-DeviceAttribute AttributeProxy::read() 
+
+DeviceAttribute AttributeProxy::read()
 {
 	return(dev_proxy->read_attribute(attr_name));
 }
@@ -1234,8 +1233,8 @@ DeviceAttribute AttributeProxy::read()
 // AttributeProxy::write() - write attribute
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::write(DeviceAttribute& attr_value) 
+
+void AttributeProxy::write(DeviceAttribute& attr_value)
 {
 	attr_value.set_name(attr_name);
 	dev_proxy->write_attribute(attr_value);
@@ -1246,8 +1245,8 @@ void AttributeProxy::write(DeviceAttribute& attr_value)
 // AttributeProxy::write_read() - write then read attribute
 //
 //-----------------------------------------------------------------------------
- 
-DeviceAttribute AttributeProxy::write_read(DeviceAttribute& attr_value) 
+
+DeviceAttribute AttributeProxy::write_read(DeviceAttribute& attr_value)
 {
 	attr_value.set_name(attr_name);
 	return dev_proxy->write_read_attribute(attr_value);
@@ -1259,8 +1258,8 @@ DeviceAttribute AttributeProxy::write_read(DeviceAttribute& attr_value)
 //				      (only for polled attribute)
 //
 //-----------------------------------------------------------------------------
- 
-vector<DeviceAttributeHistory> *AttributeProxy::history(int depth) 
+
+vector<DeviceAttributeHistory> *AttributeProxy::history(int depth)
 {
 
 	return(dev_proxy->attribute_history(attr_name, depth));
@@ -1273,8 +1272,8 @@ vector<DeviceAttributeHistory> *AttributeProxy::history(int depth)
 //					    (in mS)
 //
 //-----------------------------------------------------------------------------
- 
-int AttributeProxy::get_poll_period() 
+
+int AttributeProxy::get_poll_period()
 {
 	return(dev_proxy->get_attribute_poll_period(attr_name));
 }
@@ -1286,8 +1285,8 @@ int AttributeProxy::get_poll_period()
 //			    it to the list of polled objects
 //
 //-----------------------------------------------------------------------------
- 
-void AttributeProxy::poll(int period) 
+
+void AttributeProxy::poll(int period)
 {
 	dev_proxy->poll_attribute(attr_name, period);
 }
@@ -1322,12 +1321,17 @@ void AttributeProxy::stop_poll()
 //
 //-----------------------------------------------------------------------------
 
-int AttributeProxy::subscribe_event (EventType event, CallBack *callback, 
+int AttributeProxy::subscribe_event (EventType event, CallBack *callback,
                                     const vector<string> &filters)
 {
 	return subscribe_event (event, callback, filters, false);
 }
 
+int AttributeProxy::subscribe_event (EventType event, CallBack *callback)
+{
+    vector<string> filters;
+	return subscribe_event (event, callback, filters, false);
+}
 
 //-----------------------------------------------------------------------------
 //
@@ -1337,19 +1341,44 @@ int AttributeProxy::subscribe_event (EventType event, CallBack *callback,
 //
 //-----------------------------------------------------------------------------
 
-int AttributeProxy::subscribe_event (EventType event, CallBack *callback, 
+int AttributeProxy::subscribe_event (EventType event, CallBack *callback,
                                     const vector<string> &filters, bool stateless)
 {
-  	if (ApiUtil::instance()->get_event_consumer() == NULL)
+    ApiUtil *api_ptr = ApiUtil::instance();
+    if (api_ptr->get_zmq_event_consumer() == NULL)
 	{
-		ApiUtil::instance()->create_event_consumer();
+		api_ptr->create_zmq_event_consumer();
 	}
 
-	//ApiUtil::instance()->get_event_consumer()->connect(dev_proxy);
 
-	
-	return ApiUtil::instance()->get_event_consumer()->subscribe_event(dev_proxy, 
-	                           attr_name, event, callback, filters, stateless);
+	int ret;
+	try
+	{
+        ret = api_ptr->get_zmq_event_consumer()->subscribe_event(dev_proxy,attr_name, event, callback, filters, stateless);
+	}
+	catch (DevFailed &e)
+	{
+	    string reason(e.errors[0].reason.in());
+	    if (reason == "API_CommandNotFound")
+	    {
+            if (ApiUtil::instance()->get_notifd_event_consumer() == NULL)
+            {
+                ApiUtil::instance()->create_notifd_event_consumer();
+            }
+
+            ret = api_ptr->get_notifd_event_consumer()->subscribe_event(dev_proxy,attr_name, event, callback, filters, stateless);
+	    }
+	    else
+            throw;
+	}
+
+	return ret;
+}
+
+int AttributeProxy::subscribe_event (EventType event, CallBack *callback,bool stateless)
+{
+    vector<string> vs;
+    return subscribe_event(event,callback,vs,stateless);
 }
 
 //-----------------------------------------------------------------------------
@@ -1360,18 +1389,41 @@ int AttributeProxy::subscribe_event (EventType event, CallBack *callback,
 //
 //-----------------------------------------------------------------------------
 
-int AttributeProxy::subscribe_event (EventType event, int event_queue_size, 
+int AttributeProxy::subscribe_event (EventType event, int event_queue_size,
                                     const vector<string> &filters, bool stateless)
 {
-	if (ApiUtil::instance()->get_event_consumer() == NULL)
+    ApiUtil *api_ptr = ApiUtil::instance();
+	if (api_ptr->get_zmq_event_consumer() == NULL)
 	{
-		ApiUtil::instance()->create_event_consumer();
+		api_ptr->create_zmq_event_consumer();
 	}
-	
-	return ApiUtil::instance()->get_event_consumer()->subscribe_event(dev_proxy, 
-	                           attr_name, event, event_queue_size, filters, stateless);
+
+    int ret;
+    try
+    {
+        ret = api_ptr->get_zmq_event_consumer()->subscribe_event(dev_proxy,attr_name, event, event_queue_size, filters, stateless);
+    }
+    catch (DevFailed &e)
+    {
+        string reason(e.errors[0].reason.in());
+        if (reason == "API_CommandNotFound")
+        {
+            if (api_ptr->get_notifd_event_consumer() == NULL)
+            {
+                api_ptr->create_notifd_event_consumer();
+            }
+            ret = api_ptr->get_notifd_event_consumer()->subscribe_event(dev_proxy,attr_name, event, event_queue_size, filters, stateless);
+        }
+        else
+            throw;
+    }
+	return ret;
 }
 
-
+int AttributeProxy::subscribe_event (EventType event, int event_queue_size,bool stateless)
+{
+    vector<string> vs;
+    return subscribe_event(event,event_queue_size,vs,stateless);
+}
 
 } // End of Tango namespace
