@@ -1,8 +1,8 @@
 //
 // dbapi.h -	include file for TANGO database api
 //
-// 
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011
+//
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -13,12 +13,12 @@
 // it under the terms of the GNU Lesser General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // Tango is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -46,11 +46,6 @@ class DbDevExportInfo;
 class DbServerInfo;
 class DbHistory;
 
-class DatabaseExt;
-class DbDeviceExt;
-class DbClassExt;
-class DbServerExt;
-class DbDatumExt;
 class FileDatabase;
 class DbServerCache;
 class Util;
@@ -60,7 +55,7 @@ typedef vector<DbDatum> DbData;
 typedef vector<DbDevInfo> DbDevInfos;
 typedef vector<DbDevExportInfo> DbDevExportInfos;
 typedef vector<DbDevImportInfo> DbDevImportInfos;
- 
+
 #define		POGO_DESC	"Description"
 #define		POGO_TITLE	"ProjectTitle"
 
@@ -69,56 +64,71 @@ typedef vector<DbDevImportInfo> DbDevImportInfos;
 //                 interface for TANGO database api
 //
 
-class DatabaseExt
-{
-public:
-	DatabaseExt():db_tg(NULL) {};
-	
-	Tango::Util 	*db_tg;
-	omni_mutex		map_mutex;
-};
+/****************************************************************************************
+ * 																						*
+ * 					The Database class													*
+ * 					------------------													*
+ * 																						*
+ ***************************************************************************************/
 
 class Database : public Tango::Connection
 {
-private :	
+private :
 	virtual string get_corba_name(bool);
 	virtual string build_corba_name() {return string("nada");}
 	virtual int get_lock_ctr() {return 0;}
 	virtual void set_lock_ctr(int) {}
-		
+
+    class DatabaseExt
+    {
+    public:
+        DatabaseExt():db_tg(NULL) {};
+
+        Tango::Util 	*db_tg;
+        omni_mutex		map_mutex;
+    };
+
+#ifdef HAS_UNIQUE_PTR
+    unique_ptr<DatabaseExt>     ext;
+#else
+	DatabaseExt			        *ext;
+#endif
+
 	bool				db_multi_svc;
 	vector<string>		multi_db_port;
 	vector<string>		multi_db_host;
-	DatabaseExt			*ext;
-	FileDatabase 		*filedb; 
+	FileDatabase 		*filedb;
 	string 				file_name;
 	int					serv_version;
-	
+
 	AccessProxy			*access_proxy;
 	bool				access_checked;
 	DevErrorList		access_except_errors;
-	
+
 	map<string,string>	dev_class_cache;
 	string				db_device_name;
-	
+
 	bool				access_service_defined;
-	
+
 	DbDatum         make_string_array(string, CORBA::Any_var &);
 	vector<DbHistory> make_history_array(bool, CORBA::Any_var &);
-	
+
 	void check_access();
 	inline string dev_name();
-	void get_server_release();
+	void set_server_release();
 	void check_access_and_get();
-	
+
 public :
 	Database(CORBA::ORB *orb=NULL);
 	Database(string &host, int port, CORBA::ORB *orb=NULL);
 	Database(string &file);
-	
+
+	Database(const Database &);
+	Database & operator=(const Database &);
+
 	void write_filedatabase();
 	void reread_filedatabase();
-	void write_event_channel_ior_filedatabase(string &,string &);
+	void write_event_channel_ior_filedatabase(string &);
 	void build_connection ();
 	void post_reconnection();
 	~Database();
@@ -127,13 +137,14 @@ public :
 	AccessControlType check_access_control(string &);
 	bool is_control_access_checked() {return access_checked;}
 	void set_access_checked(bool val) {access_checked = val;}
-	
+
 	void set_tango_utils(Tango::Util *ptr) {ext->db_tg=ptr;}
-	
+	int get_server_release() {return serv_version;}
+
 	DevErrorList &get_access_except_errors() {return access_except_errors;}
 	void clear_access_except_errors() {access_except_errors.length(0);}
 	bool is_command_allowed(string &,string &);
-	
+
 	bool is_multi_tango_host() {return db_multi_svc;}
 	vector<string> &get_multi_host() {return multi_db_host;}
 	vector<string> &get_multi_port() {return multi_db_port;}
@@ -156,7 +167,7 @@ public :
 	void register_service(string &,string &,string &);
 	void unregister_service(string &,string &);
 	CORBA::Any *fill_server_cache(string &,string &);
-	
+
 //
 // device methods
 //
@@ -181,7 +192,7 @@ public :
 	DbDatum get_device_exported_for_class(string &);
 	void put_device_alias(string &,string &);
 	void delete_device_alias(string &);
-	
+
 //
 // server methods
 //
@@ -189,7 +200,7 @@ public :
 	void delete_server(string &);
 	void export_server(DbDevExportInfos &);
 	void unexport_server(string &);
-	
+
 	DbServerInfo get_server_info(string &);
 	void put_server_info(DbServerInfo &);
 	void delete_server_info(string &);
@@ -200,15 +211,15 @@ public :
 	DbDatum get_server_list(string &);
 	DbDatum get_host_server_list(string &);
 	DbDatum get_device_class_list(string &);
-	
+
 //
 // property methods
 //
 
 	void get_property(string, DbData &,DbServerCache *dsc);
-	void get_property(string st, DbData &db) {get_property(st,db,NULL);}	
-	void get_property_forced(string, DbData &,DbServerCache *dsc = NULL);  
-	void put_property(string, DbData &);  
+	void get_property(string st, DbData &db) {get_property(st,db,NULL);}
+	void get_property_forced(string, DbData &,DbServerCache *dsc = NULL);
+	void put_property(string, DbData &);
 	void delete_property(string, DbData &);
 	vector<DbHistory> get_property_history(string &,string &);
 	DbDatum get_object_list(string &);
@@ -221,14 +232,14 @@ public :
 	vector<DbHistory> get_device_property_history(string &,string &);
 	DbDatum get_device_property_list(string &,string &);
 	void get_device_property_list(string &,const string &,vector<string> &,DbServerCache *dsc = NULL);
-	
+
 	void get_device_attribute_property(string, DbData &, DbServerCache *dsc);
 	void get_device_attribute_property(string st, DbData &db) {get_device_attribute_property(st,db,NULL);}
 	void put_device_attribute_property(string, DbData &);
 	void delete_device_attribute_property(string, DbData &);
 	void delete_all_device_attribute_property(string, DbData &);
 	vector<DbHistory> get_device_attribute_property_history(string &,string &,string &);
-	
+
 	void get_class_property(string, DbData &, DbServerCache *dsc);
 	void get_class_property(string st,DbData &db) {get_class_property(st,db,NULL);}
 	void put_class_property(string, DbData &);
@@ -236,14 +247,14 @@ public :
 	vector<DbHistory> get_class_property_history(string &,string &);
 	DbDatum get_class_list(string &);
 	DbDatum get_class_property_list(string &);
-	
+
 	void get_class_attribute_property(string, DbData &, DbServerCache *dsc);
 	void get_class_attribute_property(string st,DbData &db) {get_class_attribute_property(st,db,NULL);}
 	void put_class_attribute_property(string, DbData &);
 	void delete_class_attribute_property(string, DbData &);
 	vector<DbHistory> get_class_attribute_property_history(string &,string &,string &);
 	DbDatum get_class_attribute_list(string &,string &);
-	
+
 
 // attribute methods
 
@@ -251,7 +262,7 @@ public :
 	DbDatum get_attribute_alias_list(string &);
 	void put_attribute_alias(string &,string &);
 	void delete_attribute_alias(string &);
-	
+
 // event methods
 
 	void export_event(DevVarStringArray *);
@@ -265,7 +276,7 @@ public :
 //
 
 
-inline string Database::dev_name() 
+inline string Database::dev_name()
 {
 	if (db_device_name.empty() == true)
 	{
@@ -274,8 +285,8 @@ inline string Database::dev_name()
 	}
 	return db_device_name;
 }
-	
-	
+
+
 //
 // Some macros to call the Db server
 // These macros will do some retries in case of
@@ -309,7 +320,7 @@ inline string Database::dev_name()
 		else \
 			throw; \
 	}
-		
+
 #define CALL_DB_SERVER_NO_RET(NAME,SEND) \
 	{ \
 		bool retry_mac = true; \
@@ -329,7 +340,7 @@ inline string Database::dev_name()
 			MANAGE_EXCEPT(NAME) \
 		} \
 	}
-	
+
 #define CALL_DB_SERVER(NAME,SEND,RET) \
 	{ \
 		bool retry_mac = true; \
@@ -349,7 +360,7 @@ inline string Database::dev_name()
 			MANAGE_EXCEPT(NAME) \
 		} \
 	}
-		
+
 //
 // DbProperty - a database object for accessing general properties which
 //                  are stored in the database
@@ -379,8 +390,18 @@ private :
 	Database 	*dbase;
 	int 		db_ind;
 	bool 		ext_dbase;
-	
-	DbDeviceExt	*ext;
+
+    class DbDeviceExt
+    {
+    public:
+        DbDeviceExt() {};
+    };
+
+#ifdef HAS_UNIQUE_PTR
+    unique_ptr<DbDeviceExt>     ext;
+#else
+	DbDeviceExt	                *ext;
+#endif
 
 public :
 	DbDevice(string &);
@@ -445,7 +466,18 @@ private :
 	Database 	*dbase;
 	int 		db_ind;
 	bool 		ext_dbase;
-	DbServerExt	*ext;
+
+    class DbServerExt
+    {
+    public:
+        DbServerExt() {};
+    };
+
+#ifdef HAS_UNIQUE_PTR
+    unique_ptr<DbServerExt> ext;
+#else
+	DbServerExt	            *ext;
+#endif
 
 public :
 	DbServer(string);
@@ -471,8 +503,18 @@ private :
 	Database 	*dbase;
 	int 		db_ind;
 	bool 		ext_dbase;
-	
-	DbClassExt	*ext;
+
+    class DbClassExt
+    {
+    public:
+        DbClassExt() {};
+    };
+
+#ifdef HAS_UNIQUE_PTR
+    unique_ptr<DbClassExt>  ext;
+#else
+	DbClassExt	            *ext;
+#endif
 
 public :
 	DbClass(string, Database*);
@@ -501,7 +543,7 @@ public :
 		wrongtype_flag,
 		numFlags
 	};
-	
+
 	string name;
 	vector<string> value_string;
 //
@@ -514,14 +556,14 @@ public :
 	DbDatum(const DbDatum &);
 	DbDatum &operator=(const DbDatum &);
 
-	int size() {return value_string.size();}
+	size_t size() {return value_string.size();}
 	bool is_empty();
-	
+
 	void exceptions(bitset<numFlags> fl) { exceptions_flags = fl;}
 	bitset<numFlags> exceptions() {return exceptions_flags;}
 	void reset_exceptions(except_flags fl) {exceptions_flags.reset((size_t)fl);}
 	void set_exceptions(except_flags fl) {exceptions_flags.set((size_t)fl);}
-		
+
 //
 // insert methods
 //
@@ -541,7 +583,7 @@ public :
 	void operator << (const char *);
 //	void operator << (const char *&);
 	void operator << (string&);
-	
+
 	void operator << (vector<string>&);
 	void operator << (vector<short>&);
 	void operator << (vector<unsigned short>&);
@@ -551,7 +593,7 @@ public :
 	void operator << (vector<DevULong64>&);
 	void operator << (vector<float>&);
 	void operator << (vector<double>&);
-	
+
 //
 // extract methods
 //
@@ -568,7 +610,7 @@ public :
 	bool operator >> (double&);
 	bool operator >> (const char*&);
 	bool operator >> (string&);
-	
+
 	bool operator >> (vector<string>&);
 	bool operator >> (vector<short>&);
 	bool operator >> (vector<unsigned short>&);
@@ -584,8 +626,18 @@ private :
 	int 				value_type;
 	int 				value_size;
 	bitset<numFlags> 	exceptions_flags;
-	
-	DbDatumExt			*ext;
+
+    class DbDatumExt
+    {
+    public:
+        DbDatumExt() {};
+    };
+
+#ifdef HAS_UNIQUE_PTR
+    unique_ptr<DbDatumExt>  ext;
+#else
+	DbDatumExt			    *ext;
+#endif
 };
 //
 // DbHistory data object for receiving data history from the
@@ -611,16 +663,16 @@ public:
   string get_date();
   DbDatum get_value();
   bool is_deleted();
-  
+
 private:
-  
+
   string  propname;   // Property name
   string  attname;    // Attribute name (Not used for device properties)
-  DbDatum value;      // Property value  
+  DbDatum value;      // Property value
   string  date;       // Update date
   bool    deleted;    // Deleted flag
-  
-  string format_mysql_date(string );  
+
+  string format_mysql_date(string );
   void make_db_datum(vector<string> &);
 };
 //
@@ -669,6 +721,14 @@ public :
 	int level;
 };
 
+
+/****************************************************************************************
+ * 																						*
+ * 					The DbServerCache class												*
+ * 					------------------													*
+ * 																						*
+ ***************************************************************************************/
+
 //
 // DbServerCache data object to implement a DB cache
 // used during the DS startup sequence
@@ -690,7 +750,7 @@ public:
 		int				prop_nb;
 		int 			*props_idx;
 	}PropEltIdx;
-	
+
 	typedef struct
 	{
 		int 			first_idx;
@@ -698,13 +758,13 @@ public:
 		int				att_nb;
 		int				*atts_idx;
 	}AttPropEltIdx;
-	
+
 	typedef struct
 	{
 		PropEltIdx		dev_prop;
 		AttPropEltIdx 	dev_att_prop;
 	}DevEltIdx;
-	
+
 	typedef struct
 	{
 		PropEltIdx 		class_prop;
@@ -713,12 +773,12 @@ public:
 		int 			dev_nb;
 		DevEltIdx		*devs_idx;
 	}ClassEltIdx;
-	
+
 	DbServerCache(Database *,string &,string &);
 	~DbServerCache();
-	
+
 	const DevVarLongStringArray *import_adm_dev();
-	const DevVarLongStringArray *import_notifd_event();	
+	const DevVarLongStringArray *import_notifd_event();
 	const DevVarLongStringArray *import_adm_event();
 	const DevVarStringArray *get_class_property(DevVarStringArray *);
 	const DevVarStringArray *get_dev_property(DevVarStringArray *);
@@ -727,7 +787,8 @@ public:
 	const DevVarStringArray *get_dev_att_property(DevVarStringArray *);
 	const DevVarStringArray *get_obj_property(DevVarStringArray *);
 	const DevVarStringArray *get_device_property_list(DevVarStringArray *);
-	
+	const DevVarLongStringArray *import_tac_dev(string &);
+
 	const EltIdx &get_imp_dat() {return imp_adm;}
 	const EltIdx &get_imp_notifd_event() {return imp_notifd_event;}
 	const EltIdx &get_imp_adm_event() {return imp_adm_event;}
@@ -738,7 +799,7 @@ public:
 	int get_class_nb() {return class_nb;}
 	const ClassEltIdx *get_classes_elt() {return classes_idx;}
 	int get_data_nb() {return n_data;}
-	
+
 private:
 	void prop_indexes(int &,int &,PropEltIdx &,const DevVarStringArray *);
 	void prop_att_indexes(int &,int &,AttPropEltIdx &,const DevVarStringArray *);
@@ -747,58 +808,32 @@ private:
 	int find_dev_att(DevString,int &,int &);
 	int find_obj(DevString obj_name,int &);
 	void get_obj_prop_list(DevVarStringArray *,PropEltIdx &);
-	
+
 	CORBA::Any_var			received;
 	const DevVarStringArray *data_list;
 	int 					n_data;
-	
+
 	EltIdx					imp_adm;
 	EltIdx					imp_notifd_event;
 	EltIdx					imp_adm_event;
+	EltIdx                  imp_tac;
 	PropEltIdx				ctrl_serv_prop;
 	PropEltIdx				DServer_class_prop;
 	PropEltIdx				Default_prop;
 	PropEltIdx				adm_dev_prop;
 	int 					class_nb;
 	ClassEltIdx				*classes_idx;
-	
+
 	DevVarLongStringArray	imp_adm_data;
 	DevVarLongStringArray	imp_notifd_event_data;
 	DevVarLongStringArray	imp_adm_event_data;
+	DevVarLongStringArray	imp_tac_data;
 	DevVarStringArray		ret_obj_prop;
 	DevVarStringArray		ret_dev_list;
 	DevVarStringArray		ret_obj_att_prop;
 	DevVarStringArray		ret_prop_list;
 };
 
-//
-// Some extension classes
-//
-
-class DbDeviceExt
-{
-public:
-	DbDeviceExt() {};
-};
-
-class DbClassExt
-{
-public:
-	DbClassExt() {};
-};
-
-class DbServerExt
-{
-public:
-	DbServerExt() {};
-};
-
-class DbDatumExt
-{
-public:
-	DbDatumExt() {};
-};
-
 } // End of Tango namespace
 
 #endif /* _DBAPI_H */
