@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_cache.cpp 15556 2011-02-11 08:25:58Z taurel $";
+static const char *RcsId = "$Id: dbapi_cache.cpp 20500 2012-06-05 08:50:39Z taurel $";
 
 //+============================================================================
 //
@@ -10,7 +10,7 @@ static const char *RcsId = "$Id: dbapi_cache.cpp 15556 2011-02-11 08:25:58Z taur
 //
 // author(s) :		E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -21,65 +21,16 @@ static const char *RcsId = "$Id: dbapi_cache.cpp 15556 2011-02-11 08:25:58Z taur
 // it under the terms of the GNU Lesser General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // Tango is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 15556 $
-//
-// $Log$
-// Revision 3.14  2010/09/09 13:43:38  taurel
-// - Add year 2010 in Copyright notice
-//
-// Revision 3.13  2009/03/30 15:02:58  taurel
-// - Fix last bugs before Tango 7 ??
-//
-// Revision 3.12  2009/03/13 09:32:27  taurel
-// - Small changes to fix Windows VC8 warnings in Warning level 3
-//
-// Revision 3.11  2009/02/23 14:27:53  taurel
-// - Added a DeviceProxy::get_property_list() method
-//
-// Revision 3.10  2009/01/21 12:45:15  taurel
-// - Change CopyRights for 2009
-//
-// Revision 3.9  2008/10/06 15:02:16  taurel
-// - Changed the licensing info from GPL to LGPL
-//
-// Revision 3.8  2008/10/02 16:09:25  taurel
-// - Add some licensing information in each files...
-//
-// Revision 3.7  2008/06/14 11:28:07  taurel
-// - DevEncoded attribute data type implementation work going on
-//
-// Revision 3.6  2008/05/20 12:42:29  taurel
-// - Commit after merge with release 7 branch
-//
-// Revision 3.5  2008/03/11 14:36:44  taurel
-// - Apply patches from Frederic Picca about compilation with gcc 4.2
-// Revision 3.2.2.1  2008/02/07 15:56:58  taurel
-// - First implementation of the Controlled Access done
-//
-// Revision 3.4  2008/01/25 15:45:58  taurel
-// - Some changes in the Db cache
-// - A lighter system to shutdown DS in case of dynamic attribute
-//
-// Revision 3.3  2008/01/08 14:33:52  taurel
-// - strcasecmp() is not named like this on Windows !!
-//
-// Revision 3.2  2007/10/17 13:42:04  taurel
-// - Bug found in Db server cache
-//
-// Revision 3.1  2007/10/16 08:25:47  taurel
-// - Add management of the TC connection establishment timeout for DB access
-// - Add DB server cache in DS used during DS startup sequence
-// - Comment out the sleep time during DS startup sequence
-//
+// $Revision: 20500 $
 //
 //-============================================================================
 
@@ -88,7 +39,7 @@ static const char *RcsId = "$Id: dbapi_cache.cpp 15556 2011-02-11 08:25:58Z taur
 #endif
 
 #include <tango.h>
-                                                  
+
 using namespace CORBA;
 
 namespace Tango
@@ -109,7 +60,7 @@ namespace Tango
 
 DbServerCache::DbServerCache(Database *db,string &ds_name,string &host)
 {
-	
+
 //
 // Get all data from database
 //
@@ -123,10 +74,10 @@ DbServerCache::DbServerCache(Database *db,string &ds_name,string &host)
 		cout3 << "Got an exception while getting cache data from database !!" << endl;
 		throw;
 	}
-	
+
 	received.inout() >>= data_list;
 	n_data = data_list->length();
-	
+
 //
 // Extract the different blocks from the big list
 // First, the device server admin device parameters
@@ -134,12 +85,17 @@ DbServerCache::DbServerCache(Database *db,string &ds_name,string &host)
 
 	int start_idx = 0;
 	int stop_idx = 0;
-	
+
 	if (n_data == 2)
 	{
 		imp_adm.first_idx = start_idx;
 		imp_adm.last_idx = 1;
-		
+
+		ctrl_serv_prop.first_idx = -1;
+		DServer_class_prop.first_idx = -1;
+		Default_prop.first_idx = -1;
+		adm_dev_prop.first_idx = -1;
+
 		return;
 	}
 
@@ -150,7 +106,7 @@ DbServerCache::DbServerCache(Database *db,string &ds_name,string &host)
 //
 // Event factory
 //
-	
+
 	start_idx = stop_idx + 1;
 	if (::strcmp((*data_list)[start_idx + 1],"Not Found") == 0)
 	{
@@ -162,11 +118,11 @@ DbServerCache::DbServerCache(Database *db,string &ds_name,string &host)
 	}
 	imp_notifd_event.first_idx = start_idx;
 	imp_notifd_event.last_idx = stop_idx;
-	
+
 //
 // Server event
 //
-	
+
 	start_idx = stop_idx + 1;
 	if (::strcmp((*data_list)[start_idx + 1],"Not Found") == 0)
 	{
@@ -178,29 +134,29 @@ DbServerCache::DbServerCache(Database *db,string &ds_name,string &host)
 	}
 	imp_adm_event.first_idx = start_idx;
 	imp_adm_event.last_idx = stop_idx;
-	
+
 //
 // DServer class prop
 //
 
 	prop_indexes(start_idx,stop_idx,DServer_class_prop,data_list);
-	
+
 //
 // Default prop
 //
-	
+
 	prop_indexes(start_idx,stop_idx,Default_prop,data_list);
-	
+
 //
 // Admin device prop
 //
-	
+
 	prop_indexes(start_idx,stop_idx,adm_dev_prop,data_list);
-	
+
 //
 // Embedded classes number
 //
-	
+
 	start_idx = stop_idx + 1;
 	class_nb = ::atoi((*data_list)[start_idx + 1]);
 	stop_idx = stop_idx  + 2;
@@ -212,51 +168,67 @@ DbServerCache::DbServerCache(Database *db,string &ds_name,string &host)
 //
 // Embedded class prop
 //
-	
+
 		prop_indexes(start_idx,stop_idx,classes_idx[cl_loop].class_prop,data_list);
-		
+
 //
 // Embedded class attribute prop
 //
-	
+
 		prop_att_indexes(start_idx,stop_idx,classes_idx[cl_loop].class_att_prop,data_list);
 
 //
 // Device list
 //
-	
+
 		start_idx = stop_idx + 1;
 		int nb_dev = ::atoi((*data_list)[start_idx + 1]);
 		stop_idx = stop_idx + nb_dev + 2;
-		
+
 		classes_idx[cl_loop].dev_list.first_idx = start_idx;
 		classes_idx[cl_loop].dev_list.last_idx = stop_idx;
 		classes_idx[cl_loop].dev_nb = nb_dev;
 		classes_idx[cl_loop].devs_idx = new DevEltIdx [nb_dev];
-	
+
 		for (int loop = 0;loop < nb_dev;loop++)
 		{
-	
+
 //
 // Device properties
 //
-			
+
 			prop_indexes(start_idx,stop_idx,classes_idx[cl_loop].devs_idx[loop].dev_prop,data_list);
-			
+
 //
 // Device attribute properties
 //
-		
+
 			prop_att_indexes(start_idx,stop_idx,classes_idx[cl_loop].devs_idx[loop].dev_att_prop,data_list);
 		}
 	}
-	
+
 //
 // Control System Service(s) property
 //
 
 	prop_indexes(start_idx,stop_idx,ctrl_serv_prop,data_list);
-	
+
+//
+// Tac device import info
+//
+
+    start_idx = stop_idx + 1;
+    if (stop_idx == n_data)
+    {
+        stop_idx = -1;
+    }
+    else
+    {
+        stop_idx = n_data;
+        imp_tac.first_idx = start_idx;
+        imp_tac.last_idx = stop_idx;
+    }
+
 }
 
 //-----------------------------------------------------------------------------
@@ -277,16 +249,16 @@ const DevVarLongStringArray *DbServerCache::import_adm_dev()
 	{
 		Tango::Except::throw_exception("aaa","bbb","ccc");
 	}
-	
+
 	imp_adm_data.lvalue.length(2);
 	imp_adm_data.svalue.length(5);
-	
+
 	imp_adm_data.lvalue[0] = ::atoi((*data_list)[imp_adm.first_idx + 5]);
 	imp_adm_data.lvalue[1] = ::atoi((*data_list)[imp_adm.last_idx]);
 
 	for (int loop = 0;loop < 5;loop++)
 		imp_adm_data.svalue[loop] = CORBA::string_dup((*data_list)[imp_adm.first_idx + loop]);
-	
+
 	return &imp_adm_data;
 }
 
@@ -295,7 +267,7 @@ const DevVarLongStringArray *DbServerCache::import_adm_dev()
 //  DbServerCache::import_notifd_event()
 //
 //	This method returns to the caller of the info necessary to import the
-//	event associated to the notifd running on the DS host. 
+//	event associated to the notifd running on the DS host.
 //	The returned data are the same than the one returned by
 //  the classical API
 //
@@ -309,16 +281,16 @@ const DevVarLongStringArray *DbServerCache::import_notifd_event()
 	{
 		Tango::Except::throw_exception("aaa","bbb","ccc");
 	}
-	
+
 	imp_notifd_event_data.lvalue.length(2);
 	imp_notifd_event_data.svalue.length(5);
-	
+
 	imp_notifd_event_data.lvalue[0] = ::atoi((*data_list)[imp_notifd_event.first_idx + 4]);
 	imp_notifd_event_data.lvalue[1] = ::atoi((*data_list)[imp_notifd_event.last_idx]);
 
 	for (int loop = 0;loop < 4;loop++)
 		imp_notifd_event_data.svalue[loop] = CORBA::string_dup((*data_list)[imp_notifd_event.first_idx + loop]);
-	
+
 	return &imp_notifd_event_data;
 }
 
@@ -327,7 +299,7 @@ const DevVarLongStringArray *DbServerCache::import_notifd_event()
 //  DbServerCache::import_adm_event()
 //
 //	This method returns to the caller of the info necessary to import the
-//	event associated to the DS server. 
+//	event associated to the DS server.
 //	The returned data are the same than the one returned by
 //  the classical API
 //
@@ -342,16 +314,16 @@ const DevVarLongStringArray *DbServerCache::import_adm_event()
 	{
 		Tango::Except::throw_exception("aaa","bbb","ccc");
 	}
-	
+
 	imp_adm_event_data.lvalue.length(2);
 	imp_adm_event_data.svalue.length(5);
-	
+
 	imp_adm_event_data.lvalue[0] = ::atoi((*data_list)[imp_adm_event.first_idx + 4]);
 	imp_adm_event_data.lvalue[1] = ::atoi((*data_list)[imp_notifd_event.last_idx]);
 
 	for (int loop = 0;loop < 4;loop++)
 		imp_adm_event_data.svalue[loop] = CORBA::string_dup((*data_list)[imp_adm_event.first_idx + loop]);
-	
+
 	return &imp_adm_event_data;
 }
 
@@ -374,29 +346,29 @@ const DevVarLongStringArray *DbServerCache::import_adm_event()
 const DevVarStringArray *DbServerCache::get_class_property(DevVarStringArray *in_param)
 {
 	int ret_length = 2;
-	
+
 	if (TG_strcasecmp((*in_param)[0],"DServer") == 0)
 	{
 		ret_obj_prop.length(ret_length);
 		ret_obj_prop[0] = CORBA::string_dup("DServer");
-		
+
 		get_obj_prop(in_param,DServer_class_prop);
 	}
 	else if (TG_strcasecmp((*in_param)[0],"Default") == 0)
 	{
 		ret_obj_prop.length(ret_length);
 		ret_obj_prop[0] = CORBA::string_dup("Default");
-		
+
 		get_obj_prop(in_param,Default_prop);
 	}
 	else
 	{
 		ret_obj_prop.length(ret_length);
 		ret_obj_prop[0] = CORBA::string_dup((*in_param)[0]);
-		
+
 		int cl_idx = find_class((*in_param)[0]);
 		if (cl_idx != -1)
-		{			
+		{
 			get_obj_prop(in_param,classes_idx[cl_idx].class_prop);
 		}
 		else
@@ -408,7 +380,7 @@ const DevVarStringArray *DbServerCache::get_class_property(DevVarStringArray *in
 										   (const char *)"DbServerCache::get_dev_property");
 		}
 	}
-	
+
 	return &ret_obj_prop;
 }
 
@@ -435,10 +407,10 @@ void DbServerCache::get_obj_prop(DevVarStringArray *in_param,PropEltIdx &obj,boo
 	int ret_length = 2;
 	int found_prop = 0;
 	char n_prop_str[256];
-	
+
 	int nb_wanted_prop = in_param->length() - 1;
 	int obj_prop = obj.prop_nb;
-	
+
 	for (int loop = 0;loop < nb_wanted_prop;loop++)
 	{
 		int lo;
@@ -448,13 +420,13 @@ void DbServerCache::get_obj_prop(DevVarStringArray *in_param,PropEltIdx &obj,boo
 			{
 				int old_ret_length = ret_length;
 				int nb_elt = obj.props_idx[lo + 1];
-				
+
 				ret_length = ret_length + 2 + nb_elt;
-				
+
 				ret_obj_prop.length(ret_length);
 				ret_obj_prop[old_ret_length] = CORBA::string_dup((*in_param)[loop + 1]);
 				ret_obj_prop[old_ret_length + 1] = CORBA::string_dup((*data_list)[obj.props_idx[lo] + 1]);
-				
+
 				for (int k = 0;k < nb_elt;k++)
 				{
 					ret_obj_prop[old_ret_length + 2 + k] = CORBA::string_dup((*data_list)[obj.props_idx[lo] + 2 + k]);
@@ -469,22 +441,22 @@ void DbServerCache::get_obj_prop(DevVarStringArray *in_param,PropEltIdx &obj,boo
 			ret_length = ret_length + 2;
 			if (dev_prop == true)
 				ret_length++;
-			
+
 			ret_obj_prop.length(ret_length);
 			ret_obj_prop[old_length] = CORBA::string_dup((*in_param)[loop + 1]);
 			ret_obj_prop[old_length + 1] = CORBA::string_dup("0");
 			if (dev_prop == true)
 				ret_obj_prop[old_length + 2] = CORBA::string_dup(" ");
 			found_prop++;
-			
+
 		}
-	}	
+	}
 	::sprintf(n_prop_str,"%d",found_prop);
 	ret_obj_prop[1] = CORBA::string_dup(n_prop_str);
-	
+
 	cout4 << "DbCache --> Data returned for a get_obj_property for object " << (*in_param)[0] << endl;
 	for (unsigned int ll=0;ll< ret_obj_prop.length();ll++)
-		cout4 << "    DbCache --> Returned string = " << ret_obj_prop[ll] << endl;	
+		cout4 << "    DbCache --> Returned string = " << ret_obj_prop[ll] << endl;
 }
 
 //-----------------------------------------------------------------------------
@@ -510,12 +482,20 @@ const DevVarStringArray *DbServerCache::get_dev_property(DevVarStringArray *in_p
 //
 // There is a special case for the dserver admin device
 //
-	
+
 	if (TG_strncasecmp((*in_param)[0],"dserver/",8) == 0)
 	{
+	    if (adm_dev_prop.first_idx == -1)
+	    {
+			TangoSys_OMemStream o;
+			o << "Device " << (*in_param)[0] << " not found in DB cache" << ends;
+
+			Tango::Except::throw_exception((const char *)"DB_DeviceNotFoundInCache",o.str(),
+										   (const char *)"DbServerCache::get_dev_property");
+	    }
 		ret_obj_prop.length(ret_length);
 		ret_obj_prop[0] = CORBA::string_dup((*in_param)[0]);
-		
+
 		get_obj_prop(in_param,adm_dev_prop,true);
 	}
 	else
@@ -532,13 +512,13 @@ const DevVarStringArray *DbServerCache::get_dev_property(DevVarStringArray *in_p
 		}
 		else
 		{
-			ret_obj_prop.length(ret_length);			
+			ret_obj_prop.length(ret_length);
 			ret_obj_prop[0] = CORBA::string_dup((*in_param)[0]);
-			
-			get_obj_prop(in_param,classes_idx[class_ind].devs_idx[dev_ind].dev_prop,true);			
+
+			get_obj_prop(in_param,classes_idx[class_ind].devs_idx[dev_ind].dev_prop,true);
 		}
 	}
-	
+
 	return &ret_obj_prop;
 }
 
@@ -618,10 +598,10 @@ const DevVarStringArray *DbServerCache::get_class_att_property(DevVarStringArray
 {
 	int found_att = 0;
 	char n_att_str[256];
-	
+
 	ret_obj_att_prop.length(2);
 	ret_obj_att_prop[0] = CORBA::string_dup((*in_param)[0]);
-	
+
 	int cl_idx = find_class((*in_param)[0]);
 	if (cl_idx != -1)
 	{
@@ -656,7 +636,7 @@ const DevVarStringArray *DbServerCache::get_class_att_property(DevVarStringArray
 						tmp_idx = tmp_idx + nb_elt + 2;
 						nb_to_copy = nb_to_copy + 2 + nb_elt;
 					}
-	
+
 					int old_length = ret_obj_att_prop.length();
 					ret_obj_att_prop.length(old_length + nb_to_copy);
 					for (int j = 0;j < nb_to_copy;j++)
@@ -675,7 +655,7 @@ const DevVarStringArray *DbServerCache::get_class_att_property(DevVarStringArray
 			}
 		}
 		::sprintf(n_att_str,"%d",found_att);
-		ret_obj_att_prop[1] = CORBA::string_dup(n_att_str);		
+		ret_obj_att_prop[1] = CORBA::string_dup(n_att_str);
 	}
 	else
 	{
@@ -688,7 +668,7 @@ const DevVarStringArray *DbServerCache::get_class_att_property(DevVarStringArray
 
 	cout4 << "DbCache --> Returned data for a get_class_att_property for class " << (*in_param)[0] << endl;
 	for (unsigned int ll=0;ll< ret_obj_att_prop.length();ll++)
-		cout4 << "    DbCache --> Returned object att prop = " << ret_obj_att_prop[ll] << endl;	
+		cout4 << "    DbCache --> Returned object att prop = " << ret_obj_att_prop[ll] << endl;
 
 	return &ret_obj_att_prop;
 }
@@ -714,12 +694,12 @@ const DevVarStringArray *DbServerCache::get_dev_att_property(DevVarStringArray *
 {
 	int found_att = 0;
 	char n_att_str[256];
-	
+
 	ret_obj_att_prop.length(2);
 	ret_obj_att_prop[0] = CORBA::string_dup((*in_param)[0]);
 
 	int class_ind,dev_ind;
-	
+
 	int ret_value = find_dev_att((*in_param)[0],class_ind,dev_ind);
 	if (ret_value != -1)
 	{
@@ -744,7 +724,7 @@ const DevVarStringArray *DbServerCache::get_dev_att_property(DevVarStringArray *
 						tmp_idx = tmp_idx + nb_elt + 2;
 						nb_to_copy = nb_to_copy + 2 + nb_elt;
 					}
-	
+
 					int old_length = ret_obj_att_prop.length();
 					ret_obj_att_prop.length(old_length + nb_to_copy);
 					for (int j = 0;j < nb_to_copy;j++)
@@ -777,7 +757,7 @@ const DevVarStringArray *DbServerCache::get_dev_att_property(DevVarStringArray *
 
 	cout4 << "DbCache --> Returned data for a get_dev_att_property for device " << (*in_param)[0] << endl;
 	for (unsigned int ll=0;ll< ret_obj_att_prop.length();ll++)
-		cout4 << "    DbCache --> Returned object att prop = " << ret_obj_att_prop[ll] << endl;	
+		cout4 << "    DbCache --> Returned object att prop = " << ret_obj_att_prop[ll] << endl;
 
 	return &ret_obj_att_prop;
 }
@@ -831,6 +811,9 @@ int DbServerCache::find_dev_att(DevString dev_name,int &class_ind,int &dev_ind)
 int DbServerCache::find_obj(DevString obj_name,int &obj_ind)
 {
 
+    if (ctrl_serv_prop.first_idx == -1)
+        return -1;
+
 	if (TG_strcasecmp((*data_list)[ctrl_serv_prop.first_idx],obj_name) == 0)
 	{
 		obj_ind = ctrl_serv_prop.first_idx;
@@ -839,6 +822,7 @@ int DbServerCache::find_obj(DevString obj_name,int &obj_ind)
 
 	return -1;
 }
+
 //-----------------------------------------------------------------------------
 //
 // DbServerCache::~DbServerCache() - destructor of the DbServerCache class
@@ -847,25 +831,18 @@ int DbServerCache::find_obj(DevString obj_name,int &obj_ind)
 
 DbServerCache::~DbServerCache()
 {
-	if (DServer_class_prop.props_idx != NULL)
-		delete [] DServer_class_prop.props_idx;
-	
-	if (Default_prop.props_idx != NULL)
-		delete [] Default_prop.props_idx;
-	
-	if (adm_dev_prop.props_idx != NULL)
-		delete [] adm_dev_prop.props_idx;
-	
-	if (ctrl_serv_prop.props_idx != NULL)
-		delete [] ctrl_serv_prop.props_idx;
-	
+    delete [] DServer_class_prop.props_idx;
+    delete [] Default_prop.props_idx;
+    delete [] adm_dev_prop.props_idx;
+    delete [] ctrl_serv_prop.props_idx;
+
 	for (int cl_loop = 0;cl_loop < class_nb;cl_loop++)
 	{
 		if (classes_idx[cl_loop].class_prop.props_idx != NULL)
 			delete [] classes_idx[cl_loop].class_prop.props_idx;
 		if (classes_idx[cl_loop].class_att_prop.atts_idx != NULL)
 			delete [] classes_idx[cl_loop].class_att_prop.atts_idx;
-		
+
 		for (int dev_loop = 0;dev_loop < classes_idx[cl_loop].dev_nb;dev_loop++)
 		{
 			if (classes_idx[cl_loop].devs_idx[dev_loop].dev_prop.props_idx != NULL)
@@ -918,7 +895,7 @@ void DbServerCache::prop_indexes(int &start,int &stop,PropEltIdx &obj,const DevV
 		obj.props_idx[id++] = nb_elt;
 		stop = stop + nb_elt + 2;
 	}
-	
+
 	obj.last_idx = stop;
 	obj.first_idx = start;
 	obj.prop_nb = nb_prop;
@@ -953,12 +930,12 @@ void DbServerCache::prop_att_indexes(int &start,int &stop,AttPropEltIdx &obj,con
 		obj.atts_idx = NULL;
 		return;
 	}
-	
+
 	int id = 0;
 	obj.att_nb = nb_att;
 	obj.atts_idx = new int[nb_att];
 	stop = start + 2;
-		
+
 	for (int ll = 0;ll < nb_att;ll++)
 	{
 		obj.atts_idx[id++] = stop;
@@ -973,7 +950,7 @@ void DbServerCache::prop_att_indexes(int &start,int &stop,AttPropEltIdx &obj,con
 		if (ll == (nb_att - 1))
 			stop--;
 	}
-	
+
 	obj.last_idx = stop;
 	obj.first_idx = start;
 }
@@ -996,9 +973,7 @@ void DbServerCache::prop_att_indexes(int &start,int &stop,AttPropEltIdx &obj,con
 
 
 const DevVarStringArray *DbServerCache::get_obj_property(DevVarStringArray *in_param)
-{	
-	int ret_length = 2;
-
+{
 	int obj_ind;
 	int res = find_obj((*in_param)[0],obj_ind);
 	if (res == -1)
@@ -1011,12 +986,14 @@ const DevVarStringArray *DbServerCache::get_obj_property(DevVarStringArray *in_p
 	}
 	else
 	{
-		ret_obj_prop.length(ret_length);			
+	    int ret_length = 2;
+
+		ret_obj_prop.length(ret_length);
 		ret_obj_prop[0] = CORBA::string_dup((*in_param)[0]);
-			
+
 		get_obj_prop(in_param,ctrl_serv_prop,true);
-	}			
-	
+	}
+
 	return &ret_obj_prop;
 }
 
@@ -1041,10 +1018,10 @@ const DevVarStringArray *DbServerCache::get_device_property_list(DevVarStringArr
 //
 // There is a special case for the dserver admin device
 //
-	
+
 	if (TG_strncasecmp((*in_param)[0],"dserver/",8) == 0)
 	{
-		ret_prop_list.length(0);	
+		ret_prop_list.length(0);
 		get_obj_prop_list(in_param,adm_dev_prop);
 	}
 	else
@@ -1061,11 +1038,11 @@ const DevVarStringArray *DbServerCache::get_device_property_list(DevVarStringArr
 		}
 		else
 		{
-			ret_prop_list.length(0);		
-			get_obj_prop_list(in_param,classes_idx[class_ind].devs_idx[dev_ind].dev_prop);			
+			ret_prop_list.length(0);
+			get_obj_prop_list(in_param,classes_idx[class_ind].devs_idx[dev_ind].dev_prop);
 		}
 	}
-	
+
 	return &ret_prop_list;
 }
 
@@ -1089,7 +1066,7 @@ void DbServerCache::get_obj_prop_list(DevVarStringArray *in_param,PropEltIdx &ob
 // First analyse the wildcard
 //
 
-	bool before,after,wildcard_used;
+	bool before = false,after = false,wildcard_used;
 	string before_str;
 	string after_str;
 	string wildcard((*in_param)[1]);
@@ -1129,10 +1106,10 @@ void DbServerCache::get_obj_prop_list(DevVarStringArray *in_param,PropEltIdx &ob
 	}
 	else
 		wildcard_used = false;
-	
+
 	int ret_length = 1;
 	int obj_prop = obj.prop_nb;
-	
+
 	int lo;
 	string::size_type pos_after,pos_before;
 
@@ -1141,7 +1118,7 @@ void DbServerCache::get_obj_prop_list(DevVarStringArray *in_param,PropEltIdx &ob
 //
 
 	for (lo = 0;lo < obj_prop * 2;lo = lo + 2)
-	{		
+	{
 
 //
 // Check according to the user wildcard, if we have to returned this property name
@@ -1159,7 +1136,7 @@ void DbServerCache::get_obj_prop_list(DevVarStringArray *in_param,PropEltIdx &ob
 				{
 					string tmp_name((*data_list)[obj.props_idx[lo]]);
 					transform(tmp_name.begin(),tmp_name.end(),tmp_name.begin(),::tolower);
-					
+
 					pos_after = tmp_name.rfind(after_str);
 					if ((pos_after != string::npos) && (pos_after == (tmp_name.size() - after_str.size())))
 						store = true;
@@ -1202,11 +1179,89 @@ void DbServerCache::get_obj_prop_list(DevVarStringArray *in_param,PropEltIdx &ob
 		if (store == true)
 		{
 			ret_prop_list.length(ret_length);
-			ret_prop_list[ret_length - 1] = CORBA::string_dup((*data_list)[obj.props_idx[lo]]);		
+			ret_prop_list[ret_length - 1] = CORBA::string_dup((*data_list)[obj.props_idx[lo]]);
 			ret_length = ret_length + 1;
 		}
 	}
 }
 
+//-----------------------------------------------------------------------------
+//
+//  DbServerCache::import_tac_dev()
+//
+//	This method returns to the caller of the info necessary to import the
+//	TAC device. The returned data are the same than the one returned by
+//  the classical API
+//
+//	This method returns a pointer to a DevVarLongStringArray initilised
+//  with the Tango access control import data
+//-----------------------------------------------------------------------------
+
+const DevVarLongStringArray *DbServerCache::import_tac_dev(string &tac_dev)
+{
+
+//
+// Throw exception if no info in cache
+//
+
+	if (imp_tac.last_idx == -1 || imp_tac.first_idx >= (int)data_list->length())
+	{
+		Tango::Except::throw_exception((const char *)"API_DatabaseCacheAccess",
+                                       (const char *)"No TAC device in Db cache",
+                                       (const char *)"DbServerCache::import_tac_dev");
+	}
+
+//
+// Throw exception if the device in cache is not the requested one
+//
+
+    if (tac_dev.size() != strlen((*data_list)[imp_tac.first_idx]))
+    {
+		Tango::Except::throw_exception((const char *)"API_DatabaseCacheAccess",
+                                       (const char *)"Device not available from cache",
+                                       (const char *)"DbServerCache::import_tac_dev");
+    }
+
+    string local_tac_dev(tac_dev);
+    string cache_tac_dev((*data_list)[imp_tac.first_idx]);
+
+    transform(local_tac_dev.begin(),local_tac_dev.end(),local_tac_dev.begin(),::tolower);
+    transform(cache_tac_dev.begin(),cache_tac_dev.end(),cache_tac_dev.begin(),::tolower);
+
+    if (local_tac_dev != cache_tac_dev)
+    {
+        Tango::Except::throw_exception((const char *)"API_DatabaseCacheAccess",
+                                       (const char *)"Device not available from cache",
+                                       (const char *)"DbServerCache::import_tac_dev");
+    }
+
+//
+// Throw exception if the device is not found in DB
+//
+
+	if (::strcmp((*data_list)[imp_tac.first_idx + 1],"Not Found") == 0)
+	{
+		TangoSys_OMemStream o;
+		o << "Device " << tac_dev << " not defined in database" << ends;
+
+		Tango::Except::throw_exception((const char *)"DB_DeviceNotDefined",o.str(),
+									   (const char *)"DbServerCache::import_tac_dev");
+	}
+
+//
+// Return cache data
+//
+
+	imp_tac_data.lvalue.length(2);
+	imp_tac_data.svalue.length(5);
+
+	imp_tac_data.lvalue[0] = ::atoi((*data_list)[imp_tac.first_idx + 5]);
+	imp_tac_data.lvalue[1] = ::atoi((*data_list)[imp_tac.last_idx - 1]);
+
+	for (int loop = 0;loop < 5;loop++)
+		imp_tac_data.svalue[loop] = CORBA::string_dup((*data_list)[imp_tac.first_idx + loop]);
+
+	return &imp_tac_data;
+}
 
 } // End of Tango namespace
