@@ -1,20 +1,20 @@
-static const char *RcsId = "$Id: command.cpp 15556 2011-02-11 08:25:58Z taurel $\n$Name$";
+static const char *RcsId = "$Id: command.cpp 20437 2012-05-31 12:33:29Z taurel $\n$Name$";
 
 //+============================================================================
 //
 // file :               Command.cpp
 //
 // description :        C++ source code for the Command and templCommand classes.
-//			The Command class is the root class for all derived 
+//			The Command class is the root class for all derived
 //			Command classes. The TemplCommand class is a template
-//			command class use for command which does take input 
+//			command class use for command which does take input
 //			nor outout parameters.
 //
 // project :            TANGO
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -25,184 +25,16 @@ static const char *RcsId = "$Id: command.cpp 15556 2011-02-11 08:25:58Z taurel $
 // it under the terms of the GNU Lesser General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // Tango is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 15556 $
-//
-// $Log$
-// Revision 3.12  2010/09/09 13:44:46  taurel
-// - Add year 2010 in Copyright notice
-//
-// Revision 3.11  2009/01/21 12:49:04  taurel
-// - Change CopyRights for 2009
-//
-// Revision 3.10  2008/10/06 15:00:36  taurel
-// - Changed the licensing info from GPL to LGPL
-//
-// Revision 3.9  2008/10/03 06:51:36  taurel
-// - Add some licensing info in each files
-//
-// Revision 3.8  2007/04/20 14:40:25  taurel
-// - Ported to Windows 64 bits x64 architecture
-//
-// Revision 3.7  2007/04/16 14:56:36  taurel
-// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
-// - Ported to omniORB4.1
-// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
-// - Added a new filterable field in the archive event
-//
-// Revision 3.6  2007/03/06 08:18:03  taurel
-// - Added 64 bits data types for 64 bits computer...
-//
-// Revision 3.5  2005/05/04 11:50:30  taurel
-// - Changes for 32<-->64 bits data exchange
-//
-// Revision 3.4  2005/04/15 11:34:07  taurel
-// - Changes to support Tango on 64 bits computer
-// - Support for Linux 2.6 kernel with NPTL (Signal management)
-//
-// Revision 3.3  2004/07/07 08:39:57  taurel
-//
-// - Fisrt commit after merge between Trunk and release 4 branch
-// - Add EventData copy ctor, asiignement operator and dtor
-// - Add Database and DeviceProxy::get_alias() method
-// - Add AttributeProxy ctor from "device_alias/attribute_name"
-// - Exception thrown when subscribing two times for exactly yhe same event
-//
-// Revision 3.2.2.2  2004/05/27 14:01:41  taurel
-// - Add a casting to make gcc 2.95 quiet
-//
-// Revision 3.2.2.1  2004/03/09 16:36:36  taurel
-// - Added HP aCC port (thanks to Claudio from Elettra)
-// - Some last small bugs fixes
-//
-// Revision 3.2  2003/05/28 14:55:08  taurel
-// Add the include (conditionally) of the include files generated by autoconf
-//
-// Revision 3.1  2003/05/16 08:46:15  taurel
-// Many changes for release 3.0.1. The most important ones are :
-// - Timeout are backs
-// - Multiple db servers (change in TANGO_HOST syntax)
-// - Added methods to print DeviceData, DeviceDataHistory, DeviceAttribute and DeviceAttributeHistory instances
-// - Attributes name stored in blackbox
-// - Remove check if a class is created without any device
-// - It's now possible to create a DeviceProxy from its alias name
-// - Command, attribute names are case insensitive
-// - Change parameters of some DeviceProxy logging methods
-// - Change parameters of DeviceProxy asynchronous replies calls
-// - New serialization model in device server (no serialization model)
-// - Win32 (2000) device server service does not exit at loggoff anymore
-// - Miscellaneous bug fixes
-//
-// Revision 3.0  2003/03/25 16:41:52  taurel
-// Many changes for Tango release 3.0 including
-// - Added full logging features
-// - Added asynchronous calls
-// - Host name of clients now stored in black-box
-// - Three serialization model in DS
-// - Fix miscellaneous bugs
-// - Ported to gcc 3.2
-// - Added ApiUtil::cleanup() and destructor methods
-// - Some internal cleanups
-// - Change the way how TangoMonitor class is implemented. It's a recursive
-//   mutex
-//
-// Revision 2.9  2003/01/09 12:03:16  taurel
-// - Ported to gcc 3.2
-// - Added ApiUtil::cleanup() and ApiUtil::~ApiUtil() methods
-// - Replace some ORB * by ORB_ptr
-// - Use CORBA::ORB::is_nil() instead of comparing to NULL
-//
-// Revision 2.8  2002/12/16 12:06:21  taurel
-// No change in code at all but only forgot th emost important line in
-// list of updates in the previous release :
-// - Change underlying ORB from ORBacus to omniORB
-//
-// Revision 2.7  2002/12/16 10:15:36  taurel
-// - New method get_device_list() in Util class
-// - Util::get_class_list takes DServer device into account
-// - Util::get_device_by_name() takes DServer device into account
-// - Util::get_device_list_by_class() takes DServer device into account
-// - New parameter to the attribute::set_value() method to enable CORBA to free
-// memory allocated for the attribute
-//
-// Revision 2.6  2002/10/17 07:43:05  taurel
-// Fix bug in history stored by the polling thread :
-// - We need one copy of the attribute data to build an history!!! It is true
-// also for command which return data created by the DeviceImpl::create_xxx
-// methods. Chnage in pollring.cpp/pollring.h/dserverpoll.cpp/pollobj.cpp
-// and pollobj.h
-//
-// Revision 2.5  2002/10/15 11:27:18  taurel
-// Fix bugs in device.cpp file :
-// - Protect the state and status CORBA attribute with the device monitor
-// Add the "TgLibVers" string as a #define in tango_config.h
-//
-// Revision 2.4  2002/08/12 15:06:53  taurel
-// Several big fixes and changes
-//   - Remove HP-UX specific code
-//   - Fix bug in polling alogorithm which cause the thread to enter an infinite
-//     loop (pollthread.cpp)
-//   - For bug for Win32 device when trying to set attribute config
-//     (attribute.cpp)
-//
-// Revision 2.3  2002/07/02 15:22:24  taurel
-// Miscellaneous small changes/bug fixes for Tango CPP release 2.1.0
-//     - classes reference documentation now generated using doxygen instead of doc++
-//     - A little file added to the library which summarizes version number.
-//       The RCS/CVS "ident" command will now tells you that release library x.y.z is composed
-//       by C++ client classes set release a.b and C++ server classes set release c.d
-//     - Fix incorrect field setting for DevFailed exception re-thrown from a CORBA exception
-//     - It's now not possible to poll the Init command
-//     - It's now possible to define a default class doc. per control system
-//       instance (using property)
-//     - The test done to check if attribute value has been set before it is
-//       returned to caller is done only if the attribute quality is set to VALID
-//     - The JTCInitialize object is now stored in the Util
-//     - Windows specific : The tango.h file now also include winsock.h
-//
-// Revision 2.2  2002/04/30 10:50:40  taurel
-// Don't check alarm on attribute if attribute quality factor is INVALID
-//
-// Revision 2.1  2002/04/29 12:24:03  taurel
-// Fix bug in attribute::set_value method and on the check against min and max value when writing attributes
-//
-// Revision 2.0  2002/04/09 14:45:09  taurel
-// See Tango WEB pages for list of changes
-//
-// Revision 1.6  2001/10/08 09:03:11  taurel
-// See tango WEB pages for list of changes
-//
-// Revision 1.5  2001/07/04 12:27:09  taurel
-// New methods re_throw_exception(). Read_attributes supports AllAttr mnemonic A new add_attribute()method in DeviceImpl class New way to define attribute properties New pattern to prevent full re-compile For multi-classes DS, it is now possible to use the Util::get_device_by_name() method in device constructor Adding << operator ovebloading Fix devie CORBA ref. number when device constructor sends an excep.
-//
-// Revision 1.4  2001/05/04 09:28:12  taurel
-// Fix bugs in DServer::restart() method and in Util::get_device_by_name() method
-//
-// Revision 1.3  2001/03/30 08:03:44  taurel
-// Fix bugs in attributes. For linux, add signal_handler in its own thread, change the way to kill server. For all system, change DevRestart philosophy.
-//
-// Revision 1.2  2001/03/09 08:20:14  taurel
-// Fix bug in the MultiClassAttribute::init_class_attribute() method. Also remove the DbErr_DeviceNotDefined define.
-//
-// Revision 1.1.1.1  2001/02/27 08:46:20  taurel
-// Imported sources
-//
-// Revision 1.3  2000/04/13 10:40:40  taurel
-// Added attribute support
-//
-// Revision 1.2  2000/02/04 11:00:13  taurel
-// Just update revision number
-//
-// Revision 1.1.1.1  2000/02/04 10:58:29  taurel
-// Imported sources
+// $Revision: 20437 $
 //
 //-============================================================================
 
@@ -218,8 +50,8 @@ namespace Tango
 
 //+-------------------------------------------------------------------------
 //
-// method : 		Command::Command 
-// 
+// method : 		Command::Command
+//
 // description : 	constructors for abstract class Command
 //
 //--------------------------------------------------------------------------
@@ -227,21 +59,19 @@ namespace Tango
 Command::Command(const char *s,
 		 Tango::CmdArgType in,
 		 Tango::CmdArgType out)
-:name(s),in_type(in),out_type(out)
+:name(s),in_type(in),out_type(out),ext(new CommandExt)
 {
 	lower_name = name;
 	transform(lower_name.begin(),lower_name.end(),lower_name.begin(),::tolower);
-	ext = new CommandExt();
 }
 
 Command::Command(string &s,
 		 Tango::CmdArgType in,
 		 Tango::CmdArgType out)
-:name(s),in_type(in),out_type(out)
+:name(s),in_type(in),out_type(out),ext(new CommandExt)
 {
 	lower_name = name;
 	transform(lower_name.begin(),lower_name.end(),lower_name.begin(),::tolower);
-	ext = new CommandExt();
 }
 
 Command::Command(const char *s,
@@ -249,9 +79,8 @@ Command::Command(const char *s,
 		 Tango::CmdArgType out,
 		 const char *in_desc,
 		 const char *out_desc)
-:name(s),in_type(in),out_type(out)
+:name(s),in_type(in),out_type(out),ext(new CommandExt)
 {
-	ext = new CommandExt();
 	if (in_desc != NULL)
 		in_type_desc = in_desc;
 	if (out_desc != NULL)
@@ -266,9 +95,8 @@ Command::Command(string &s,
 		 string &in_desc,
 		 string &out_desc)
 :name(s),in_type(in),out_type(out),
-in_type_desc(in_desc),out_type_desc(out_desc)
+in_type_desc(in_desc),out_type_desc(out_desc),ext(new CommandExt)
 {
-	ext = new CommandExt();
 	lower_name = name;
 	transform(lower_name.begin(),lower_name.end(),lower_name.begin(),::tolower);
 }
@@ -277,9 +105,8 @@ Command::Command(const char *s,
 		 Tango::CmdArgType in,
 		 Tango::CmdArgType out,
 		 Tango::DispLevel level)
-:name(s),in_type(in),out_type(out)
+:name(s),in_type(in),out_type(out),ext(new CommandExt(level))
 {
-	ext = new CommandExt(level);
 	lower_name = name;
 	transform(lower_name.begin(),lower_name.end(),lower_name.begin(),::tolower);
 }
@@ -288,9 +115,8 @@ Command::Command(string &s,
 		 Tango::CmdArgType in,
 		 Tango::CmdArgType out,
 		 Tango::DispLevel level)
-:name(s),in_type(in),out_type(out),ext(NULL)
+:name(s),in_type(in),out_type(out),ext(new CommandExt(level))
 {
-	ext = new CommandExt(level);
 	lower_name = name;
 	transform(lower_name.begin(),lower_name.end(),lower_name.begin(),::tolower);
 }
@@ -301,9 +127,8 @@ Command::Command(const char *s,
 		 const char *in_desc,
 		 const char *out_desc,
 		 Tango::DispLevel level)
-:name(s),in_type(in),out_type(out)
+:name(s),in_type(in),out_type(out),ext(new CommandExt(level))
 {
-	ext = new CommandExt(level);
 	if (in_desc != NULL)
 		in_type_desc = in_desc;
 	if (out_desc != NULL)
@@ -319,9 +144,8 @@ Command::Command(string &s,
 		 string &out_desc,
 		 Tango::DispLevel level)
 :name(s),in_type(in),out_type(out),
-in_type_desc(in_desc),out_type_desc(out_desc)
+in_type_desc(in_desc),out_type_desc(out_desc),ext(new CommandExt(level))
 {
-	ext = new CommandExt(level);
 	lower_name = name;
 	transform(lower_name.begin(),lower_name.end(),lower_name.begin(),::tolower);
 }
@@ -329,7 +153,7 @@ in_type_desc(in_desc),out_type_desc(out_desc)
 //+----------------------------------------------------------------------------
 //
 // method : 		Command::extract()
-// 
+//
 // description : 	Command extract methods. These are very simple methods
 //			but overloaded many times for all Tango types.
 //
@@ -338,7 +162,7 @@ in_type_desc(in_desc),out_type_desc(out_desc)
 void Command::throw_bad_type(const char *type)
 {
 	TangoSys_OMemStream o;
-	
+
 	o << "Incompatible command argument type, expected type is : Tango::" << type << ends;
 	Except::throw_exception((const char *)"API_IncompatibleCmdArgumentType",
 			      o.str(),
@@ -489,10 +313,16 @@ void Command::extract(const CORBA::Any &in,Tango::DevState &data)
 		throw_bad_type("DevState");
 }
 
+void Command::extract(const CORBA::Any &in,const Tango::DevEncoded *&data)
+{
+	if ((in >>= data) == false)
+		throw_bad_type("DevEncoded");
+}
+
 //+----------------------------------------------------------------------------
 //
 // method : 		Command::insert()
-// 
+//
 // description : 	Command insert methods. These are very simple methods
 //			but overloaded many times for all Tango types.
 //
@@ -515,7 +345,7 @@ void Command::alloc_any(CORBA::Any *&any_ptr)
 CORBA::Any *Command::insert()
 {
 	CORBA::Any *out_any;
-	alloc_any(out_any);	
+	alloc_any(out_any);
 	return out_any;
 }
 
@@ -524,8 +354,8 @@ CORBA::Any *Command::insert(Tango::DevBoolean data)
 	CORBA::Any *out_any;
 	alloc_any(out_any);
 	CORBA::Any::from_boolean tmp(data);
-	
-	(*out_any) <<= tmp;	
+
+	(*out_any) <<= tmp;
 	return out_any;
 }
 
@@ -533,17 +363,17 @@ CORBA::Any *Command::insert(Tango::DevShort data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
-	
+
 CORBA::Any *Command::insert(Tango::DevLong data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -551,8 +381,8 @@ CORBA::Any *Command::insert(Tango::DevLong64 data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -560,8 +390,8 @@ CORBA::Any *Command::insert(Tango::DevFloat data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -569,8 +399,8 @@ CORBA::Any *Command::insert(Tango::DevDouble data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -578,17 +408,17 @@ CORBA::Any *Command::insert(Tango::DevUShort data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
-	
+
 CORBA::Any *Command::insert(Tango::DevULong data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -596,8 +426,8 @@ CORBA::Any *Command::insert(Tango::DevULong64 data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -605,10 +435,10 @@ CORBA::Any *Command::insert(Tango::DevString data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= data;
 	delete [] data;
-		
+
 	return out_any;
 }
 
@@ -616,8 +446,8 @@ CORBA::Any *Command::insert(const char *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -625,8 +455,8 @@ CORBA::Any *Command::insert(Tango::DevVarCharArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -634,9 +464,9 @@ CORBA::Any *Command::insert(Tango::DevVarCharArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
-	delete data;	
+	delete data;
 
 	return out_any;
 }
@@ -645,8 +475,8 @@ CORBA::Any *Command::insert(Tango::DevVarShortArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -654,10 +484,10 @@ CORBA::Any *Command::insert(Tango::DevVarShortArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -665,8 +495,8 @@ CORBA::Any *Command::insert(Tango::DevVarLongArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -674,10 +504,10 @@ CORBA::Any *Command::insert(Tango::DevVarLongArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -685,8 +515,8 @@ CORBA::Any *Command::insert(Tango::DevVarLong64Array &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -694,10 +524,10 @@ CORBA::Any *Command::insert(Tango::DevVarLong64Array *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -705,8 +535,8 @@ CORBA::Any *Command::insert(Tango::DevVarFloatArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -714,10 +544,10 @@ CORBA::Any *Command::insert(Tango::DevVarFloatArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -725,8 +555,8 @@ CORBA::Any *Command::insert(Tango::DevVarDoubleArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -734,10 +564,10 @@ CORBA::Any *Command::insert(Tango::DevVarDoubleArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -745,8 +575,8 @@ CORBA::Any *Command::insert(Tango::DevVarUShortArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -754,10 +584,10 @@ CORBA::Any *Command::insert(Tango::DevVarUShortArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -765,8 +595,8 @@ CORBA::Any *Command::insert(Tango::DevVarULongArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -774,10 +604,10 @@ CORBA::Any *Command::insert(Tango::DevVarULongArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -785,8 +615,8 @@ CORBA::Any *Command::insert(Tango::DevVarULong64Array &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -794,10 +624,10 @@ CORBA::Any *Command::insert(Tango::DevVarULong64Array *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -805,8 +635,8 @@ CORBA::Any *Command::insert(Tango::DevVarStringArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -814,10 +644,10 @@ CORBA::Any *Command::insert(Tango::DevVarStringArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -825,8 +655,8 @@ CORBA::Any *Command::insert(Tango::DevVarLongStringArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -834,10 +664,10 @@ CORBA::Any *Command::insert(Tango::DevVarLongStringArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
-	
+
 	return out_any;
 }
 
@@ -845,7 +675,7 @@ CORBA::Any *Command::insert(Tango::DevVarDoubleStringArray *data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
+
 	(*out_any) <<= (*data);
 	delete data;
 
@@ -856,8 +686,8 @@ CORBA::Any *Command::insert(Tango::DevVarDoubleStringArray &data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;		
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
@@ -865,16 +695,26 @@ CORBA::Any *Command::insert(Tango::DevState data)
 {
 	CORBA::Any *out_any;
 	alloc_any(out_any);
-	
-	(*out_any) <<= data;	
+
+	(*out_any) <<= data;
 	return out_any;
 }
 
+CORBA::Any *Command::insert(Tango::DevEncoded *data)
+{
+	CORBA::Any *out_any;
+	alloc_any(out_any);
+
+	(*out_any) <<= (*data);
+	delete data;
+
+	return out_any;
+}
 
 //+-------------------------------------------------------------------------
 //
 // method : 		TempCommand class constructors
-// 
+//
 // description : 	instance constructor
 //
 //--------------------------------------------------------------------------
@@ -882,7 +722,7 @@ CORBA::Any *Command::insert(Tango::DevState data)
 
 TemplCommand::TemplCommand(const char *s,
 			   void (DeviceImpl::*f)())
-			   :exe_ptr(f),ext(NULL)
+			   :exe_ptr(f),ext(Tango_NullPtr)
 {
 	name = s;
 	allowed_ptr = NULL;
@@ -894,7 +734,7 @@ TemplCommand::TemplCommand(const char *s,
 TemplCommand::TemplCommand(const char *s,
 			   void (DeviceImpl::*f)(),
 			   bool (DeviceImpl::*a)(const CORBA::Any &))
-			   :exe_ptr(f),ext(NULL),allowed_ptr(a)
+			   :exe_ptr(f),ext(Tango_NullPtr),allowed_ptr(a)
 {
 	name = s;
 	in_type = out_type = Tango::DEV_VOID;
@@ -904,7 +744,7 @@ TemplCommand::TemplCommand(const char *s,
 
 TemplCommand::TemplCommand(string &s,
 			   void (DeviceImpl::*f)())
-			   :exe_ptr(f),ext(NULL)
+			   :exe_ptr(f),ext(Tango_NullPtr)
 {
 	name = s;
 	in_type = out_type = Tango::DEV_VOID;
@@ -916,7 +756,7 @@ TemplCommand::TemplCommand(string &s,
 TemplCommand::TemplCommand(string &s,
 			   void (DeviceImpl::*f)(),
 			   bool (DeviceImpl::*a)(const CORBA::Any &))
-			   :exe_ptr(f),ext(NULL),allowed_ptr(a)
+			   :exe_ptr(f),ext(Tango_NullPtr),allowed_ptr(a)
 {
 	name = s;
 	in_type = out_type = Tango::DEV_VOID;
@@ -927,7 +767,7 @@ TemplCommand::TemplCommand(string &s,
 TemplCommand::TemplCommand(const char *s,
 			   void (DeviceImpl::*f)(),
 			   const char *in_desc,const char *out_desc)
-			   :exe_ptr(f),ext(NULL)
+			   :exe_ptr(f),ext(Tango_NullPtr)
 {
 	name = s;
 	if (in_desc != NULL)
@@ -944,7 +784,7 @@ TemplCommand::TemplCommand(const char *s,
 			   void (DeviceImpl::*f)(),
 			   bool (DeviceImpl::*a)(const CORBA::Any &),
 			   const char *in_desc,const char *out_desc)
-			   :exe_ptr(f),ext(NULL),allowed_ptr(a)
+			   :exe_ptr(f),ext(Tango_NullPtr),allowed_ptr(a)
 {
 	name = s;
 	if (in_desc != NULL)
@@ -960,7 +800,7 @@ TemplCommand::TemplCommand(string &s,
 			   void (DeviceImpl::*f)(),
 			   bool (DeviceImpl::*a)(const CORBA::Any &),
 			   string &in_desc,string &out_desc)
-			   :exe_ptr(f),ext(NULL),allowed_ptr(a)
+			   :exe_ptr(f),ext(Tango_NullPtr),allowed_ptr(a)
 {
 	name = s;
 	in_type_desc = in_desc;
@@ -973,7 +813,7 @@ TemplCommand::TemplCommand(string &s,
 TemplCommand::TemplCommand(string &s,
 			   void (DeviceImpl::*f)(),
 			   string &in_desc,string &out_desc)
-			   :exe_ptr(f),ext(NULL)
+			   :exe_ptr(f),ext(Tango_NullPtr)
 {
 	name = s;
 	in_type_desc = in_desc;
@@ -987,7 +827,7 @@ TemplCommand::TemplCommand(string &s,
 TemplCommand::TemplCommand(const char *s,
 			   void (DeviceImpl::*f)(),
 			   Tango::DispLevel level)
-			   :exe_ptr(f),ext(NULL)
+			   :exe_ptr(f),ext(Tango_NullPtr)
 {
 	name = s;
 	allowed_ptr = NULL;
@@ -1001,7 +841,7 @@ TemplCommand::TemplCommand(const char *s,
 			   void (DeviceImpl::*f)(),
 			   bool (DeviceImpl::*a)(const CORBA::Any &),
 			   Tango::DispLevel level)
-			   :exe_ptr(f),ext(NULL),allowed_ptr(a)
+			   :exe_ptr(f),ext(Tango_NullPtr),allowed_ptr(a)
 {
 	name = s;
 	in_type = out_type = Tango::DEV_VOID;
@@ -1013,7 +853,7 @@ TemplCommand::TemplCommand(const char *s,
 TemplCommand::TemplCommand(string &s,
 			   void (DeviceImpl::*f)(),
 			   Tango::DispLevel level)
-			   :exe_ptr(f),ext(NULL)
+			   :exe_ptr(f),ext(Tango_NullPtr)
 {
 	name = s;
 	in_type = out_type = Tango::DEV_VOID;
@@ -1027,7 +867,7 @@ TemplCommand::TemplCommand(string &s,
 			   void (DeviceImpl::*f)(),
 			   bool (DeviceImpl::*a)(const CORBA::Any &),
 			   Tango::DispLevel level)
-			   :exe_ptr(f),ext(NULL),allowed_ptr(a)
+			   :exe_ptr(f),ext(Tango_NullPtr),allowed_ptr(a)
 {
 	name = s;
 	in_type = out_type = Tango::DEV_VOID;
@@ -1040,7 +880,7 @@ TemplCommand::TemplCommand(const char *s,
 			   void (DeviceImpl::*f)(),
 			   const char *in_desc,const char *out_desc,
 			   Tango::DispLevel level)
-			   :exe_ptr(f),ext(NULL)
+			   :exe_ptr(f),ext(Tango_NullPtr)
 {
 	name = s;
 	if (in_desc != NULL)
@@ -1059,7 +899,7 @@ TemplCommand::TemplCommand(const char *s,
 			   bool (DeviceImpl::*a)(const CORBA::Any &),
 			   const char *in_desc,const char *out_desc,
 			   Tango::DispLevel level)
-			   :exe_ptr(f),ext(NULL),allowed_ptr(a)
+			   :exe_ptr(f),ext(Tango_NullPtr),allowed_ptr(a)
 {
 	name = s;
 	if (in_desc != NULL)
@@ -1077,7 +917,7 @@ TemplCommand::TemplCommand(string &s,
 			   bool (DeviceImpl::*a)(const CORBA::Any &),
 			   string &in_desc,string &out_desc,
 			   Tango::DispLevel level)
-			   :exe_ptr(f),ext(NULL),allowed_ptr(a)
+			   :exe_ptr(f),ext(Tango_NullPtr),allowed_ptr(a)
 {
 	name = s;
 	in_type_desc = in_desc;
@@ -1092,7 +932,7 @@ TemplCommand::TemplCommand(string &s,
 			   void (DeviceImpl::*f)(),
 			   string &in_desc,string &out_desc,
 			   Tango::DispLevel level)
-			   :exe_ptr(f),ext(NULL)
+			   :exe_ptr(f),ext(Tango_NullPtr)
 {
 	name = s;
 	in_type_desc = in_desc;
@@ -1107,7 +947,7 @@ TemplCommand::TemplCommand(string &s,
 //+-------------------------------------------------------------------------
 //
 // method : 		set_type
-// 
+//
 // description : 	Set the Command class type data according to the type
 //			of the object passed as parameters
 //
@@ -1122,7 +962,7 @@ void TemplCommand::set_type(const type_info &data_type,Tango::CmdArgType &type)
 	{
 		cout4 << "Command : " << name << ", Type is void" << endl;
 		type = Tango::DEV_VOID;
-	}	
+	}
 	else if (data_type == typeid(Tango::DevBoolean))
 	{
 		cout4 << "Command : " << name << ", Type is a boolean" << endl;
@@ -1173,7 +1013,7 @@ void TemplCommand::set_type(const type_info &data_type,Tango::CmdArgType &type)
 		cout4 << "Command : " << name << ", Type is a string" << endl;
 		type = Tango::DEV_STRING;
 	}
-	else if ((data_type == typeid(Tango::DevVarCharArray)) || 
+	else if ((data_type == typeid(Tango::DevVarCharArray)) ||
 		 (data_type == typeid(const Tango::DevVarCharArray *)) ||
 		 (data_type == typeid(Tango::DevVarCharArray *)))
 	{
@@ -1226,7 +1066,7 @@ void TemplCommand::set_type(const type_info &data_type,Tango::CmdArgType &type)
 		 (data_type == typeid(const Tango::DevVarULongArray *)) ||
 		 (data_type == typeid(Tango::DevVarULongArray *)))
 	{
-		cout4 << "Command : " << name << ", Type is a undigned long array" << endl;
+		cout4 << "Command : " << name << ", Type is a unsigned long array" << endl;
 		type = Tango::DEVVAR_ULONGARRAY;
 	}
 	else if ((data_type == typeid(Tango::DevVarULong64Array)) ||
@@ -1266,20 +1106,20 @@ void TemplCommand::set_type(const type_info &data_type,Tango::CmdArgType &type)
 	{
 		cout4 << "Command : " << name << ", Unknown type" << endl;
 		TangoSys_OMemStream o;
-		
-		o << "Command " << name << " defined with an unsuported type"  << ends;
+
+		o << "Command " << name << " defined with an unsupported type"  << ends;
 		Except::throw_exception((const char *)"API_CmdArgumentTypeNotSupported",
 				      o.str(),(const char *)"TemplCommand::set_type");
-	}	
+	}
 }
 
 //+-------------------------------------------------------------------------
 //
 // method : 		is_allowed
-// 
-// description : 	Check if the command is allowed. If the pointer to 
-//			DeviceImpl class method "allowed_ptr" is null, the 
-//			default mode id used (command always executed). 
+//
+// description : 	Check if the command is allowed. If the pointer to
+//			DeviceImpl class method "allowed_ptr" is null, the
+//			default mode id used (command always executed).
 //			Otherwise, the method is executed
 //
 // input : - dev_ptr : pointer to the device on which the command must be
@@ -1292,11 +1132,7 @@ void TemplCommand::set_type(const type_info &data_type,Tango::CmdArgType &type)
 
 bool TemplCommand::is_allowed(DeviceImpl *dev_ptr,const CORBA::Any &in_any)
 {
-#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 	if (allowed_ptr == NULL)
-#else
-	if (allowed_ptr == (ALLO_PTR)NULL)
-#endif
 		return true;
 	else
 		return ((dev_ptr->*allowed_ptr)(in_any));
@@ -1305,7 +1141,7 @@ bool TemplCommand::is_allowed(DeviceImpl *dev_ptr,const CORBA::Any &in_any)
 //+-------------------------------------------------------------------------
 //
 // method : 		execute
-// 
+//
 // description : 	Execute the method associated with the command
 //			(stored in the exe_ptr data)
 //
@@ -1318,7 +1154,7 @@ bool TemplCommand::is_allowed(DeviceImpl *dev_ptr,const CORBA::Any &in_any)
 //
 //--------------------------------------------------------------------------
 
-CORBA::Any *TemplCommand::execute(DeviceImpl *dev_ptr,const CORBA::Any &in_any)
+CORBA::Any *TemplCommand::execute(DeviceImpl *dev_ptr,TANGO_UNUSED(const CORBA::Any &in_any))
 {
 
 //
