@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: device_4.cpp 15556 2011-02-11 08:25:58Z taurel $";
+static const char *RcsId = "$Id: device_4.cpp 20437 2012-05-31 12:33:29Z taurel $";
 
 //+============================================================================
 //
@@ -15,7 +15,7 @@ static const char *RcsId = "$Id: device_4.cpp 15556 2011-02-11 08:25:58Z taurel
 //
 // author(s) :          E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -26,16 +26,16 @@ static const char *RcsId = "$Id: device_4.cpp 15556 2011-02-11 08:25:58Z taurel
 // it under the terms of the GNU Lesser General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // Tango is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 15556 $
+// $Revision: 20437 $
 //
 // $Log$
 // Revision 3.12  2010/09/09 13:45:22  taurel
@@ -115,10 +115,10 @@ namespace Tango
 
 //+-------------------------------------------------------------------------
 //
-// method : 		Device_4Impl::Device_4Impl 
-// 
-// description : 	constructors for the device_impl class from the 
-//			class object pointer, the device name, 
+// method : 		Device_4Impl::Device_4Impl
+//
+// description : 	constructors for the device_impl class from the
+//			class object pointer, the device name,
 //			the description field, the state and the status.
 //			Device_4Impl inherits from DeviceImpl. These constructors
 //			simply call the correct DeviceImpl class
@@ -127,51 +127,43 @@ namespace Tango
 //--------------------------------------------------------------------------
 
 Device_4Impl::Device_4Impl(DeviceClass *device_class,string &dev_name):
-Device_3Impl(device_class,dev_name)
+Device_3Impl(device_class,dev_name),ext_4(Tango_NullPtr)
 {
 	ext->idl_version = 4;
-	
-	ext_4 = new Device_4ImplExt();
 }
-		
+
 Device_4Impl::Device_4Impl(DeviceClass *device_class,
 			   string &dev_name,
 			   string &desc):
-Device_3Impl(device_class,dev_name,desc)
+Device_3Impl(device_class,dev_name,desc),ext_4(Tango_NullPtr)
 {
 	ext->idl_version = 4;
-
-	ext_4 = new Device_4ImplExt();
 }
-		
+
 Device_4Impl::Device_4Impl(DeviceClass *device_class,
 	           	   string &dev_name,string &desc,
 	           	   Tango::DevState dev_state,string &dev_status):
-Device_3Impl(device_class,dev_name,desc,dev_state,dev_status),ext_4(NULL)
+Device_3Impl(device_class,dev_name,desc,dev_state,dev_status),ext_4(Tango_NullPtr)
 {
 	ext->idl_version = 4;
+}
 
-	ext_4 = new Device_4ImplExt();
-} 
-		   
 Device_4Impl::Device_4Impl(DeviceClass *device_class,
 	           	   const char *dev_name,
 			   const char *desc,
 	           	   Tango::DevState dev_state,
 	           	   const char *dev_status):
-Device_3Impl(device_class,dev_name,desc,dev_state,dev_status)
+Device_3Impl(device_class,dev_name,desc,dev_state,dev_status),ext_4(Tango_NullPtr)
 {
 	ext->idl_version = 4;
-
-	ext_4 = new Device_4ImplExt();
 }
 
 
 //+-------------------------------------------------------------------------
 //
 // method : 		Device_4Impl::read_attribute_history_4
-// 
-// description : 	CORBA operation to read attribute value history from 
+//
+// description : 	CORBA operation to read attribute value history from
 //			the polling buffer.
 //
 // argument: in :	- name : attribute name
@@ -186,21 +178,21 @@ throw(Tango::DevFailed, CORBA::SystemException)
 {
 	TangoMonitor &mon = get_poll_monitor();
 	AutoTangoMonitor sync(&mon);
-	
+
 	cout4 << "Device_4Impl::read_attribute_history_4 arrived, requested history depth = " << n << endl;
-	
+
 //
 // Record operation request in black box
 //
 
 	blackbox_ptr->insert_op(Op_Read_Attr_history_4);
-		
-	Tango::DevAttrHistory_4 *back;
+
+	Tango::DevAttrHistory_4 *back = NULL;
 	vector<PollObj *> &poll_list = get_poll_obj_list();
 	long nb_poll = poll_list.size();
-		
+
 //
-// Check that the device supports this attribute. This method returns an 
+// Check that the device supports this attribute. This method returns an
 // exception in case of unsupported attribute
 //
 
@@ -208,7 +200,7 @@ throw(Tango::DevFailed, CORBA::SystemException)
 
 	string attr_str(name);
 	transform(attr_str.begin(),attr_str.end(),attr_str.begin(),::tolower);
-		
+
 //
 // Check that the wanted attribute is polled.
 //
@@ -257,12 +249,12 @@ throw(Tango::DevFailed, CORBA::SystemException)
 //
 // Allocate memory for the returned value
 //
-			
+
 	try
 	{
 		back = new Tango::DevAttrHistory_4;
 		back->dates.length(n);
-	}	
+	}
 	catch (bad_alloc)
 	{
 		Except::throw_exception((const char *)"API_MemoryAllocation",
@@ -273,9 +265,9 @@ throw(Tango::DevFailed, CORBA::SystemException)
 //
 // Init attribute name in the returned structure
 //
-	
+
 	back->name = CORBA::string_dup(name);
-	
+
 //
 // Get attribute value history
 // Trick: To identify the state used as an attribute from a classical attribute with type
@@ -295,14 +287,14 @@ throw(Tango::DevFailed, CORBA::SystemException)
 //+-------------------------------------------------------------------------
 //
 // method : 		Device_2Impl::command_inout_history_4
-// 
-// description : 	CORBA operation to read command result history from 
+//
+// description : 	CORBA operation to read command result history from
 //			the polling buffer.
 //
 // argument: in :	- command : command name
 //			- n : history depth (in record number)
 //
-// This method returns a pointer to a DevCmdHistoryList with one 
+// This method returns a pointer to a DevCmdHistoryList with one
 // DevCmdHistory structure for each command record
 //
 //--------------------------------------------------------------------------
@@ -314,7 +306,7 @@ throw(Tango::DevFailed, CORBA::SystemException)
 	AutoTangoMonitor sync(&mon);
 
 	cout4 << "Device_4Impl::command_inout_history_4 arrived" << endl;
-	Tango::DevCmdHistory_4 *back;
+	Tango::DevCmdHistory_4 *back = NULL;
 
 	string cmd_str(command);
 
@@ -323,7 +315,7 @@ throw(Tango::DevFailed, CORBA::SystemException)
 //
 
 	blackbox_ptr->insert_op(Op_Command_inout_history_4);
-			
+
 //
 // Check that device supports this command. Also checks if the device
 // implements IDL >=3 when state or status history is requested
@@ -334,9 +326,9 @@ throw(Tango::DevFailed, CORBA::SystemException)
 
 	bool status_cmd = false;
 	bool state_cmd = false;
-	
+
 	long vers = get_dev_idl_version();
-	
+
 	if (vers >= 3)
 	{
 		if (cmd_str == "state")
@@ -344,7 +336,7 @@ throw(Tango::DevFailed, CORBA::SystemException)
 		else if (cmd_str == "status")
 			status_cmd = true;
 	}
-			
+
 //
 // Check that the command is polled
 //
@@ -369,10 +361,10 @@ throw(Tango::DevFailed, CORBA::SystemException)
 				{
 					polled_cmd = poll_list[i];
 				}
-			}			
+			}
 		}
 	}
-	
+
 	if (polled_cmd == NULL)
 	{
 		TangoSys_OMemStream o;
@@ -381,7 +373,7 @@ throw(Tango::DevFailed, CORBA::SystemException)
 					o.str(),
 					(const char *)"Device_4Impl::command_inout_history_4");
 	}
-		
+
 //
 // Check that some data is available in cache
 //
@@ -406,12 +398,12 @@ throw(Tango::DevFailed, CORBA::SystemException)
 //
 // Allocate memory for the returned value
 //
-		
+
 	try
 	{
 		back = new Tango::DevCmdHistory_4;
 		back->dates.length(n);
-	}	
+	}
 	catch (bad_alloc)
 	{
 		Except::throw_exception((const char *)"API_MemoryAllocation",
@@ -428,12 +420,12 @@ throw(Tango::DevFailed, CORBA::SystemException)
 
 	if ((state_cmd == true) || (status_cmd == true))
 	{
-		Tango::DevAttrHistory_4 *back_attr;
+		Tango::DevAttrHistory_4 *back_attr = NULL;
 		try
 		{
 			back_attr = new Tango::DevAttrHistory_4;
 			back_attr->dates.length(n);
-		}	
+		}
 		catch (bad_alloc)
 		{
 			Except::throw_exception((const char *)"API_MemoryAllocation",
@@ -444,7 +436,7 @@ throw(Tango::DevFailed, CORBA::SystemException)
 		if (status_cmd == true)
 		{
 			polled_cmd->get_attr_history(n,back_attr,Tango::DEV_STRING);
-			
+
 			back->dates = back_attr->dates;
 			back->errors = back_attr->errors;
 			back->errors_array = back_attr->errors_array;
@@ -460,9 +452,9 @@ throw(Tango::DevFailed, CORBA::SystemException)
 // Trick: To identify the state used as an attribute from a classical attribute with type
 // DEV_STATE, use DEV_VOID for state as data type.
 //
-			
+
 			polled_cmd->get_attr_history(n,back_attr,Tango::DEV_VOID);
-			
+
 			back->dates = back_attr->dates;
 			back->errors = back_attr->errors;
 			back->errors_array = back_attr->errors_array;
@@ -494,17 +486,17 @@ throw(Tango::DevFailed, CORBA::SystemException)
 				break;
 			}
 		}
-		polled_cmd->get_cmd_history(n,back,cmd_type);	
+		polled_cmd->get_cmd_history(n,back,cmd_type);
 	}
-	
-	cout4 << "Leaving Device_4Impl::command_inout_history_4 method" << endl;			
+
+	cout4 << "Leaving Device_4Impl::command_inout_history_4 method" << endl;
 	return back;
 }
 
 //+-------------------------------------------------------------------------
 //
-// method : 		Device_4Impl::command_inout_4 
-// 
+// method : 		Device_4Impl::command_inout_4
+//
 // description : 	Method called for each command_inout operation executed
 //			from any client on a Tango device version 4.
 //
@@ -518,7 +510,7 @@ CORBA::Any *Device_4Impl::command_inout_4(const char *in_cmd,
 throw (Tango::DevFailed, CORBA::SystemException)
 {
 	cout4 << "Device_4Impl::command_inout_4 arrived, source = " << source << ", command = " << in_cmd << endl;
-	
+
 //
 // Record operation request in black box
 //
@@ -532,7 +524,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 	bool state_status_cmd = false;
 	if ((TG_strcasecmp(in_cmd,"state") == 0) || (TG_strcasecmp(in_cmd,"status") == 0))
 		state_status_cmd = true;
-		
+
 //
 // Check if the device is locked and if it is valid
 // If the lock is not valid any more, clear it
@@ -542,20 +534,20 @@ throw (Tango::DevFailed, CORBA::SystemException)
 	{
 		check_lock("command_inout4",in_cmd);
 	}
-                
+
 //
 // Call the Device_2Impl command_inout
 //
 
 	ext->store_in_bb = false;
 	return (command_inout_2(in_cmd,in_data,source));
-		
+
 }
 
 //+-------------------------------------------------------------------------
 //
-// method : 		Device_4Impl::read_attributes_4 
-// 
+// method : 		Device_4Impl::read_attributes_4
+//
 // description : 	Method called for each read_attributes operation executed
 //			from any client on a Tango device version 4.
 //
@@ -574,7 +566,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 	if (ext->store_in_bb == true)
 		blackbox_ptr->insert_attr(names,cl_id,4,source);
 	ext->store_in_bb = true;
-	
+
 //
 // Build a sequence with the names of the attribute to be read.
 // This is necessary in case of the "AllAttr" shortcut is used
@@ -585,10 +577,10 @@ throw (Tango::DevFailed, CORBA::SystemException)
 	unsigned long nb_dev_attr = dev_attr->get_attr_nb();
 	Tango::DevVarStringArray real_names(nb_names);
 	unsigned long i;
-	
+
 	if (nb_names == 1)
 	{
-		string att_name(names[0]);				
+		string att_name(names[0]);
 		if (att_name == AllAttr)
 		{
 			real_names.length(nb_dev_attr);
@@ -614,12 +606,12 @@ throw (Tango::DevFailed, CORBA::SystemException)
 
 	Tango::AttributeValueList_4 *back;
 	Tango::AttributeValueList_3 *back3 = NULL;
-	
+
 	try
-	{	
+	{
 		Tango::AttributeValue_4 *l_back = new Tango::AttributeValue_4 [nb_names];
 		back = new Tango::AttributeValueList_4(nb_names,nb_names,l_back,true);
-		
+
 		for (unsigned long loop = 0;loop < nb_names;loop++)
 			(*back)[loop].value.union_no_data(true);
 	}
@@ -629,18 +621,18 @@ throw (Tango::DevFailed, CORBA::SystemException)
 				        (const char *)"Can't allocate memory in server",
 				        (const char *)"Device_4Impl::read_attributes_4");
 	}
-				
+
 //
 // If the source parameter specifies device, call the read_attributes method
 // which does not throw exception except for major fault (cant allocate
 // memory,....)
 //
 	vector <long> idx_in_back;
-	
+
 	if (source == Tango::DEV)
 	{
 		try
-		{	
+		{
 			AutoTangoMonitor sync(this);
 			read_attributes_no_except(real_names,back3,back,false,idx_in_back);
 		}
@@ -666,7 +658,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 	}
 	else
 	{
-	
+
 //
 // It must be now CACHE_DEVICE (no other choice), first try to get
 // values from cache
@@ -707,15 +699,15 @@ throw (Tango::DevFailed, CORBA::SystemException)
        				names_from_device.length(nb_attr);
        				names_from_device[nb_attr - 1] = real_names[i];
        				idx_in_back.push_back(i);
-				
+
        				(*back)[i].err_list.length(0);
        			}
-       		}	
+       		}
        	}
 
 		if (nb_attr != 0)
 		{
-				
+
 //
 // Try to get their values from device
 //
@@ -731,7 +723,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 				throw;
 			}
 		}
-	}	
+	}
 
 	return back;
 }
@@ -740,7 +732,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 //+-------------------------------------------------------------------------
 //
 // method : 		Device_4Impl::write_attributes_4
-// 
+//
 // description : 	CORBA operation to write attribute(s) value
 //
 // argument: in :	- values: The new attribute(s) value to be set.
@@ -753,7 +745,7 @@ throw (Tango::MultiDevFailed, Tango::DevFailed, CORBA::SystemException)
 {
 	AutoTangoMonitor sync(this,true);
 	cout4 << "Device_4Impl::write_attributes_4 arrived" << endl;
-	
+
 //
 // Record operation request in black box
 //
@@ -765,9 +757,9 @@ throw (Tango::MultiDevFailed, Tango::DevFailed, CORBA::SystemException)
 //
 // Check if the device is locked and by who
 //
-	
+
 	check_lock("write_attributes_4");
-	
+
 //
 // Call the Device_3Impl write_attributes
 //
@@ -780,7 +772,7 @@ throw (Tango::MultiDevFailed, Tango::DevFailed, CORBA::SystemException)
 //+-------------------------------------------------------------------------
 //
 // method :		Device_4Impl::set_attribute_config_4
-// 
+//
 // description :	CORBA operation to set attribute configuration locally
 //			and in the Tango database
 //
@@ -796,16 +788,16 @@ throw (Tango::DevFailed, CORBA::SystemException)
 {
 	AutoTangoMonitor sync(this,true);
 	cout4 << "Device_4Impl::set_attribute_config_4 arrived" << endl;
-	
+
 //
-// The attribute conf. is protected by two monitors. One protects access between 
+// The attribute conf. is protected by two monitors. One protects access between
 // get and set attribute conf. The second one protects access between set and
 // usage. This is the classical device monitor
 //
 
 	TangoMonitor &mon1 = get_att_conf_monitor();
 	AutoTangoMonitor sync1(&mon1);
-	
+
 //
 // Record operation request in black box
 //
@@ -815,9 +807,9 @@ throw (Tango::DevFailed, CORBA::SystemException)
 //
 // Check if the device is locked and by who
 //
-		
+
 	check_lock("set_attribute_config_4");
-		
+
 //
 // Call the Device_3Impl set_attribute_config
 //
@@ -829,9 +821,9 @@ throw (Tango::DevFailed, CORBA::SystemException)
 //+-------------------------------------------------------------------------
 //
 // method : 		Device_4Impl::write_read_attributes_4
-// 
+//
 // description : 	CORBA operation to write then read attribute(s) value
-//					Nowdays, it is possible to write then read attribute for
+//					Nowadays, it is possible to write then read attribute for
 //					only 1 attribute at a time even if the IDL is written for
 //					several attribute
 //
@@ -845,7 +837,7 @@ throw (Tango::MultiDevFailed,Tango::DevFailed, CORBA::SystemException)
 {
 	AutoTangoMonitor sync(this,true);
 	cout4 << "Device_4Impl::write_read_attributes_4 arrived" << endl;
-	
+
 //
 // Record operation request in black box
 //
@@ -855,47 +847,47 @@ throw (Tango::MultiDevFailed,Tango::DevFailed, CORBA::SystemException)
 //
 // Check if the device is locked and by who
 //
-			
+
 	check_lock("write_read_attributes_4");
 
 //
 // Check the attribute write type (only READ_WRITE or READ_WITH_WRITE allowed)
 //
-	
+
 	Tango::Attribute &att = dev_attr->get_attr_by_name(values[0].name);
 	Tango::AttrWriteType awt = att.get_writable();
 	if ((awt == Tango::READ) || (awt == Tango::WRITE))
 	{
 		TangoSys_OMemStream o;
 		o << "Attribute " << values[0].name << " is not a READ_WRITE or READ_WITH_WRITE attribute" << ends;
-		
-		Except::throw_exception((const char *)"API_IncompatibleAttrDataType",o.str(),
-								(const char *)"Device_4Impl::write_read_attribute_4");		
+
+		Except::throw_exception((const char *)"API_AttrNotWritable",o.str(),
+								(const char *)"Device_4Impl::write_read_attribute_4");
 	}
-	
+
 //
 // First, write the attribute
 //
-	
+
 	ext->store_in_bb = false;
 	write_attributes_4(values,cl_id);
 
 //
 // Now, read the attribute
 //
-	
+
 	Tango::DevVarStringArray att_name(1);
 	att_name.length(1);
 	att_name[0] = CORBA::string_dup(values[0].name);
 	Tango::ClntIdent dummy_cl_id;
 	Tango::CppClntIdent cci = 0;
 	dummy_cl_id.cpp_clnt(cci);
-	
+
 	ext->store_in_bb = false;
 	Tango::AttributeValueList_4 *read_val_ptr = read_attributes_4(att_name,Tango::DEV,dummy_cl_id);
-	
+
 	return read_val_ptr;
 }
-	
+
 
 } // End of Tango namespace
