@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: deviceclass.cpp 15556 2011-02-11 08:25:58Z taurel $\n$Name$";
+static const char *RcsId = "$Id: deviceclass.cpp 20437 2012-05-31 12:33:29Z taurel $\n$Name$";
 
 //+============================================================================
 //
@@ -13,7 +13,7 @@ static const char *RcsId = "$Id: deviceclass.cpp 15556 2011-02-11 08:25:58Z taur
 //
 // author(s) :		E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -24,165 +24,16 @@ static const char *RcsId = "$Id: deviceclass.cpp 15556 2011-02-11 08:25:58Z taur
 // it under the terms of the GNU Lesser General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // Tango is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 15556 $
-//
-// $Log$
-// Revision 1.39  2011/01/10 14:39:27  taurel
-// - Some compilation errors while compiling Tango 7.2.3
-//
-// Revision 1.38  2011/01/10 13:09:02  taurel
-// - No retry on command to get data for cache during DS startup
-// - Only three reties during DbDevExport
-// - Device are deleted by omniORB even if not exported into Tango database
-//
-// Revision 1.37  2010/09/24 14:06:15  taurel
-// - For Python DS, do not give full device ownership to the POA.
-// Otherwise, a python DS crashes at exit.
-//
-// Revision 1.36  2010/09/09 13:45:22  taurel
-// - Add year 2010 in Copyright notice
-//
-// Revision 1.35  2010/06/23 09:13:28  taurel
-// - Change some comments
-//
-// Revision 1.34  2010/06/21 12:38:23  taurel
-// - Implement a much faster server shutdown sequence
-//
-// Revision 1.33  2009/09/18 09:18:06  taurel
-// - End of attribute serialization implementation?
-//
-// Revision 1.32  2009/01/21 12:49:04  taurel
-// - Change CopyRights for 2009
-//
-// Revision 1.31  2008/10/06 15:00:36  taurel
-// - Changed the licensing info from GPL to LGPL
-//
-// Revision 1.30  2008/10/03 06:51:36  taurel
-// - Add some licensing info in each files
-//
-// Revision 1.29  2008/10/02 09:09:47  taurel
-// - First implementation of multiple polling thread(s)
-//
-// Revision 1.28  2008/09/04 07:37:05  taurel
-// - Fix bug in memorized attributes
-// - Changes for the new IDL 4
-//
-// Revision 1.27  2008/05/20 12:44:11  taurel
-// - Commit after merge with release 7 branch
-//
-// Revision 1.26  2008/03/11 14:38:25  taurel
-// - Apply patches from Frederic Picca about compilation with gcc 4.2
-//
-// Revision 1.25  2008/01/25 15:44:51  taurel
-// - Some changes in the Db cache
-// - A lighter system to shutdown DS in case of dynamic attribute
-//
-// Revision 1.24  2007/12/12 10:17:18  taurel
-// - Db calls during DS startup has a separate timeout and some retries
-// Revision 1.23.2.1  2008/02/07 15:58:14  taurel
-// - First implementation of the Controlled Access done
-//
-// Revision 1.23  2007/11/08 12:03:44  taurel
-// - Start implementing user interceptors
-// - Fix bug in poll thread pproperty management when removing polling object
-// - Set a database timeout to 6 sec
-//
-// Revision 1.22  2007/10/16 08:22:51  taurel
-// - Add management of the TC connection establishment timeout for DB access
-// - Add DB server cache in DS used during DS startup sequence
-// - Comment out the sleep time during DS startup sequence
-//
-// Revision 1.21  2007/05/22 12:48:40  taurel
-// - Clarified an error message
-//
-// Revision 1.20  2007/05/17 07:59:07  taurel
-// - The polling is not configured via a separate thread any more. The polling thread add_obj_polling method has been modified to support a parameter telling to the polling thread when it has to polled the object.
-// Add device name in monitor print message
-// Add device_destroyer method in DeviceClass class
-//
-// Revision 1.19  2007/05/11 07:29:29  taurel
-// - Added device_destroyer() methods. These method are doing the opposite of
-// the device_factory() one. They remove (and delete) a device from the list
-// of available device in the class. They do not mmodify the DB
-//
-// Revision 1.18  2007/04/20 14:40:29  taurel
-// - Ported to Windows 64 bits x64 architecture
-//
-// Revision 1.17  2007/04/16 14:56:36  taurel
-// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
-// - Ported to omniORB4.1
-// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
-// - Added a new filterable field in the archive event
-//
-// Revision 1.16  2007/03/06 08:18:04  taurel
-// - Added 64 bits data types for 64 bits computer...
-//
-// Revision 1.15  2006/11/20 16:26:37  taurel
-// - Small changes when removing servant in case the device is not exported
-//
-// Revision 1.14  2006/06/21 14:52:19  taurel
-// - Just fixing a logging print message
-//
-// Revision 1.13  2006/06/12 14:22:33  jensmeyer
-// Corrected error handling for during initialisation of memorised attributes.
-//
-// Revision 1.12  2006/05/18 08:52:37  taurel
-// - Miscellaneous changes due to Python device server ported to Windows
-// - Fix some bugs discovered by Windows VC8 using the test suite
-// - Update Windows resource file include path
-// - Fix some Windows VC8 warnings
-//
-// Revision 1.11  2006/03/09 08:18:06  taurel
-// - Change the order of device server destruction. First devices then class.
-// It was the contrary....
-//
-// Revision 1.10  2006/02/20 08:51:48  taurel
-// - A small edit mistake in delete_dev method...
-//
-// Revision 1.9  2006/02/20 08:30:00  taurel
-// - Add a new method in DeviceClass to delete a device at run-time
-//
-// Revision 1.8  2006/02/17 16:53:04  jensmeyer
-// Corrections when porting to VC7 under windows
-//
-// Revision 1.7  2006/01/27 14:27:10  taurel
-// - Fix a severe incompatibility problem introduced by all the modifs done for
-// PY DS
-// - Duplicate some EventSupplier class methods (instead of using template) in order to be able to generate Tango shared library on Suse 9.3
-//
-// Revision 1.6  2006/01/20 15:30:13  taurel
-// - Fixes in the default command management
-//
-// Revision 1.5  2006/01/20 08:30:05  taurel
-// - Added necessary changes to support Device server written in Python
-//
-// Revision 1.4  2005/11/07 12:15:00  jensmeyer
-// Corrected initialisation of memorised attributes. No longer exits when receiving an
-// exception during initialisation. Also added an option to only initialise the attribute setpoint and not writing to the attribute.
-// set_memorized_init(false) -> only applies setpoint.
-// set_memorized_init(true) -> also writes setpoint value to the attribute.
-//
-// Revision 1.3  2005/06/29 08:31:16  taurel
-// - Last commit before release 5.2 ?
-//
-// Revision 1.2  2005/01/13 08:31:12  taurel
-// - Merge trunk with Release_5_0 from brach Release_5_branch
-//
-// Revision 1.1.2.2  2004/10/27 05:59:46  taurel
-// - Some minor changes to compile on all our supported platforms
-//
-// Revision 1.1.2.1  2004/10/22 11:28:30  taurel
-// Splitted device.cpp/device.h files in two sets device.cpp/device.h for the DeviceImpl
-// class and deviceclass.cpp/deviceclass.h for the DeviceClass class
+// $Revision: 20437 $
 //
 //-============================================================================
 
@@ -192,6 +43,7 @@ static const char *RcsId = "$Id: deviceclass.cpp 15556 2011-02-11 08:25:58Z taur
 
 #include <tango.h>
 #include <new>
+#include <iterator>
 
 #include <basiccommand.h>
 #include <blackbox.h>
@@ -217,29 +69,23 @@ static void lower_cmd_name(string &cmd)
 //+----------------------------------------------------------------------------
 //
 // method :		DeviceClass::DeviceClass(string &s)
-// 
-// description :	DeviceClass constructor. Protected method which will 
+//
+// description :	DeviceClass constructor. Protected method which will
 //			be called automatically by the compiler.
 //
 //-----------------------------------------------------------------------------
 
-DeviceClass::DeviceClass(string &s):name(s)
+DeviceClass::DeviceClass(string &s):name(s),ext(new DeviceClassExt)
 {
 
-//
-// Create the DeviceClassExt instance
-//
-
-	ext = new DeviceClassExt;
-	
 //
 // Create the associated DbClass object
 //
 
 	db_class = new DbClass(name,Tango::Util::instance()->get_database());
-		
+
 //
-// initialise command_list with State, Status and Init 
+// initialise command_list with State, Status and Init
 //
 
 	try
@@ -264,9 +110,9 @@ DeviceClass::DeviceClass(string &s):name(s)
 //
 // Retrieve basic class resource
 //
-	
+
 	get_class_system_resource();
-	
+
 //
 // Create the multi class attribute object
 //
@@ -277,14 +123,14 @@ DeviceClass::DeviceClass(string &s):name(s)
 // Give a default value for device type
 //
 
-	type = DescNotSet;
-			
+	type = NotSet;
+
 }
 
 //+----------------------------------------------------------------------------
 //
 // method :		DeviceClass::get_class_system_resource(string &s)
-// 
+//
 // description :	Method to retrieve some basic class resource(s)
 //			The resource to be retrived are :
 //				- The class doc URL
@@ -303,11 +149,13 @@ void DeviceClass::get_class_system_resource()
 	{
 		Database *db = tg->get_database();
 		DbData db_data;
-		
+
 		db_data.push_back(DbDatum("doc_url"));
 		db_data.push_back(DbDatum("cvs_tag"));
 		db_data.push_back(DbDatum("cvs_location"));
 		db_data.push_back(DbDatum("AllowedAccessCmd"));
+		db_data.push_back(DbDatum("svn_tag"));
+		db_data.push_back(DbDatum("svn_location"));
 
 		try
 		{
@@ -322,22 +170,22 @@ void DeviceClass::get_class_system_resource()
 					o.str(),
 					(const char *)"DeviceClass::get_class_system_resource");
 		}
-		
+
 		if (db_data[1].is_empty() == false)
 			db_data[1] >> ext->cvs_tag;
 		if (db_data[2].is_empty() == false)
 			db_data[2] >> ext->cvs_location;
-		
+
 //
 // Init allowed commands vector (in lowercase letters)
 //
-			
+
 		if (db_data[3].is_empty() == false)
 		{
 			db_data[3] >> allowed_cmds;
 			for_each(allowed_cmds.begin(),allowed_cmds.end(),lower_cmd_name);
 		}
-					
+
 		if (db_data[0].is_empty() == true)
 		{
 			cout4 << "doc_url property for class " << name << " is not defined in database" << endl;
@@ -346,15 +194,15 @@ void DeviceClass::get_class_system_resource()
 				db->get_class_property("Default",db_data,tg->get_db_cache());
 			}
 			catch (Tango::DevFailed &)
-			{			
+			{
 				TangoSys_OMemStream o;
 				o << "Database error while trying to retrieve properties for class " << name.c_str() << ends;
-					
+
 				Except::throw_exception((const char *)"API_DatabaseAccess",
 							o.str(),
 							(const char *)"DeviceClass::get_class_system_resource");
-			}			
-			
+			}
+
 			if (db_data[0].is_empty() == true)
 			{
 				doc_url = DefaultDocUrl;
@@ -364,22 +212,27 @@ void DeviceClass::get_class_system_resource()
 		}
 		else
 			db_data[0] >> doc_url;
-		
+
+		if (db_data[4].is_empty() == false)
+			db_data[4] >> ext->svn_tag;
+		if (db_data[5].is_empty() == false)
+			db_data[5] >> ext->svn_location;
+
 	}
 	else
 	{
 		doc_url = DefaultDocUrl;
-	}	
+	}
 }
 
 //+----------------------------------------------------------------------------
 //
 // method : 		DeviceClass::set_memorized_values()
-// 
+//
 // description : 	Write the memorized attribute with the value
 //			stored in database
 //
-// in :			all : Flag set to true if memorized values must be 
+// in :			all : Flag set to true if memorized values must be
 //			      applied to all class devices
 //			idx : Index of the device in the device_list vector
 //			      of the device for which memorized values must
@@ -390,15 +243,15 @@ void DeviceClass::get_class_system_resource()
 void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 {
 	cout4 << "Entering DeviceClass::set_memorized_values() method" << endl;
-	
+
 	short sh;
 	Tango::DevVarShortArray sh_seq(1);
 	sh_seq.length(1);
-						
+
 	Tango::DevLong lg;
 	Tango::DevVarLongArray lg_seq(1);
 	lg_seq.length(1);
-		
+
 	double db;
 	Tango::DevVarDoubleArray db_seq(1);
 	db_seq.length(1);
@@ -410,11 +263,11 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 	float fl;
 	Tango::DevVarFloatArray fl_seq(1);
 	fl_seq.length(1);
-						
+
 	Tango::DevBoolean boo;
 	Tango::DevVarBooleanArray boo_seq(1);
 	boo_seq.length(1);
-		
+
 	Tango::DevUShort ush;
 	Tango::DevVarUShortArray ush_seq(1);
 	ush_seq.length(1);
@@ -422,7 +275,19 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 	Tango::DevUChar uch;
 	Tango::DevVarCharArray uch_seq(1);
 	uch_seq.length(1);
-		
+
+	Tango::DevULong ulg;
+	Tango::DevVarULongArray ulg_seq(1);
+	ulg_seq.length(1);
+
+	Tango::DevLong64 lg64;
+	Tango::DevVarLong64Array lg64_seq(1);
+	lg64_seq.length(1);
+
+	Tango::DevULong64 ulg64;
+	Tango::DevVarULong64Array ulg64_seq(1);
+	ulg64_seq.length(1);
+
 //
 // Set loop start and stop limits
 //
@@ -438,7 +303,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 		start = idx;
 		stop = idx + 1;
 	}
-								
+
 	for (unsigned long i = start;i < stop;i++)
 	{
 
@@ -449,29 +314,29 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 
 		if (device_list[i]->get_dev_idl_version() < 3)
 			continue;
-				
+
 //
 // Get list of device writable attributes
 //
 
 		AttributeValueList att_val(10);
 		vector<long> &att_list = device_list[i]->get_device_attr()->get_w_attr_list();
-		
+
 		long nb_wr = 0;
 		for (unsigned long j = 0;j < att_list.size();j++)
 		{
-						
+
 			WAttribute &att = device_list[i]->get_device_attr()->get_w_attr_by_ind(att_list[j]);
-			
+
 			if (att.is_memorized() == true)
 			{
 
 				string &mem_value = att.get_mem_value();
-				if ((mem_value != MemNotUsed) || (from_init == true))
+                if (mem_value != MemNotUsed)
 				{
 					nb_wr++;
 					att_val.length(nb_wr);
-							
+
 //
 // In order to not send
 // a new time the already memorized value into db, mark it as not memorized
@@ -479,7 +344,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 //
 
 					att.set_memorized(false);
-					
+
 //
 // The memorized value gotten from db is a string, we need to convert this string
 // to its real type before inserting it into an Any
@@ -488,7 +353,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 					TangoSys_MemStream str;
 					if (from_init == false)
 						str << mem_value << ends;
-					
+
 					try
 					{
 						switch (att.get_data_type())
@@ -502,7 +367,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 							}
 							else
 								att.get_write_value(sh);
-							
+
 							sh_seq[0] = sh;
 							att_val[nb_wr - 1].value <<= sh_seq;
 							break;
@@ -516,7 +381,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 							}
 							else
 								att.get_write_value(lg);
-								
+
 							lg_seq[0] = lg;
 							att_val[nb_wr - 1].value <<= lg_seq;
 							break;
@@ -530,7 +395,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 							}
 							else
 								att.get_write_value(db);
-							
+
 							db_seq[0] = db;
 							att_val[nb_wr - 1].value <<= db_seq;
 							break;
@@ -539,7 +404,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 							if (from_init == false)
 							{
 								att.set_write_value(mem_value);
-							
+
 								str_seq[0] = CORBA::string_dup(mem_value.c_str());
 							}
 							else
@@ -559,7 +424,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 							}
 							else
 								att.get_write_value(fl);
-								
+
 							fl_seq[0] = fl;
 							att_val[nb_wr - 1].value <<= fl_seq;
 							break;
@@ -567,23 +432,13 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 						case Tango::DEV_BOOLEAN:
 							if (from_init == false)
 							{
-#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 								if (!(str >> boolalpha >> boo))
 									throw_mem_value(device_list[i],att);
-#else
-								transform(mem_value.begin(),mem_value.end(),mem_value.begin(),::tolower);
-								if (mem_value == "true")
-									boo = true;
-								else if (mem_value == "false")
-									boo = false;
-								else
-									throw_mem_value(device_list[i],att);
-#endif
 								att.set_write_value(boo);
 							}
 							else
 								att.get_write_value(boo);
-							
+
 							boo_seq[0] = boo;
 							att_val[nb_wr - 1].value <<= boo_seq;
 							break;
@@ -597,7 +452,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 							}
 							else
 								att.get_write_value(ush);
-								
+
 							ush_seq[0] = ush;
 							att_val[nb_wr - 1].value <<= ush_seq;
 							break;
@@ -611,61 +466,103 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 							}
 							else
 								att.get_write_value(uch);
-								
+
 							uch_seq[0] = uch;
 							att_val[nb_wr - 1].value <<= uch_seq;
 							break;
+
+						case Tango::DEV_ULONG:
+							if (from_init == false)
+							{
+								if (!(str >> ulg))
+									throw_mem_value(device_list[i],att);
+								att.set_write_value(ulg);
+							}
+							else
+								att.get_write_value(ulg);
+
+							ulg_seq[0] = ulg;
+							att_val[nb_wr - 1].value <<= ulg_seq;
+							break;
+
+						case Tango::DEV_LONG64:
+							if (from_init == false)
+							{
+								if (!(str >> lg64))
+									throw_mem_value(device_list[i],att);
+								att.set_write_value(lg64);
+							}
+							else
+								att.get_write_value(lg64);
+
+							lg64_seq[0] = lg64;
+							att_val[nb_wr - 1].value <<= lg64_seq;
+							break;
+
+						case Tango::DEV_ULONG64:
+							if (from_init == false)
+							{
+								if (!(str >> ulg64))
+									throw_mem_value(device_list[i],att);
+								att.set_write_value(ulg64);
+							}
+							else
+								att.get_write_value(ulg64);
+
+							ulg64_seq[0] = ulg64;
+							att_val[nb_wr - 1].value <<= ulg64_seq;
+							break;
 						}
 
-//						
+//
 // Check the initialisation flag for memorized attributes.
 // The the flag is false, do not add the element to the att_val
 // vector. This avoids a call to write the memorozied value to the attribute.
 //
-					
+
 					if ( att.is_memorized_init() == false )
-						{
+                    {
 						nb_wr--;
 						att_val.length(nb_wr);
 						// reset memorized flag
 						att.set_memorized(true);
-						}
+                    }
 					else
-						{
+                    {
 //
 // Init the AttributeValue structure
 //
-					
+
 						att_val[nb_wr - 1].name = CORBA::string_dup(att.get_name().c_str());
 						att_val[nb_wr - 1].dim_x = 1;
-						att_val[nb_wr - 1].dim_y = 0;						
+						att_val[nb_wr - 1].dim_y = 0;
 						att_val[nb_wr - 1].quality = Tango::ATTR_VALID;
 						}
 					}
-						
+
 					catch (Tango::DevFailed &e)
 					{
 						cout3 << "Cannot configure setpoint value for memorized attribute " << att.get_name() << endl;
 						Tango::Except::print_exception(e);
-						
+
 						nb_wr--;
 						att_val.length(nb_wr);
 						// reset memorized flag
 						att.set_memorized(true);
-					}						
+					}
 				}
 			}
 		}
-			
-																	
+
+
 		if (nb_wr != 0)
 		{
-		
+
 //
 // Write attribute values. Re-throw exception if any but reset memorized flags before the
 // re-throw.
 //
-			
+
 			try
 			{
 				cout4 << "Writing data for " << att_val.length() << " attribute(s) for device " << device_list[i]->get_name() << endl;
@@ -686,7 +583,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 			catch (Tango::MultiDevFailed &e)
 			{
 				cout3 << "Cannot write setpoint(s) value for memorized attribute(s) " << device_list[i]->get_name() << endl;
-				
+
 				Tango::NamedDevFailedList e_list (e, device_list[i]->get_name(), (const char *)"DeviceClass::set_memorized_values()",
 					       			(const char *)"API_AttributeFailed");
 				Tango::Except::print_exception(e_list);
@@ -700,7 +597,7 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 				//	       			(const char *)"DeviceClass::set_memorized_values()",
 				//	       			(const char *)"API_AttributeFailed");
 			}
-						
+
 //
 // Reset memorized flags
 //
@@ -714,13 +611,13 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 	}
 
 	cout4 << "Leaving DeviceClass::set_memorized_values() method" << endl;
-	
+
 }
 
 //+----------------------------------------------------------------------------
 //
 // method : 		DeviceClass::throw_mem_value()
-// 
+//
 // description : 	Write the memorized attribute with the value
 //			stored in database
 //
@@ -729,13 +626,13 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 void DeviceClass::throw_mem_value(DeviceImpl *dev,Attribute &att)
 {
 	TangoSys_OMemStream o;
-	
+
 	o << "Memorized value for attribute ";
 	o << att.get_name();
 	o << " (device ";
 	o << dev->get_name();
 	o << ") is in an incorrect format !" << ends;
-	
+
 	Except::throw_exception((const char *)"API_AttrWrongMemValue",
 				o.str(),
 				(const char *)"DeviceClass::set_memorized_values");
@@ -744,7 +641,7 @@ void DeviceClass::throw_mem_value(DeviceImpl *dev,Attribute &att)
 //+----------------------------------------------------------------------------
 //
 // method : 		DeviceClass::~DeviceClass(string &s)
-// 
+//
 // description :	DeviceClass destructor.
 //
 //-----------------------------------------------------------------------------
@@ -758,7 +655,7 @@ DeviceClass::~DeviceClass()
 //
 
 	delete db_class;
-		
+
 //
 // Destroy the device list
 //
@@ -771,8 +668,8 @@ DeviceClass::~DeviceClass()
 
 		unsigned long nb_dev = device_list.size();
 		for (i = 0;i < nb_dev;i++)
-		{	
-				
+		{
+
 //
 // Clear vectors used to memorize info used to clean db
 // in case of devices with dyn attr removed during device
@@ -787,21 +684,20 @@ DeviceClass::~DeviceClass()
 //
 // Delete device
 //
-					
+
 			delete_dev(0,tg,r_poa);
-					
+
 //
 // Clean-up db (dyn attribute)
 //
-					
 
 			if (tg->get_polled_dyn_attr_names().size() != 0)
-				tg->clean_attr_polled_prop();			
+				tg->clean_attr_polled_prop();
 			if (tg->get_all_dyn_attr_names().size() != 0)
 				tg->clean_dyn_attr_prop();
 
 			vector<DeviceImpl *>::iterator it = device_list.begin();
-			device_list.erase(it);		
+			device_list.erase(it);
 		}
 		device_list.clear();
 		CORBA::release(r_poa);
@@ -820,27 +716,29 @@ DeviceClass::~DeviceClass()
 //
 
 	delete class_attr;
-		
+
 //
 // Unregister the class from signal handler
 //
 
 	DServerSignal::instance()->unregister_class_signal(this);
-	
+
 //
 // Delete the class extension object
 //
 
+#ifndef HAS_UNIQUE_PTR
 	delete ext;
-	
-	cout4 << "Leaving DeviceClass destructor for class " << name << endl;	
+#endif
+
+	cout4 << "Leaving DeviceClass destructor for class " << name << endl;
 }
 
 
 //+----------------------------------------------------------------------------
 //
 // method : 		DeviceClass::delete_dev()
-// 
+//
 // description :	delete a device from the class device list
 //
 //-----------------------------------------------------------------------------
@@ -859,7 +757,7 @@ void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_
 	{
 		device_list[idx]->stop_polling(false);
 	}
-	
+
 //
 // Deactivate the CORBA object
 //
@@ -869,7 +767,7 @@ void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_
 
 	if (exported_device == true)
 		r_poa->deactivate_object(device_list[idx]->get_obj_id().in());
-			
+
 //
 // Remove the servant.
 // For C++ device, this will be automatically done by the POA when the last executing call
@@ -882,14 +780,37 @@ void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_
 		dev_3->delete_dev();
 	}
 
-	cout4 << "Leaving DeviceClass delete_dev" << endl;	
+//
+// Wait for CORBA to call the device dtor
+//
+
+    if (device_list[idx] != NULL && exported_device == true)
+    {
+#ifdef _TG_WINDOWS_
+        while (device_list[idx] != NULL)
+        {
+            Sleep(10);
+        }
+#else
+        struct timespec ts;
+        ts.tv_sec = 0;
+        ts.tv_nsec = 10000000;
+
+        while (device_list[idx] != NULL)
+        {
+            nanosleep(&ts,NULL);
+        }
+#endif
+    }
+
+	cout4 << "Leaving DeviceClass delete_dev" << endl;
 }
 
 
 //+-------------------------------------------------------------------------
 //
 // method :		DeviceClass::register_signal
-// 
+//
 // description :	Method to register a class on a signal. When the
 //			signal is sent to the process, the signal_handler
 //			method of this class will be executed
@@ -905,7 +826,7 @@ void DeviceClass::register_signal(long signo)
 	cout4 << "DeviceClass::register_signal() arrived for signal " << signo << endl;
 
 	DServerSignal::instance()->register_class_signal(signo,this);
-		
+
 	cout4 << "Leaving DeviceClass::register_signal method()" << endl;
 }
 #else
@@ -914,7 +835,7 @@ void DeviceClass::register_signal(long signo,bool handler)
 	cout4 << "DeviceClass::register_signal() arrived for signal " << signo << endl;
 
 	DServerSignal::instance()->register_class_signal(signo,handler,this);
-		
+
 	cout4 << "Leaving DeviceClass::register_signal method()" << endl;
 }
 #endif
@@ -922,7 +843,7 @@ void DeviceClass::register_signal(long signo,bool handler)
 //+-------------------------------------------------------------------------
 //
 // method :		DeviceClass::unregister_signal
-// 
+//
 // description :	Method to unregister a class on a signal.
 //
 // in : 		signo : The signal number
@@ -935,14 +856,14 @@ void DeviceClass::unregister_signal(long signo)
 	cout4 << "DeviceClass::unregister_signal() arrived for signal " << signo << endl;
 
 	DServerSignal::instance()->unregister_class_signal(signo,this);
-		
+
 	cout4 << "Leaving DeviceClass::unregister_signal method()" << endl;
 }
 
 //+-------------------------------------------------------------------------
 //
 // method :		DeviceClass::signal_handler
-// 
+//
 // description :	This is the signal handler for the class. This method
 //			is defined as virtual and therefore, can be redefined
 //			by DS programmers in their own classes derived from
@@ -955,7 +876,7 @@ void DeviceClass::unregister_signal(long signo)
 void DeviceClass::signal_handler(long signo)
 {
 	cout4 << "DeviceClass::signal_handler() arrived for signal " << signo << endl;
-	
+
 	cout4 << "Leaving DeviceClass::signal_handler method()" << endl;
 }
 
@@ -963,8 +884,8 @@ void DeviceClass::signal_handler(long signo)
 //+-------------------------------------------------------------------------
 //
 // method :		DeviceClass::export_device()
-// 
-// description :	This method exports a device to the outside world. 
+//
+// description :	This method exports a device to the outside world.
 //			This is done by sending its CORBA network parameter
 //			(mainly the IOR) to the Tango database
 //
@@ -982,11 +903,11 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 	cout4 << "DeviceClass::export_device() arrived" << endl;
 
 	Device_var d;
-		
+
 	if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
 	{
-			
-// 
+
+//
 // Activate the CORBA object incarnated by the dev C++ object
 // Also call _remove_ref to give POA the full ownership of servant
 //
@@ -1020,11 +941,11 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 	}
 	else
 	{
-	
+
 //
 // For server started without db usage (Mostly the database server). In this case,
 // it is necessary to create our own CORBA object id and to bind it into the
-// OOC Boot Manager for access through a stringified object reference 
+// OOC Boot Manager for access through a stringified object reference
 // constructed using the corbaloc style
 // The API will try to connect to device using lower case letters.
 // Register device in POA with lower case letters
@@ -1050,13 +971,13 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 						o.str(),
 						(const char *)"DeviceClass::export_device");
 		}
-		
+
 		d = dev->_this();
-		dev->set_obj_id(id);		
+		dev->set_obj_id(id);
 		dev->set_d_var(Tango::Device::_duplicate(d));
-		dev->_remove_ref();	
+		dev->_remove_ref();
 	}
-			
+
 //
 // Prepare sent parameters and allocate mem for them
 //
@@ -1064,13 +985,13 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 	if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
 	{
 		Tango::Util *tg = Tango::Util::instance();
-		CORBA::ORB_ptr orb_ptr = tg->get_orb(); 
-		
+		CORBA::ORB_ptr orb_ptr = tg->get_orb();
+
 		char *s = orb_ptr->object_to_string(d);
 		string ior_string(s);
 
 		Tango::DbDevExportInfo exp;
-			
+
 		exp.name = dev->get_name();
 		exp.ior = ior_string;
 		exp.host = tg->get_host_name();
@@ -1082,7 +1003,7 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 // We are still in the server starting phase. Therefore, the db timeout is still high (13 sec the 07/01/2011)
 // with 3 retries in case of timeout
 //
-		
+
 		try
 		{
 			tg->get_database()->export_device(exp);
@@ -1094,17 +1015,17 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 			CORBA::string_free(s);
 			throw;
 		}
-		
+
 		CORBA::release(orb_ptr);
 		CORBA::string_free(s);
 	}
-	
+
 //
 // Set the DeviceImpl exported flag to true
 //
 
 	dev->set_exported_flag(true);
-			
+
 	cout4 << "Leaving DeviceClass::export_device method()" << endl;
 }
 
@@ -1112,7 +1033,7 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 //+----------------------------------------------------------------------------
 //
 // method :		DeviceClass::command_handler(string &s)
-// 
+//
 // description :	Command handler which is called by Device
 //			when a command is received. It will check
 //			to see if the command is implemented. If
@@ -1125,16 +1046,16 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 //-----------------------------------------------------------------------------
 
 CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,const CORBA::Any &in_any)
-{	
+{
 	CORBA::Any *ret = NULL;
 	vector<Command *>::iterator i_cmd;
-	
+
 	string command_lower(command);
 
 	cout4 << "Entering DeviceClass::command_handler() method" << endl;
 
 	transform(command_lower.begin(),command_lower.end(),command_lower.begin(),::tolower);
-	
+
 	for (i_cmd = command_list.begin();i_cmd < command_list.end();++i_cmd)
 	{
 		if ((*i_cmd)->get_lower_name() == command_lower)
@@ -1170,20 +1091,20 @@ CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,cons
 
 	if (i_cmd == command_list.end())
 	{
-	
+
 		cout3 << "DeviceClass::command_handler(): command " << command << " not found" << endl;
 
 
 		Command *def_cmd = get_default_command();
 		if (def_cmd != NULL)
 		{
-		
+
 //
 // Set name in default command object
 //
 
 			def_cmd->set_name(command);
-					
+
 //
 // Call the always executed method
 //
@@ -1202,7 +1123,7 @@ CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,cons
 						o.str(),
 						(const char *)"DeviceClass::command_handler");
 			}
-	
+
 //
 // Execute command
 //
@@ -1212,9 +1133,9 @@ CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,cons
 		}
 		else
 		{
-		
-					
-//		
+
+
+//
 // Throw an exception to client
 //
 
@@ -1226,8 +1147,8 @@ CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,cons
 					      (const char *)"DeviceClass::command_handler");
 		}
 	}
-	
-	cout4 << "Leaving DeviceClass::command_handler() method" << endl;	
+
+	cout4 << "Leaving DeviceClass::command_handler() method" << endl;
 	return ret;
 }
 
@@ -1235,7 +1156,7 @@ CORBA::Any *DeviceClass::command_handler(DeviceImpl *device,string &command,cons
 //+----------------------------------------------------------------------------
 //
 // method :		DeviceClass::add_wiz_dev_prop()
-// 
+//
 // description :	Method to add a device property definition for the
 //			device  wizard
 //
@@ -1254,7 +1175,7 @@ void DeviceClass::add_wiz_dev_prop(string &p_name,string &desc,string &def)
 
 	string name_low = p_name;
 	transform(name_low.begin(),name_low.end(),name_low.begin(),::tolower);
-	
+
 //
 // Check that this property is not already in the vector
 //
@@ -1267,7 +1188,7 @@ void DeviceClass::add_wiz_dev_prop(string &p_name,string &desc,string &def)
 		if (tmp_name == name_low)
 			break;
 	}
-	
+
 	if (ite != wiz_dev_prop.end())
 	{
 		TangoSys_OMemStream o;
@@ -1277,7 +1198,7 @@ void DeviceClass::add_wiz_dev_prop(string &p_name,string &desc,string &def)
 					o.str(),
 					(const char *)"DeviceClass::add_wiz_dev_prop");
 	}
-	
+
 //
 // Insert data in vector
 //
@@ -1299,7 +1220,7 @@ void DeviceClass::add_wiz_dev_prop(string &p_name,string &desc)
 //+----------------------------------------------------------------------------
 //
 // method :		DeviceClass::add_wiz_class_prop()
-// 
+//
 // description :	Method to add a class property definition for the
 //			device  wizard
 //
@@ -1317,7 +1238,7 @@ void DeviceClass::add_wiz_class_prop(string &p_name,string &desc,string &def)
 
 	string name_low = p_name;
 	transform(name_low.begin(),name_low.end(),name_low.begin(),::tolower);
-	
+
 //
 // Check that this property is not already in the vector
 //
@@ -1340,7 +1261,7 @@ void DeviceClass::add_wiz_class_prop(string &p_name,string &desc,string &def)
 					o.str(),
 					(const char *)"DeviceClass::add_wiz_dev_prop");
 	}
-	
+
 //
 // Insert data in vector
 //
@@ -1360,7 +1281,7 @@ void DeviceClass::add_wiz_class_prop(string &p_name,string &desc)
 //+----------------------------------------------------------------------------
 //
 // method :		DeviceClass::device_destroyer
-// 
+//
 // description :	Method to remove and delete a device from the running devices
 //					belonging to the Tango class
 //
@@ -1370,27 +1291,27 @@ void DeviceClass::add_wiz_class_prop(string &p_name,string &desc)
 
 void DeviceClass::device_destroyer(const string &dev_name)
 {
-	
+
 //
 // Check that the class know this device
 //
-  
-  	unsigned long k;				
+
+  	unsigned long k;
   	for (k = 0;k < device_list.size();k++)
   	{
   		if (device_list[k]->get_name() == dev_name)
   			break;
   	}
-  
+
   	if (k == device_list.size())
   	{
 		TangoSys_OMemStream o;
 		o << "Device " << dev_name << " not in Tango class device list!" << ends;
-		
+
   		Tango::Except::throw_exception((const char *)"API_CantDestroyDevice",o.str(),
   					       (const char *)"DeviceClass::device_destroyer");
   	}
-	
+
 //
 // Check if the device is polled
 // If yes, ask polling thread to stop polling it
@@ -1400,20 +1321,20 @@ void DeviceClass::device_destroyer(const string &dev_name)
 	{
 		device_list[k]->stop_polling();
 	}
-	
-			
+
+
 //
 // Delete the device
 //
- 
+
 	Tango::Util *tg = Tango::Util::instance();
-	 			
+
 	PortableServer::POA_ptr r_poa = tg->get_poa();
 	delete_dev(k,tg,r_poa);
 	vector<DeviceImpl *>::iterator it = device_list.begin();
 	it += k;
 	device_list.erase(it);
-	CORBA::release(r_poa);  	  				
+	CORBA::release(r_poa);
 }
 
 void DeviceClass::device_destroyer(const char *dev_name)
@@ -1425,8 +1346,8 @@ void DeviceClass::device_destroyer(const char *dev_name)
 //+----------------------------------------------------------------------------
 //
 // method :		DeviceClass::is_command_allowed
-// 
-// description :	Method to check if a command is allowed even if the device 
+//
+// description :	Method to check if a command is allowed even if the device
 //					is locked by another client. It follows the definition
 //					of the Tango control access system to define what is an
 //					allowed command
@@ -1438,15 +1359,87 @@ void DeviceClass::device_destroyer(const char *dev_name)
 bool DeviceClass::is_command_allowed(const char *cmd)
 {
 	bool ret = true;
-	
+
 	string tmp_cmd(cmd);
 	transform(tmp_cmd.begin(),tmp_cmd.end(),tmp_cmd.begin(),::tolower);
 
 	vector<string>::iterator pos = find(allowed_cmds.begin(),allowed_cmds.end(),tmp_cmd);
 	if (pos == allowed_cmds.end())
 		ret = false;
-	
+
 	return ret;
 }
 
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DeviceClass::get_mcast_event()
+//
+// description : 	Get for all class devices and for all attributes multicast
+//					event parameters (if any)
+//
+// in :	dserv : Pointer to the DServer device
+//
+//-----------------------------------------------------------------------------
+
+void DeviceClass::get_mcast_event(DServer *dserv)
+{
+	cout4 << "Entering DeviceClass::get_mcast_event() method" << endl;
+	vector<string> m_cast;
+
+	for (unsigned int i = 0;i < device_list.size();++i)
+	{
+		vector<Attribute *> &att_list = device_list[i]->get_device_attr()->get_attribute_list();
+		for (unsigned int j = 0;j < att_list.size();++j)
+		{
+			dserv->mcast_event_for_att(device_list[i]->get_name_lower(),att_list[j]->get_name_lower(),m_cast);
+			if (m_cast.size() != 0)
+				att_list[j]->set_mcast_event(m_cast);
+		}
+	}
+}
+
+//+----------------------------------------------------------------------------
+//
+// method :		DeviceClass::get_cmd_by_name
+//
+// description :	Get a reference to the Command object
+//
+// in : 	cmd_name : The command name
+//
+//-----------------------------------------------------------------------------
+
+Command &DeviceClass::get_cmd_by_name(const string &cmd_name)
+{
+	vector<Command *>::iterator pos;
+
+#ifdef HAS_LAMBDA_FUNC
+    pos = find_if(command_list.begin(),command_list.end(),
+                    [&] (Command *cmd) -> bool
+                    {
+                        if (cmd_name.size() != cmd->get_lower_name().size())
+                            return false;
+                        string tmp_name(cmd_name);
+                        transform(tmp_name.begin(),tmp_name.end(),tmp_name.begin(),::tolower);
+                        return cmd->get_lower_name() == tmp_name;
+                    });
+#else
+	pos = find_if(command_list.begin(),command_list.end(),
+		      bind2nd(WantedCmd<Command *,const char *,bool>(),cmd_name.c_str()));
+#endif
+
+	if (pos == command_list.end())
+	{
+		cout3 << "DeviceClass::get_cmd_by_name throwing exception" << endl;
+		TangoSys_OMemStream o;
+
+		o << cmd_name << " command not found" << ends;
+		Except::throw_exception((const char *)"API_CommandNotFound",
+				      o.str(),
+				      (const char *)"DeviceClass::get_cmd_by_name");
+	}
+
+	return *(*pos);
+}
+
 } // End of Tango namespace
