@@ -1,10 +1,10 @@
-static const char *RcsId = "$Id: dserverlock.cpp 15556 2011-02-11 08:25:58Z taurel $\n$Name$";
+static const char *RcsId = "$Id: dserverlock.cpp 20437 2012-05-31 12:33:29Z taurel $\n$Name$";
 
 //+=============================================================================
 //
 // file :               dserverlock.cpp
 //
-// description :        C++ source for the DServer class and its commands. 
+// description :        C++ source for the DServer class and its commands.
 //			The class is derived from Device. It represents the
 //			CORBA servant object which will be accessed from the
 //			network. All commands which can be executed on a
@@ -14,7 +14,7 @@ static const char *RcsId = "$Id: dserverlock.cpp 15556 2011-02-11 08:25:58Z taur
 //
 // author(s) :          E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010,2011,2012
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -25,53 +25,16 @@ static const char *RcsId = "$Id: dserverlock.cpp 15556 2011-02-11 08:25:58Z taur
 // it under the terms of the GNU Lesser General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-// 
+//
 // Tango is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 15556 $
-//
-// $Log$
-// Revision 3.8  2010/09/09 13:45:22  taurel
-// - Add year 2010 in Copyright notice
-//
-// Revision 3.7  2009/03/30 15:03:44  taurel
-// - Fix last bugs before Tango 7 ??
-//
-// Revision 3.6  2009/03/18 12:18:43  taurel
-// - Fix warnings reported when compiled with the option -Wall
-//
-// Revision 3.5  2009/01/21 12:49:04  taurel
-// - Change CopyRights for 2009
-//
-// Revision 3.4  2008/10/06 15:01:09  taurel
-// - Changed the licensing info from GPL to LGPL
-//
-// Revision 3.3  2008/10/03 06:52:31  taurel
-// - Add some licensing info in each files
-//
-// Revision 3.2  2008/09/01 14:48:05  taurel
-// - Some more bugs in locking feature
-//
-// Revision 3.1  2008/05/20 12:44:11  taurel
-// - Commit after merge with release 7 branch
-//
-// Revision 1.1.2.4  2008/01/03 16:05:52  taurel
-// - Some changes in locking feature implementation
-//
-// Revision 1.1.2.3  2007/12/20 14:29:01  taurel
-// - Some more work on locking
-//
-// Revision 1.1.2.2  2007/12/19 15:54:47  taurel
-// - Still some work going on for the locking feature
-//
-// Revision 1.1.2.1  2007/11/22 12:33:11  taurel
-// - First part of the device locking implementation
+// $Revision: 20437 $
 //
 //-=============================================================================
 
@@ -88,7 +51,7 @@ namespace Tango
 //+----------------------------------------------------------------------------
 //
 // method : 		DServer::lock_device()
-// 
+//
 // description : 	command to lock device
 //
 // args: - in_data : Pointer to a structure with:
@@ -100,7 +63,7 @@ namespace Tango
 void DServer::lock_device(const Tango::DevVarLongStringArray *in_data)
 {
 	NoSyncModelTangoMonitor mon(this);
-	
+
 	string dev_name(in_data->svalue[0]);
 	int lock_validity = in_data->lvalue[0];
 
@@ -117,16 +80,16 @@ void DServer::lock_device(const Tango::DevVarLongStringArray *in_data)
 					       	(const char*)"Cannot retrieve client identification",
 					        (const char*)"DServer::lock_device");
 	}
-	
+
 	cout4 << "Client identification = " << *cl << endl;
-	
+
 	if (cl->client_ident == false)
 	{
 		Except::throw_exception((const char*)"API_ClientTooOld",
 					       	(const char*)"Your client cannot lock devices. You are using a too old Tango release. Please, update to tango V7 or more",
 					        (const char*)"DServer::lock_device");
-	}	
-	
+	}
+
 //
 // Transform device name to lower case
 //
@@ -135,10 +98,10 @@ void DServer::lock_device(const Tango::DevVarLongStringArray *in_data)
 
 	string local_dev_name(get_name());
 	transform(local_dev_name.begin(),local_dev_name.end(),local_dev_name.begin(),::tolower);
-	
+
 	string d_name_lower(dev_name);
 	transform(d_name_lower.begin(),d_name_lower.end(),d_name_lower.begin(),::tolower);
-		
+
 //
 // Refuse to lock the admin device
 //
@@ -149,14 +112,14 @@ void DServer::lock_device(const Tango::DevVarLongStringArray *in_data)
 								(const char *)"Impossible to lock device server administration device",
 								(const char *)"DServer::lock_device");
 	}
-				
+
 //
 // Get device ptr
 //
 
 	DeviceImpl *the_dev;
 	the_dev = tg->get_device_by_name(dev_name);
-		
+
 //
 // Refuse to lock database device
 //
@@ -174,14 +137,14 @@ void DServer::lock_device(const Tango::DevVarLongStringArray *in_data)
 //
 
 	the_dev->lock(cl,lock_validity);
-	
+
 }
 
 
 //+----------------------------------------------------------------------------
 //
 // method : 		DServer::un_lock_device()
-// 
+//
 // description : 	command to un lock a device
 //
 // args: - dev_name : The device name
@@ -192,8 +155,8 @@ Tango::DevLong DServer::un_lock_device(const Tango::DevVarLongStringArray *in_da
 {
 	NoSyncModelTangoMonitor mon(this);
 
-	cout4 << "In un_lock_device command for device " << in_data->svalue[0] << endl;	
-	
+	cout4 << "In un_lock_device command for device " << in_data->svalue[0] << endl;
+
 //
 // Get client identification
 //
@@ -205,27 +168,27 @@ Tango::DevLong DServer::un_lock_device(const Tango::DevVarLongStringArray *in_da
 					       	(const char*)"Cannot retrieve client identification",
 					        (const char*)"DServer::un_lock_device");
 	}
-	
+
 	cout4 << "Client identification = " << *cl << endl;
-	
+
 	if ((cl->client_ident == false) && (in_data->lvalue[0] == 0))
 	{
 		Except::throw_exception((const char*)"API_ClientTooOld",
 					       	(const char*)"Your client cannot un-lock devices. You are using a too old Tango release. Please, update to tango V7 or more",
 					        (const char*)"DServer::un_lock_device");
-	}	
+	}
 
 //
 // Get the device and unlock it
 //
 
-	DevLong ctr;
+	DevLong ctr = 0;
 	Tango::Util *tg = Tango::Util::instance();
-	
+
 	for (unsigned int loop = 0;loop < in_data->svalue.length();++loop)
 	{
 		string d_name(in_data->svalue[loop]);
-	
+
 		if (d_name == get_name())
 			ctr = ext->lock_ctr;
 		else
@@ -233,18 +196,18 @@ Tango::DevLong DServer::un_lock_device(const Tango::DevVarLongStringArray *in_da
 			DeviceImpl *the_dev = tg->get_device_by_name(d_name);
 			ctr = the_dev->unlock((bool)in_data->lvalue[0]);
 		}
-		
+
 		if (loop > 0)
 			ctr = 0;
 	}
-	
+
 	return ctr;
 }
 
 //+----------------------------------------------------------------------------
 //
 // method : 		DServer::re_lock_devices()
-// 
+//
 // description : 	command to re-lock devices
 //
 // args: - dev_name_list : Name of the device(s) to be re-locked
@@ -273,16 +236,16 @@ void DServer::re_lock_devices(const Tango::DevVarStringArray *dev_name_list)
 					       	(const char*)"Cannot retrieve client identification",
 					        (const char*)"DServer::re_lock_devices");
 	}
-	
+
 	cout4 << "Client identification = " << *cl << endl;
-	
+
 	if (cl->client_ident == false)
 	{
 		Except::throw_exception((const char*)"API_ClientTooOld",
 					       	(const char*)"Your client cannot re_lock devices. You are using a too old Tango release. Please, update to tango V7 or more",
 					        (const char*)"DServer::re_lock_devices");
-	}	
-	
+	}
+
 //
 // ReLock the devices
 // If we have an error in this loop, memorize it and throw the exception at the
@@ -293,16 +256,16 @@ void DServer::re_lock_devices(const Tango::DevVarStringArray *dev_name_list)
 
 	DevErrorList errors;
 	long nb_error = 0;
-	
+
 	for (loop = 0;loop < nb_dev;loop++)
 	{
 		string d_name((*dev_name_list)[loop]);
-				
+
 //
 // Get device ptr
 //
 
-		DeviceImpl *the_dev;
+		DeviceImpl *the_dev = NULL;
 		try
 		{
 			the_dev = tg->get_device_by_name(d_name);
@@ -316,7 +279,7 @@ void DServer::re_lock_devices(const Tango::DevVarStringArray *dev_name_list)
 			errors[nb_error].severity = e.errors[0].severity;
 			nb_error++;
 		}
-		
+
 //
 // ReLock the device
 //
@@ -339,7 +302,7 @@ void DServer::re_lock_devices(const Tango::DevVarStringArray *dev_name_list)
 //
 // Throw the exception if we had one during the loop
 //
-	
+
 	if (nb_error != 0)
 	{
 		throw Tango::DevFailed(errors);
@@ -349,7 +312,7 @@ void DServer::re_lock_devices(const Tango::DevVarStringArray *dev_name_list)
 //+----------------------------------------------------------------------------
 //
 // method : 		DServer::dev_lock_status()
-// 
+//
 // description : 	command to get device lock status
 //
 // args: - dev_name : The device name
@@ -360,8 +323,8 @@ Tango::DevVarLongStringArray *DServer::dev_lock_status(Tango::ConstDevString dev
 {
 	NoSyncModelTangoMonitor mon(this);
 
-	cout4 << "In dev_lock_status command for device " << dev_name << endl;	
-	
+	cout4 << "In dev_lock_status command for device " << dev_name << endl;
+
 //
 // Get the device and get its lock status
 //
@@ -370,7 +333,7 @@ Tango::DevVarLongStringArray *DServer::dev_lock_status(Tango::ConstDevString dev
 
 	Tango::Util *tg = Tango::Util::instance();
 	DeviceImpl *the_dev = tg->get_device_by_name(d_name);
-	return the_dev->lock_status();	
+	return the_dev->lock_status();
 }
 
 
