@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: tango_admin.cpp 16195 2011-03-22 15:43:53Z taurel $";
+static const char *RcsId = "$Id: tango_admin.cpp 20288 2012-05-23 11:01:15Z taurel $";
 
 //+============================================================================
 //
@@ -41,7 +41,7 @@ static const char *RcsId = "$Id: tango_admin.cpp 16195 2011-03-22 15:43:53Z taur
 // You should have received a copy of the GNU General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 16195 $
+// $Revision: 20288 $
 //
 //-============================================================================
 
@@ -568,11 +568,30 @@ int ping_database(int nb_sec)
 	else
 		nb_loop = nb_sec << 1;
 
+
+	struct timespec ts;
+	ts.tv_sec = 0;
+	ts.tv_nsec = 500000000;
+
+//
+// First sleep for 1 sec before trying to access the db
+// This was needed when ported to Natty (Ubuntu 11.04) in the
+// tango-db startup script. Db process did not start if tango 
+// admin starts pinging db device too early !!
+//
+
+	if (nb_loop != 1)
+	{
+		ts.tv_sec = 1;
+		ts.tv_nsec = 0;
+
+		nanosleep(&ts,NULL);
+	}
+
 //
 // re-try the call every 500 mS
 //
 
-	struct timespec ts;
 	ts.tv_sec = 0;
 	ts.tv_nsec = 500000000;
 
@@ -866,7 +885,7 @@ int delete_server(char *d_name,bool with_res)
 // Delete all device properties
 //
 
-				if (prop_list.size() != 0)
+				if (prop_list.empty() == false)
 				{
 					Tango::DbData dbd;
 					
@@ -1262,7 +1281,7 @@ int tac_enabled(void)
 		vector<string> service_list;
 		db_datum >> service_list;
 
-		if (service_list.size() == 0)
+		if (service_list.empty() == true)
 			ret = 0;
 	}
 	catch (Tango::DevFailed &e)
